/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.Breadcrumbs");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.string.BidiComplex");
dojo.require("jazz.util.ResizeNotifier");

(function(){

var BidiComplex = dojox.string.BidiComplex;

/**
 * Constructs a new Breadcrumbs
 * @name jazz.ui.Breadcrumbs
 * @class <p>This is a simple widget which manages the dom structure for a list
 *  of breadcrumb segments. It provides API to set the list of breadcrumbs
 *  as well as to read and update it again later.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 * @since 0.7
 */ 
dojo.declare("jazz.ui.Breadcrumbs",	[dijit._Widget, dijit._Templated], /** @lends jazz.ui.Breadcrumbs.prototype  */{
	/**
	 * An array of objects representing the segments in the breadcrumbs. It can
	 * be passed in the constructor to the widget. It can also be read and updated
	 * at any time during the lifecycle of the widget. The refresh function must be
	 * called after updating the segments to have the changes reflected in the dom.
	 * Each segment contains the following attributes:
	 * <ol>
	 * 	<li>String label (required): A string attribute which will be added to the dom as
	 *	text (not html)</li>
	 *  <li>String icon: Gives a url for the icon to be
	 *  used for the segment. Must be a 16x16 icon.</li>
	 *  <li>String tooltip: Used for the 'title' attribute
	 *  of the domNode for the segment. This will result in a tooltip
	 *  containing the specified text when the user hovers the node.</li>
	 *  <li>Function onClick: A function to fire on click of the segment. The actual
	 *  click event for the link will be provided to the handler.</li>
	 *  <li>String href: The href to use for the link. If supplied in tandem with
	 *  an 'onClick' handler, the href will still be placed on the link,
	 *  but the default navigation will be aborted in favour of the click
	 *  handler. If neither 'href' nor 'onClick' is defined, the segment
	 *  will be rendered as plain text.</li>
	 * </ol>
	 */
	segments: [],
	
	/**
	 * Refreshes the content of the widget. Should be called after any change to the segments attribute. This will
	 * cause the widget to update its dom based on the current segments.
	 */
	refresh: function() {
		// summary:
		//		Refreshes the content of the widget.
		// description:
		//		Should be called after any change to the segments attribute. This will
		//		cause the widget to update its dom based on the current segments.
		this._segmentNodes = [];
		this._liveSegments = dojo.clone(this.segments);
		while (this.domNode.firstChild)
			this.domNode.removeChild(this.domNode.firstChild);
		for (var i = 0; i < this._liveSegments.length; i++) {
			this._createCrumb(i);
		}
		setTimeout(dojo.hitch(this, this._truncate), 0);
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
	
	/** @private */
	templateString: "<div class='jazz-ui-Breadcrumbs j-action-secondary'></div>",
	
	/** @private */
	postCreate: function() {
		this.refresh();
		this._resizeHandle = jazz.util.ResizeNotifier.register({context: this, callback: this._truncate});
	},
	
	_createCrumb: function(i){
		var obj = this._liveSegments[i];
		var span = dojo.create("span", null, this.domNode), label;
		if (obj.icon)
			dojo.create("img", {src: obj.icon, title: obj.tooltip ? obj.tooltip : ""}, span);
		if (obj.href || obj.onClick) {
			label = dojo.create("a", {href: obj.href, className: "j-action-secondary", title: obj.tooltip ? obj.tooltip : ""}, span);
			if (obj.onClick) {
				this.connect(label,"onclick", function(event) {
					event.preventDefault();
					obj.onClick(event);
				});
			}
		} else {
			label = dojo.create("span", {className: "plain-text", title: obj.tooltip ? obj.tooltip : ""}, span);
		}
		label.appendChild(document.createTextNode(BidiComplex.createDisplayString(obj.label, "EMAIL")));
		span.appendChild(document.createTextNode(" > "));
		obj.domNode = span;
	},
	
	_truncate: function() {
		if (!this.domNode) return;
		var n, i, l = this._liveSegments.length;
		if (this._lastTruncate) {
			if (this._lastTruncate.width === this.domNode.clientWidth) {
				return;
			} else if (this._lastTruncate.width > this.domNode.clientWidth) {
				n = this._lastTruncate.n;
				i = this._lastTruncate.i;
			} else {
				this._showAllSegments(l);
			}
		}
		if (typeof n === "undefined") {
			n = 0;
			i = Math.floor((l - 1) / 2);
		}
		while (this.domNode.clientHeight > 16 && n < l - 2) {
			if (!this._ellipsis) {
				this._ellipsis = document.createTextNode("... > ");
				dojo.place(this._ellipsis, this._liveSegments[i].domNode, "after");
			}
			i += n % 2 === l % 2 ? -n : n;
			dojo.addClass(this._liveSegments[i].domNode, "hidden");
			n++;
		}
		this._lastTruncate = {
			width: this.domNode.clientWidth,
			n: n,
			i: i
		}
	},
	
	_showAllSegments: function(l) {
		if (this._ellipsis && this._ellipsis.parentNode) {
			this._ellipsis.parentNode.removeChild(this._ellipsis);
			delete this._ellipsis;
		}
		for (var i = 0; i < l; i++) {
			dojo.removeClass(this._liveSegments[i].domNode, "hidden");
		}
	},
	
	destroy: function() {
		jazz.util.ResizeNotifier.unregister(this._resizeHandle);
		this.inherited(arguments);
	}
});

})();
