/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.Dialog");

dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.underlay");
dojo.require("dojo.fx");

(function() {

//
// Aliases
//
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var underlay = jazz.ui.underlay;
var openDialogs = [];
var BASE_ZINDEX = 999;
var MIN_SIZE = 80;

/**
 * Constructs a new Dialog. All fields may be initialized in the widget constructor, but the provided
 * setters must be used to change them for existing instances.
 * @name jazz.ui.Dialog
 * @class <p>A Dialog widget for web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends jazz.ui.StyledBox
 * @since 0.7
 */ 
dojo.declare("jazz.ui.Dialog", jazz.ui.StyledBox, /** @lends jazz.ui.Dialog.prototype */{
	
	/**
	 * DOM node for contents of the dialog.
	 * @type DomNode
	 * @required
	 */
	contentNode: null,
	
	/**
	 * DOM node that should get focus when the dialog opens
	 * @type DomNode
	 */
	focusNode: null,
	
	/**
	 * Specify the width (eg. "400px", "40%"). This is the outer width of the StyledBox (including trim).
	 * Zero and empty string result in "width: auto" (not recommended if {@link jazz.ui.Dialog.center} is true 
	 * due to inconsistent behavior in IE7 of position: absolute nodes). 
	 * @type String
	 * @default "40%"
	 */
	width: "40%",
	
	/**
	 * Specify the height. This is the inner height of the content area of the StyledBox (not including trim).
	 * Zero and empty string result in "height: auto".
	 * @type String
	 * @default ""
	 */
	height: "",
	
	/**
	 * Specify the title of the box. 
	 * @type String
	 * @default ""
	 */
	primaryTitle: "",
	
	/**
	 * Specify the secondary title of the box. Appears beside primary title, in a lighter color 
	 * @type String
	 * @default ""
	 */
	secondaryTitle: "",
	
	/**
	 * Specify the trim style of the StyledBox. Valid values are available under {@link jazz.ui.StyledBox.TrimStyles}.
	 * @type String
	 * @default BLUE_ALPHA_SHADOW
	 */
	trim: TrimStyles.BLUE_ALPHA_SHADOW,
	
	/**
	 * Specify whether the StyledBox can be closed by the user. 
	 * @type Boolean
	 * @default true
	 */
	closable: true,
	
	/**
	 * Specify whether the header color should get darker on hover. 
	 * @type Boolean
	 * @default false
	 */
	headerHoverState: false,
	
	/**
	 * Specify whether the header should default to hover state color. This is a slightly darker rendering for the header color.
	 * If true, {@link jazz.ui.StyledBox#headerHoverState} is ignored.
	 * @type Boolean
	 * @default true
	 */
	headerDefaultToHoverState: true,
	
	/**
	 * Specify whether the Dialog should have a solid background color. 
	 * @type Boolean
	 * @default false
	 */
	showBackground: false,
	
	/**
	 * Specify whether the Dialog should be modal (i.e. should the user be able to interact with stuff behind the dialog?) 
	 * @type Boolean
	 * @default true
	 */
	modal: true,
	
	/**
	 * Specify whether the Dialog can be moved with drag and drop. 
	 * @type Boolean
	 * @default true
	 */
	moveable: true,	
	
	/**
	 * Specify whether the Dialog should be positioned in the center of the window.  If false, the
	 * consumer is responsible for appending the dialog to the DOM. 
	 * @type Boolean
	 * @default true
	 */
	center: true,
	
	/**
	 * Specify whether the escape key should close the dialog. 
	 * @type Boolean
	 * @default true
	 */
	closeOnEsc: true,

	/**
	 * Specify whether the clicking outside the dialog should close the dialog. 
	 * @type Boolean
	 * @default false
	 */
	closeOnBlur: false,
	
	/**
	 * Specify the path to a 16x16 icon to appear in the header of the box to the left of the title. 
	 * @type String
	 * @default ""
	 */
	iconPath: "",
	
	/**
	 * Specify whether the dialog should destroy the widget passed in 'contentNode' on close (if it's a widget) 
	 * @type Boolean
	 * @default false
	 */
	destroyContents: false,
	
	/**
	 * Customize the arguments that are passed to jazz.ui.underlay when the dialog opens. Only used if {@link jazz.ui.Dialog#modal} is true. 
	 * @type Object
	 * @default null
	 */
	underlayArgs: null,
	
	/**
	 * A function to be called when the user asks to close the dialog. This function can return boolean false to veto the close. 
	 * @type Object
	 * @default null
	 */	
	onClose: null,
	
	/**
	 * A function to be called when the dialog is opened. 
	 * @type Object
	 * @default null
	 */
	onOpen: null,
	
	/**
	 * Call to update the position of the dialog. Useful for re-centering the dialog after adding content to the dialog. 
	 * @param {event} event The event object, if this is attached to a DOM event.
	 */
	updatePosition: function(event) {
		if (this._isClosed) return;
		if (this.center) {
			var top, left, noAnimX, noAnimY, 
				de = window.document.documentElement,
				height = de.clientHeight,
				width = de.clientWidth,
				scrollTop = de.scrollTop ? de.scrollTop : window.document.body.scrollTop,
				scrollLeft = de.scrollLeft ? de.scrollLeft : window.document.body.scrollLeft,
				scrollHeight = de.scrollHeight ? de.scrollHeight : window.document.body.scrollHeight,
				coords = dojo.marginBox(this.domNode);
				
			if (this.moveable && event && this._customPosition) {
				if (event.type == "scroll") {
					top = this._position.t;
					left = this._position.l;
				} else if (event.type == "resize") {
					if (this._position.l + coords.w > width) {
						left = width - coords.w;
					} else {
						left = this._position.l;
						noAnimX = true;
					}
					if (this._position.t + coords.h > height) {
						top = height - coords.h;
					} else {
						top = this._position.t;
						noAnimY = true;
					}
				}
			} else {
				top = height / 2 - coords.h / 2;
				left = width / 2 - coords.w / 2;
			}
			
			if (top < 2 || top + coords.h > height) {
				if (this._lastTop == null) {
					if (top < 2) top = 2;
				} else if (event && event.type == "scroll" && this._lastTop === scrollTop) {
					top = coords.t - scrollTop;
				} else if (this._lastTop >= scrollTop) {
					top = Math.min(coords.t - scrollTop, Math.max(2, scrollTop + height - coords.h));
				} else if (this._lastTop < scrollTop) {
					top = Math.max(coords.t - scrollTop, height - coords.h);
				}
			}
			if (left < 2 || left + coords.w > width) {
				if (this._lastLeft == null) {
					if (left < 2) left = 2;
				} else if (event && event.type == "scroll" && this._lastLeft === scrollLeft) {
					left = coords.l - scrollLeft;
				} else if (this._lastLeft >= scrollLeft) {
					left = Math.min(2, coords.l - scrollLeft);
				} else if (this._lastLeft < scrollLeft) {
					left = Math.max(coords.l - scrollLeft, width - coords.w);
				}
			}
			if (this.anim && this.anim.status() == "playing") {
				this.anim.stop();
				this._position = {l: coords.l, t: coords.t};
			}
			if (this._position && !(noAnimX && noAnimY)) {
				if (coords.t + coords.h < scrollTop) {
					coords.t = scrollTop - coords.h;
				} else if (coords.t > scrollTop + height) {
					coords.t = scrollTop + height;
				}
				this.anim = dojo.animateProperty({node: this.domNode, duration: 150, 
					properties: {
						top: {start: coords.t, end: top + scrollTop, unit: "px"},
						left: {start: coords.l, end: left + scrollLeft, unit: "px"}
					}
				});
				this.connect(this.anim, "onEnd", function() {
					jazz.ui.underlay.update();
				});
				this.anim.play();
			} else {
				dojo.marginBox(this.domNode, {l: left + scrollLeft, t: top + scrollTop});
			}
			this._lastTop = scrollTop; 
			this._lastLeft = scrollLeft;
			this._position = {l: left, t: top};
		}
	},
	
	/**
	 * Close the dialog.
	 * @param {event} event The event object, if this is attached to a DOM event.
	 */
	close: function(event) {
		if (this._isClosed || this.inherited(arguments) === false) return;
		this._isClosed = true;
		if (this._blurEvent) this.disconnect(this._blurEvent);
		openDialogs.splice(dojo.indexOf(openDialogs,this),1);
		var length = openDialogs.length;
		if (length == 0 || !openDialogs[length-1].modal) {
			underlay.hide() 
		}
		updateZIndexes();
		if (this._lastFocus && this._lastFocus.node && dojo.isDescendant(this._lastFocus.node, document.body)) {
			dijit.focus(this._lastFocus);
		}
	},
	
	// Internals

	postCreate: function() {
		this._lastFocus = dijit.getFocus(this);
		
		// Hitch the user's onOpen callback to the input of _onOpen,
		// then nullify onOpen so that this.inherited doesn't call it 
		// before dialog is appended to the DOM.
		this._onOpen = dojo.hitch(this,this._onOpen,this.onOpen);
		this.onOpen = null;
		
		this.inherited(arguments);
		
		openDialogs.push(this);
		updateZIndexes();
		dijit.setWaiRole(this.domNode, "dialog");		

		if (this.modal) {
			var args = {zIndex: BASE_ZINDEX, stopEvents: !this.closeOnBlur, animate: false};
			dojo.mixin(args, this.underlayArgs);
			underlay.show(args);
			var focusFn = function(e) {
				var node = this._headerPrimaryLink||this._headerText;
				node.focus();
			};
			this.connect(dojo.create("span",{ tabIndex: 0 },this.domNode),"onfocus",focusFn);
			this.connect(dojo.create("span",{ tabIndex: 0 },this.domNode,"first"),"onfocus",focusFn);
			if (this.closeOnBlur) {
				this._blurEvent = this.connect(document.body, "onclick", this._closeOnBlur);
			}
		}

		if (this.center){
			this._registerUpdateEvents();
			dojo.addClass(this.domNode, "jazz-ui-Dialog-absolute");
			document.body.appendChild(this.domNode);
			this.updatePosition();
		}
		
		if (this.moveable) {
			this.connect(this._header, "ondragstart", this.dragStopEvent);
			this.connect(this._header, "onselectstart", this.dragStopEvent);
			this.connect(document.body,"ondragstart", this.dragStopEvent);
			this.connect(document.body,"onselectstart", this.dragStopEvent);
			this.connect(this._headerButtons, "onmousedown", this._cancelBubble);
			this._clickEvent = this.connect(this._header, "onmousedown", "_onMouseDown");
			this._isDragging = false;
			this._header.style.cursor = "move";
		}
		if (this.closeOnEsc) {
			this.connect(document.documentElement, "onkeypress", this._checkForEsc);
		}
		
		//Stop bleed-through in high contrast mode and IE6 form elements.
		this._backgroundIframe = new dijit.BackgroundIframe(this.domNode);
		this._onOpen(/* onOpen - hitched to input (see above) */);
	},
	_closeOnBlur: function(event) {
		if (event.target == dojo.byId("jazz-ui-underlay")) {
			this.close();	
		}
	},
	_onOpen: function(onOpen) {
		var focus = this.focusNode || this._headerText;
		if (focus.nodeName == "INPUT") {
			dijit.selectInputText(focus);
		} else {
			dijit.focus(focus);
		}
		if (onOpen && typeof onOpen == "function") {
			onOpen();
		}
	},
	_checkForEsc: function (event) {
		if (event.keyCode == 27) {
			this.close();
		}
	},
	_cancelBubble: function(event) {
		event.cancelBubble = true;
	},
	dragStopEvent: function(event) {
		if (event && (this._isDragging || this._initialMouseCoords)) {
			dojo.stopEvent(event);
		}
	},
	_onMouseDown: function(event) {
		this.disconnect(this._clickEvent);
		this._unregisterUpdateEvents();
		this._clickEvent = this.connect(document.body, "onmouseup", this._onMouseUp);
		
		var coords = dojo.coords(this.domNode, true);
		this._initialMouseCoords = {
			x: event.pageX,
			y: event.pageY,
			offsetX: event.pageX - coords.x,
			offsetY: event.pageY - coords.y
		}
		dojo.addClass(document.body,"jazz-ui-Dialog-moveCursor");
		this._moveEvent = this.connect(dojo.doc, "onmousemove", this._onMouseMoveInitial);
		this.dragStopEvent(event);
	},
	_onMouseMoveInitial: function(e) {
		var difX = Math.abs(e.pageX - this._initialMouseCoords.x);
		var difY = Math.abs(e.pageY - this._initialMouseCoords.y);
		if (!this._isDragging && (difX >= 3 || difY >= 3)) {
			this._isDragging = true;
			this.disconnect(this._moveEvent);
			this._moveEvent = this.connect(dojo.doc, "onmousemove", this._onMouseMove);
			this._onMouseMove(e);
		} else {
			this.dragStopEvent(e);
		}
	},
	_onMouseMove: function(e) {
		this.dragStopEvent(e);
		var x = e.pageX - this._initialMouseCoords.offsetX;
		var y = e.pageY - this._initialMouseCoords.offsetY;
		this._customPosition = {l: x, t: y};
		this.domNode.style.top = y + "px";
		this.domNode.style.left = x + "px";
	},
	_onMouseUp: function(event) {
		this._isDragging = false;
		dojo.removeClass(document.body,"jazz-ui-Dialog-moveCursor");
		this.disconnect(this._clickEvent);
		this.disconnect(this._moveEvent);
		this._clickEvent = this.connect(this._header, "onmousedown", this._onMouseDown);
		this._registerUpdateEvents();
		
		var de = document.documentElement;
		var scrollTop = de.scrollTop ? de.scrollTop : document.body.scrollTop;
		var scrollLeft = de.scrollLeft ? de.scrollLeft : document.body.scrollLeft;
		if (this._customPosition) {
			this._position = {l: this._customPosition.l - scrollLeft, t: this._customPosition.t - scrollTop};
		}
		this._initialMouseCoords = null;
	},
	_registerUpdateEvents: function(){
		if (!this._onResizeEvent) this._onResizeEvent = this.connect(window, "onresize", this.updatePosition);
		if (!this._onScrollEvent) this._onScrollEvent = this.connect(window, "onscroll", this.updatePosition);
	},
	_unregisterUpdateEvents: function() {
		this.disconnect(this._onResizeEvent);
		this._onResizeEvent = null;
		this.disconnect(this._onScrollEvent);
		this._onScrollEvent = null;	
	}
});

function updateZIndexes() {
	for (var i = 0; i < openDialogs.length; i++) {
		openDialogs[i].domNode.style.zIndex = BASE_ZINDEX - openDialogs.length + i + 1; 
	}
}
})();