/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.FilterBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "FilterBoxMessages");

(function() {

var messages = dojo.i18n.getLocalization("jazz.ui", "FilterBoxMessages");

/**
 * Constructs a new FilterBox. All fields may be initialized in the widget constructor, but unless indicated otherwise, the provided
 * setters must be used to change them for existing instances.
 * @name jazz.ui.FilterBox
 * @class <p>This is a simple input field with some additional logic to support use as a
 * 	filter input box. Includes grey text to guide the user when empty, a button
 * 	to clear the content when populated and an onChange handler to respond to user
 * 	input.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 * @since 0.7
 */ 
dojo.declare("jazz.ui.FilterBox", [dijit._Widget, dijit._Templated], /** @lends jazz.ui.FilterBox.prototype */ {
	
	/**
	 * The text value of the box.
	 * @field 
	 * @type String
	 */
	value: "",
	/**
	 * A function that will be called any time the content of the input box changes.
	 * This function can be overriden at any time and the new one will start being used
	 * instantly. The value will always be provided in lower case.
	 * @field 
	 * @type Function
	 * @param {String} value The new value of the FilterBox
	 */
	onChange: null,
	/**
	 * A function that will be called when the enter key is pressed with focus
	 * on the input box.
	 * @field
	 * @type Function
	 */
	onEnter: null,
	/**
	 * Indicates if the FilterBox is enabled or not.
	 * @field 
	 * @type Boolean
	 */
	enabled: true,
	/**
	 * The grey text that should appear when the filter box is empty. By default it says
	 * "Type to filter".
	 * @field 
	 * @type String
	 */
	defaultText: messages.filter,
	/**
	 * The amonut of time in milliseconds to wait after input before firing
	 * onChange event.
	 * @field
	 * @type Integer
	 */
	dwellTime: 0,
	
	/**
	 * Can be called to enable or disable the box.
	 * @param {Boolean} enabled The new state of the box.
	 */
	setEnabled: function(/* boolean */ enabled) {
		this._box.disabled = enabled ? false : true;
		this.enabled = enabled;
	},
	
	/**
	 * Can be called to programatically update the value of the box.
	 * @param {String} value The new value.
	 */
	setValue: function(value) {
		if (!value) {
			this.clear();
			return;
		}
		dojo.removeClass(this._box, "helpMessage");
		dojo.removeClass(this._clearLink, "hidden");
		this._box.value = value;
		this.value = value;
	},
	
	/**
	 * Resets the FilterBox to show the defaultText as if the user cleared it themselves.
	 */
	clear: function() {
		dojo.addClass(this._box, "helpMessage");
		dojo.addClass(this._clearLink, "hidden");
		this._box.value = this.defaultText;
		this._handleChange("");
	},
	
	/**
	 * Focuses the FilterBox.
	 */
	focus: function() {
		this._box.focus();
	},
	
	/**
	 * Blurs the FilterBox.
	 */
	blur: function() {
		this._box.blur();
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
	
	// TODO: this is a hack to allow AppearancePanel to customize the tooltip (work item 124839)
	_clearTooltip: messages.clear,
	/** @private */
	templatePath: dojo.moduleUrl("jazz.ui", "templates/FilterBox.html"),
	
	/** @private */
	constructor: function() {
		this.messages = messages;
	},
	
	/** @private */
	postCreate: function() {
		if (this.value)
			this.setValue(this.value);
		if (!this.enabled)
			this._box.disabled = true;
		this.connect(this._box, "onkeypress", dojo.hitch(this, this._handleKeyPress));
	},
	
	_focus: function(event) {
		if (dojo.hasClass(this._box, "helpMessage")) {
			this._box.value = "";
			dojo.removeClass(this._box, "helpMessage");
			if (this.value !== "") {
				dojo.removeClass(this._clearLink, "hidden");
			}
		}
	},
	
	_blur: function(event) {
		if (this._box.value.length == 0) {
			this.clear();
		}
	},
	
	_keyup: function(event) {
		this._handleChange(this._box.value);
	},
	
	_handleChange: function(newValue) {
		if (newValue === "") {
			dojo.addClass(this._clearLink, "hidden");
		} else {
			dojo.removeClass(this._clearLink, "hidden");
		}
		if (this.onChange) {
			if (newValue !== this.value && this.dwellTime === 0) {
				this._fireOnChange(newValue);
			} else {
				this._clearOnChangeTimeout();
				this._onChangeTimeout = setTimeout(
					dojo.hitch(this, this._fireOnChange, newValue),
					this.dwellTime);
			}
		}
	},

	_clearOnChangeTimeout: function () {
		if (this._onChangeTimeout) {
			clearTimeout(this._onChangeTimeout);
			delete this._onChangeTimeout;
		}
	},

	_fireOnChange: function(newValue) {
		this._clearOnChangeTimeout();
		var oldValue = this.value;
		// TODO: _maintainCase is a hack for AppearancePanel to allow the widget to be used for widget titles (work item 117824)
		this.value = this._maintainCase ? newValue : newValue.toLowerCase();
		if (this.value !== oldValue) {
			this.onChange(this.value);
		}
	},
	
	_clear: function(event) {
		if (event)
			event.preventDefault();
		this.clear();
	},

	_handleKeyPress: function(event) {
		if (!this.onEnter) return;
		if (event.keyCode === dojo.keys.ENTER) {
			this.onEnter();
		}
	}
});
	
})();
