/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.MegaMenu");

dojo.require("jazz.ui.Menu");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.ui.internal.NodeMenuItem");
dojo.require("jazz.ui.internal.ImgMenuItem");
dojo.require("jazz.ui.FilterBox");
dojo.require("dijit.MenuSeparator");
dojo.require("jazz.ui.PaginationControl");

(function() {

var COLUMN_WIDTH = 277;

dojo.declare("jazz.ui.MegaMenu", [dijit._Widget, dijit._Templated, dijit._Container, dijit._Contained], {

	/**
	 * Determines whether a filter box should be shown at the top of the component menu.
	 * It searches the text values of all menu items and hides the ones that don't match.
	 */
	showFilter: false,
		
	/**
	 * Adds a menu item to the menu by args.  Supports extra stuff that vanilla dijit.MenuItems do not (i.e. href and icon).
	 * @param {Object} args - Accepted values: 	icon (path to icon)
	 * 											iconClass (css class for iconNode) 
	 * 											href (href to visit on click)
	 * 											onClick
	 * 											label
	 * 											id
	 * 											wrapHint (See jazz.ui.MegaMenu.WrapHints for constants - NO_WRAP, AUTO_WRAP, FORCE_WRAP)
	 * 											title
	 * @param {Number} [insertIndex]
	 */
	addItemByArgs: function(args, insertIndex) {
		return this.add(new jazz.ui.internal.ImgMenuItem(args), {
			insertIndex: insertIndex,
			wrapHint: args.wrapHint
		});
	},
	/**
	 * Adds a title menu item to the menu.
	 * 
	 * @param {String} title
	 * @param {Number|object} [insertIndex|args] args: insertIndex, wrapHint
	 */
	addTitle: function(title, args) {
		if (typeof args == "number") args = { insertIndex: args };
		var item = new jazz.ui.internal.NodeMenuItem({label: title, isTitle: true, focusable: args && args.focusable});
		this.addChild(item, args);
		return item;
	},
	/**
	 * Adds menu items, strings or domNodes to the component menu. Items must be compatible with dijit.Menu.
	 * @param {dijit.MenuItem|string|domNode} item
	 * @param {Number|object} [insertIndex|args]  args: insertIndex, wrapHint, iconIndent
	 */
	add: function(item, args) {
		if (args == null) {
			args = {};
		} else {
			var insertIndex = args;
			if (typeof insertIndex == "number" || typeof insertIndex == "string") {
				args = {};
				args.insertIndex = insertIndex;
			}
		}
		if (item && item.nodeType || dojo.isString(item)) {
			//domNode or string
			var val = item;
			if (args.iconIndent === false) {
				item = new jazz.ui.internal.NodeMenuItem();
			} else {
				item = new jazz.ui.internal.NodeMenuItem({className: "menu-text"});	
			}
			if (!args.wrapHint) args.wrapHint = WrapHints.NO_WRAP;
			if (val.nodeType) {
				dojo.place(val, item.containerNode, "only");
			} else {
				item.attr("label", val)
			}
		} else if (item && item instanceof dijit._Widget) {
			args.wrapHint = item.wrapHint;
		} else {
			console.error("[ComponentMenu] Error adding item.  Invalid type.");
		}
		if (this._getIndexOfChild(item) >= 0) {
			this.removeChild(item);
		}
		this.addChild(item, args);
		return item;
	},
	/**
	 * Adds a separator at the specified index or at end of menu.
	 * @param {Object} [insertIndex]
	 */
	addSeparator: function(insertIndex){
		return this.add(new dijit.MenuSeparator(), insertIndex);
	},
	/**
	 * 
	 * @param {Object} menuItem
	 * @param {number|Object} insertIndex or args:
	 * 		- wrapHint (jazz.app.MegaMenu.WrapHints constant)
	 * 		- insertIndex (number)
	 */
	addChild: function(menuItem, args) {
		if (args == null) args = {};
		var wh = args.wrapHint || WrapHints.AUTO_WRAP;
		menuItem.wrapHint = wh;
		if (typeof args == "number" || typeof args == "string"){
			var insertIndex = args;
		} else if (typeof args.insertIndex == "number") {
			insertIndex = args.insertIndex;
		}
		if (insertIndex === null) insertIndex = this._items.length;
		var addMenu;
		if (typeof insertIndex == "number") {
			this._items.splice(insertIndex,0,menuItem);
			//figure out which menu, and normalize
			var count = 0;
			for (var i = 0; i < this._menus.length; i++) {
				var curLength = this._menus[i].getChildren().length;
				if (insertIndex <= count + curLength) {
					//add it here
					insertIndex -= count;
					addMenu = this._menus[i];
					break;
				} else {
					count += curLength;
				}
			}
		} else {
			switch(insertIndex) {
				case "first":
					this._items.splice(0,0,menuItem);
					addMenu = this._menus[0];
					break;
				default:
					this._items.push(menuItem);
			}
		}
		if (!addMenu) addMenu = this._menus[this._menus.length - 1];
		addMenu.addChild(menuItem, insertIndex);	
		if (this._items.length > 1 && wh == WrapHints.FORCE_WRAP || insertIndex != null) {
			this._queueReflow();
		}
	},
	/**
	 * Removes the menuItem from the menu and returns it.  Pass the item or the item's index.
	 * Does not destroy the item.
	 * @param {Object|number} menuItem|index
	 */
	removeChild: function(menuItem) {
		var index = (typeof menuItem == "number") ? menuItem : this._getIndexOfChild(menuItem);
		if (index >= 0) {
			var item = this._items.splice(index,1)[0];
			try {
				var menu = item.getParent();
				if (menu && menu.focusedChild == item) {
					delete menu.focusedChild;
				}	
			} catch(e){
				// do nothing - the item isn't a _Contained
			}
			if (item && item.domNode) {
				item.domNode.parentNode.removeChild(item.domNode);
			}
			this._queueReflow();
			return item;
		}
	},
	/**
	 * Adds a bullet beside the item.
	 * @param {index|dijit.MenuItem|array of indices or menuItems}
	 */
	setSelected: function(item) {
		if (!this._selected) this._selected = [];
		var sel;
		while(this._selected.length > 0) {
			sel = this._selected.pop();
			if (sel.domNode) dojo.removeClass(sel.domNode, "selected");
		} 
		if (!item) {
			return;
		}
		if (dojo.isArray(item)) {
			for (var k = 0; k < item.length; k++) {
				if (typeof item[k] == "number") {
					item[k] = this.getChildren()[item[k]];
				}
			}
			this._selected = item;
		} else if (typeof item == "number") {
			this._selected = [ this.getChildren()[item] ];
		} else if (item.domNode) {
			this._selected = [ item ];
		}
		for (var i = 0; i < this._selected.length; i++) {
			var x = this._selected[i];
			if (x.domNode) {
				dojo.addClass(x.domNode, "selected");
				dijit.setWaiState(x.domNode, "checked", true);
			}
		}
	},
	/**
	 * Returns an array of the currently selected menu items
	 */
	getSelected: function() {
		return this._selected;
	},
	/**
	 * Returns array of children widgets.
	 */
	getChildren: function(){
		return this._items || [];
	},
	hasChildren: function() {
		return this._items.length > 0;
	},
	focus: function(dir) {
		if (this.showFilter) {
			this._filterBox.focus();
		} else {
			this._focus(dir);
		}
	},
	_focus: function(dir) {
		if (!this._noMatches) {
			var menu = (dir == "end") ? this._menus[this._menus.length - 1] : this._menus[0];
			var fc = menu._getNextFocusableChild(null, (dir=="end") ? -1 : 1);
			if (this._hidden) {
				var sibling = (dir=="end") ? "previousSibling" : "nextSibling";
				while (dojo.indexOf(this._hidden, fc) >= 0 && fc.domNode[sibling]) {
					fc = dijit.byNode(fc.domNode[sibling]);
				}
			}
			fc&&menu.focusChild(fc);
			this._pageTo(0);
		}
	},
	filter: function(value) {
		if (value == this._filterValue) return;
		var prevTitle,
			hasItems,
			hidden = [],
			show = [],
			menuItems = this._items;
		if (this._noMatches) {
			this.removeChild(this._noMatches).destroy();
			this._noMatches = null;
		}
		for (var i = 0; i < menuItems.length; i++) {
			var item = menuItems[i];
			if (item.isTitle) {
				if (prevTitle) {
					if (menuItems[i-1].isTitle) hasItems = true;
					if (!hasItems) {
						hidden.push(prevTitle);
					}
					prevTitle.domNode.style.display = hasItems ? "" : "none";
				}
				prevTitle = item;
				hasItems = false;
				continue;
			}
			var label = item.label;
			var hide = label ? label.toLowerCase().indexOf(value.toLowerCase()) < 0 : value ? true : false;
			if (hide) {
				hidden.push(item);
			} else {
				hasItems = true;
			}
			var displayVal = (dojo.isIE == 7) ? "block" : "table-row";
			item.domNode.style.display = hide ? "none" : value.length ? displayVal : "";
		}
		if (prevTitle && !menuItems[menuItems.length-1].isTitle) {
			prevTitle.domNode.style.display = hasItems ? "" : "none";
			if (!hasItems) hidden.push(prevTitle);
		}
		if (hidden.length == menuItems.length) {
			if (!this._noMatches) this._noMatches = this.addTitle("No matches.");
		}
		this._hidden = hidden;
		this._reflow();
		this._filterValue = value;
	},
	/**
	 * Called to determine whether it's safe to close the menu.
	 * 
	 * @return boolean
	 */
	canDismiss: function() {
		if (!this.showFilter) return true;
		try {
			if (this._filterBox._box.value.length > 0)
				return false;
		} catch (e) { }
		return true;
	},
	onOpen: function() {
		if (this.showFilter) this._filterBox.focus();
		if (!this._resizeHandle) {
			this._resizeHandle = jazz.util.ResizeNotifier.register({
				context: this,
				callback: this._onResize,
				force: true,
				publishEvents: false
			});
		}
		this._onResize();
	},
	onClose: function() {
		if (this.showFilter) this._filterBox.clear();
		if (this._resizeHandle) {
			jazz.util.ResizeNotifier.unregister(this._resizeHandle);
			this._resizeHandle = null;
		}
	},
	destroyDescendants: function(preserveDom) {
		this.inherited(arguments);
		while(this._menus.length > 1) {
			this._removeMenu();
		}
		var menu = this._menus[0];
		if (menu && menu.focusedChild) {
			delete menu.focusedChild;
		}
		this._selected = null;
		this._items = [];
	},
	uninitialize: function() {
		while (this._menus.length > 0) {
			this._menus.pop().destroy();
		}
	},
	/**
	 * Internal stuff
	 */
	_items: null,
	_menus: null,
	_menuConnects: null,
	_columnMaxItems: null,
	_maxColumns: 4,
	_hidden: null,
	_selected: null,
	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/MegaMenu.html"),
	widgetsInTemplate: true,
	postMixInProperties: function(){
        this.containerNode = dojo.create("div");
    },
	postCreate: function() {
		this._items = [];
		this._menus = [];
		this._menuConnects = [];
		this._hidden = [];
		this._addMenu();
		this.connect(this._wrapper, "onscroll", this._repaginate);
	},
	onExecute: function() {
		//stub for onExecute connect (see dijit.popup API)
	},
	onCancel: function(closeAll) { 
		//stub for onCancel connect (see dijit.popup API)
	},
	onReflow: function() { 
		//stub for onReflow connect
	},
	_addMenu: function() {
		var menus = this._menus;
		if (menus.length > 0) {
			var lastMenu = menus[menus.length - 1];
			if (lastMenu.domNode && lastMenu.domNode.parentNode) {
				dojo.removeClass(lastMenu.domNode.parentNode, "column-last");	
			}
		}
		var td = dojo.create("td", { "class": "column column-last" });
		var self = this;
		var menu = new jazz.ui.Menu({
			selectable: true,
			focusFirstChild: function(){
				var child = this._getFirstFocusableChild();
				if (child) {
					child.focusNode.tabIndex = 0;
					this.focusChild(child);
					child.focusNode.tabIndex = -1;
					child.className = child.className;
				}
			},
			focusNext: function(){
				var child = this._getNextFocusableChild(this.focusedChild, 1);
				if (child && this.focusedChild && child.getIndexInParent() <= this.focusedChild.getIndexInParent()) {
					var nextIndex = (menus.length > this.__index + 1) ? this.__index + 1 : 0,
						nextMenu = menus[nextIndex];
				}
				child.focusNode.tabIndex = 0;
				if (nextMenu) {
					self._pageTo(nextIndex);
					var nextChild = nextMenu._getFirstFocusableChild();
					if (self._hidden) {
						while (dojo.indexOf(self._hidden, nextChild) >= 0 && nextChild.domNode.nextSibling) {
							nextChild = dijit.byNode(nextChild.domNode.nextSibling);
						}
					}
					nextMenu.focusChild(nextChild);	
				} else {
					this.focusChild(child);
				}
				child.focusNode.tabIndex = -1;
				child.className = child.className;
			},
			focusPrev: function() {
				var child = this._getNextFocusableChild(this.focusedChild, -1);
				if (child && this.focusedChild && child.getIndexInParent() >= this.focusedChild.getIndexInParent()) {
					var nextIndex = (this.__index > 0) ? this.__index - 1 : menus.length - 1,
						nextMenu = menus[nextIndex];
				}
				child.focusNode.tabIndex = 0;
				if (nextMenu) {
					self._pageTo(nextIndex);
					nextChild = nextMenu._getNextFocusableChild(nextMenu.getChildren[0], -1);
					if (self._hidden) {
						while (dojo.indexOf(self._hidden, nextChild) >= 0 && nextChild.domNode.previousSibling) {
							nextChild = dijit.byNode(nextChild.domNode.previousSibling);
						}
					}
					nextMenu.focusChild(nextChild, true);
				} else {
					this.focusChild(child, true);
				}
				child.focusNode.tabIndex = -1;
				child.className = child.className;
			}
		});
		menu.__index = menus.length;
		menu.getIndexOfChild = dojo.hitch(this,this._getIndexOfChild);
		var onEx = dojo.hitch(this,function() {
			this.onExecute();
		});
		this._menuConnects.push([this.connect(menu,"onExecute",onEx), this.connect(menu,"onCancel",onEx)]);
		menus.push(menu);
		menu.placeAt(td);
		this._content.appendChild(td);
		this._started && menu.startup();
	},
	startup: function() {
		dojo.query("> [widgetId]", this.containerNode).map(dijit.byNode).forEach(function(child){this.addChild(child);}, this);
		for(var i = 0; i < this._menus.length; i++) {
			this._menus[i].startup();
		}
		this.inherited(arguments);
	},
	_removeMenu: function() {
		var menu = this._menus.pop(),
			connects = this._connects.pop(),
			pn = menu.domNode.parentNode;
		while(connects.length > 0)
			this.disconnect(connects.pop());
		menu.destroy();
		if (pn) {
			if (pn.previousSibling) dojo.addClass(pn.previousSibling, "column-last");
			pn.parentNode.removeChild(pn);
		}
	},
	_updateMaxCols: function() {
		var viewport = dijit.getViewport(),
			menuPos = dojo.position(this.domNode);
		var max = Math.max(1,Math.floor((viewport.w) / COLUMN_WIDTH));
		if (max !== this._maxColumns) {
			this._breakpoints = null;
			this._maxColumns = max;
		}
	},
	_onResize: function() {
		this._updateMaxCols();
		this._queueReflow();
	},
	_queueReflow: function() {
		if (!this._reflowQueue) {
			this._reflowQueue = setTimeout(dojo.hitch(this,this._reflow),0);
		}
	},
	_reflow: function() {
		if (!this.domNode) return;
		if (!this._maxColumns) this._updateMaxCols();
		var menus = this._menus,
			viewportH = dijit.getViewport().h - 20,
			reflow = !this._breakpoints,
			items = this._items;
		for (var i = 0; i < menus.length; i++) {
			var menu = menus[i];
			if (menu.focusedChild) {
				var focus = menu.focusedChild;
			}
			if (!reflow) {
				var menuPos = dojo.position(menu.domNode);
				var menuBottom = menuPos.y + menuPos.h;
				if (menuBottom > viewportH || menuBottom + 130 < viewportH) {
					reflow = true;
				}
			}
		}
		clearTimeout(this._reflowQueue);
		this._reflowQueue = null;
		if (!reflow) {
			this.onReflow();
			return;
		}
		var menuTop = dojo.position(this._table).y;
			
		//remove the menu from the DOM for reflow performance
		this._wrapper.removeChild(this._table);
		var mc = menus[0].containerNode;
		for (i = 0; i < items.length; i++) {
			mc.appendChild(items[i].domNode);
		}
		//put the menu back on the DOM
		dojo.place(this._table, this._wrapper, "first");
		
		var posArr = [];
		for (i = 0; i < items.length; i++) {
			posArr.push(dojo.position(items[i].domNode));
		}
		
		//remove the menu from the DOM for reflow performance
		this._wrapper.removeChild(this._table);
		
		var	breakpoints = [],
			baseline = 0,
			wrapAt, wrapPos, lastBreakpoint = 0;
		for (i = 2; i < items.length; i++) {
			var item = items[i];
			if (this._hidden && dojo.indexOf(this._hidden, item) >= 0) {
				continue;
			}
			var position = posArr[i];
			if (item.wrapHint != WrapHints.NO_WRAP) {
				wrapAt = i;
				wrapPos = position;
			}
			var doWrap = (position.y + position.h - baseline >= viewportH - 10);
			if (!doWrap && item.wrapHint != WrapHints.FORCE_WRAP) continue;
			if (wrapAt != null) {
				breakpoints.push(wrapAt);
				lastBreakpoint = wrapAt;
				baseline = wrapPos.y - menuTop;
				wrapAt = null;
			}
		}
		this._updateBreakpoints(breakpoints);
		this._updatePagination(breakpoints);
		
		//put the menu back on the DOM
		dojo.place(this._table, this._wrapper, "first");
		
		if (focus && focus.domNode) {
			var parent = focus.getParent();
			if (parent) parent.focusChild(focus);
		}
		this.onReflow();
	},
	_repaginate: function(event) {
		var menus = this._menus;
		for (var i = 0; i < menus.length; i++) {
			if (menus[i].focusedChild) {
				this._pageTo(i);
			}
		}
	},
	_pageTo: function (index) {
		var pagination = this._pagination;
		if (pagination) {
			if (index > pagination._end - 1) {
				var currentPage = Math.max(index - pagination.pageSize + 1, 0);
			} else if (index < pagination._start - 1){
				currentPage = Math.min(index, pagination.total - pagination.pageSize + 1);
			} else {
				return;
			}
			pagination.startPage = currentPage;
			pagination._updateWidget();
			this._table.style.marginRight = - (COLUMN_WIDTH * (pagination.total - pagination.pageSize - currentPage)) + "px";
			this._table.style.marginLeft = -(COLUMN_WIDTH * currentPage) + "px";
		}
	},
	_updatePagination: function(breakpoints) {
		var currentPage = 0;
		if (this._pagination) {
			currentPage = Math.min(this._pagination._start - 1, breakpoints.length + 1 - this._maxColumns);
			this._pagination.destroy();
			delete this._pagination;
		}
		var tableCells = this._content.childNodes;
		if (breakpoints.length >= this._maxColumns) {
			var onClickHandler = dojo.hitch(this, function(widget, start, end){
				if (this._anim && this._anim.status() == "playing") {
					this._anim.stop();
					this._table.style.marginLeft = this._anim.properties.marginLeft + "px";
				}
				this._anim = dojo.animateProperty({node: this._table, duration: 200,
					properties:	{ marginLeft: {end: - (COLUMN_WIDTH * (start - 1)), unit: "px" } },
					onEnd: dojo.hitch(this,function() {
						this._table.style.marginRight = - (COLUMN_WIDTH * (widget.total - end)) + "px";
						this._wrapper.style.width = "";
					})
				}).play();
				this._wrapper.style.width = this._wrapper.clientWidth + "px";
			});
			this._pagination = new jazz.ui.PaginationControl({
				"class": "jazz-ui-MegaMenu-pagination",
				total: breakpoints.length + 1,
				pageSize: this._maxColumns,
				increment: 1,
				onNextClick: onClickHandler,
				onPreviousClick: onClickHandler,
				startPage: currentPage || 0
			}).placeAt(this._pageWrapper);
			this._pageWrapper.style.display = "block";
			this._table.style.marginRight = - (COLUMN_WIDTH * (breakpoints.length + 1 - this._maxColumns - currentPage)) + "px";
			this._table.style.marginLeft = currentPage ? -(COLUMN_WIDTH * currentPage) + "px" : "";
		} else {
			this._pageWrapper.style.display = "none";
			this._table.style.marginRight = "";
			this._table.style.marginLeft = "";
		}
	},
	_updateBreakpoints: function(breakpoints) {
		var menus = this._menus,
			items = this._items,
			currentBreakpoint = 0,
			currentMenu = 0;
		this._breakpoints = breakpoints;
		while (menus.length > breakpoints.length + 1) {
			this._removeMenu();
		}
		while (menus.length < breakpoints.length + 1) {
			this._addMenu();
		}
		for (var i = 0; i < items.length; i++) {
			var cb = breakpoints[currentBreakpoint];
			if (cb && i == cb) {
				currentMenu++;
				currentBreakpoint++;
			}
			var index = currentBreakpoint > 0 ? i - breakpoints[currentBreakpoint] : i;
			if (items[i].getParent() != menus[currentMenu] || items[i].getIndexInParent() != index) {
				menus[currentMenu].addChild(items[i], "last");
			}
		}
	},
	_getIndexOfChild: function(item) {
		var items = this._items;
		for (var i = 0; i < items.length; i++) {
			if (items[i] == item) {
				return i;
			}
		}
		return -1;
	},
	_setShowFilterAttr: function(showFilter) {
		this.showFilter = showFilter;
		dojo.toggleClass(this.domNode, "showFilter", showFilter);
		this._filterArea.style.display = showFilter ? "block" : "none";
		if (this.showFilter) {
			this._filterValue = "";
			if (!this._filterKP) {
				this._filterKP = this.connect(this._filterBox._box, "onkeypress", function(event) {
					var kc = event.keyCode;
					var upArrow = (kc == dojo.keys.UP_ARROW);
					if (kc == dojo.keys.DOWN_ARROW || upArrow) {
						event.preventDefault();
						this._focus(upArrow ? "end" : "start");
					}
				});
			}
			if (!this._filterKD) {
				this._filterKD = this.connect(this.domNode, "onkeydown", function(event) {
					var keys = dojo.keys;
					switch (event.keyCode) {
						case keys.DOWN_ARROW:
						case keys.UP_ARROW:
						case keys.LEFT_ARROW:
						case keys.RIGHT_ARROW:
						case keys.ESCAPE:
						case keys.SHIFT:
						case keys.CTRL:
						case keys.META:
						case keys.TAB:
							return;
					}
					this._filterBox.focus();
				});
			}
			this._filterBox.onChange = dojo.hitch(this, this.filter);
		} else {
			this.disconnect(this._filterKP);
			this.disconnect(this._filterKD);
			delete this._filterKP;
			delete this._filterKD;
		}
	}
});

var WrapHints = {
	NO_WRAP: 1,
	FORCE_WRAP: 2,
	AUTO_WRAP: 3
}
dojo.mixin(jazz.ui.MegaMenu, {
	WrapHints: WrapHints
});

})();