/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.MessageBox");
dojo.require("jazz.util.html");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.requireLocalization("jazz.ui", "MessageBoxMessages");
	
(function() {

var messages;
var NO_DISPLAY = "jazz-ui-message-box-noDisplay";
var DEFAULT_MESSAGE_PREFIX = "CRJAZ";

dojo.declare("jazz.ui.MessageBox", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("jazz.ui", "templates/MessageBox.html"),
	defaultType: "INFO",
	toggleState: "hidden", // either "hidden" or "visible" for the state of the details box
	
	//callback functions
	onClose: null,		// you can return boolean false to prevent the close
	onToggle: null,		// when the user clicks hide/show details
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("jazz.ui", "MessageBoxMessages");
		this.messagePrefix = DEFAULT_MESSAGE_PREFIX;
	},

	postCreate: function() {
		dojo.toggleClass(this.domNode, NO_DISPLAY, true);
	},

	/* showMessage(args)
	 * 
	 * Displays a message of type Error, Warning, Info, or Success/OK.
	 * 
	 * Expects a message argument of the format:
	 *  args = {
	 *  	type: <value from STATUS_TYPES enum> (optional, default=INFO),
	 *  	summary: <string message or DOM node> (required), (may include a message id at the beginning, optional)
	 *  	details: *see below* (optional)
	 *  	errorObject: <errObj from failed service call> (optional)
	 *  	closable: <boolean> displays a close link to hide the message (optional, default=false)
	 *  }
	 *  
	 *  args.details = [ {message: "message 1", code: STATUS_TYPES.ERROR},
	 *  				 {message: "message 2", code: STATUS_TYPES.OK}, ... ]
	 */
	
	showMessage: function(args) {
		if(args && args.errorObject) {
			args = this._processErrorObject(args);	
		} else if(!args.summary) {
			console.error(this.messages.missingSummary);
			return;
		}
		
		if(!args.type) {
			args.type = this.defaultType;
		} else {
			//Check to make sure the type is valid
			var typeExists = dojo.exists(args.type, jazz.ui.MessageBox.STATUS_TYPES);
			if (!typeExists) {
				console.warn(dojo.string.substitute(this.messages.typeNotFound, [args.type]));
				args.type = this.defaultType;
			}
		}

		this.messageArea.innerHTML = "";
		this.messageArea.className = "messageArea " + args.type;	

		//See if we need to append a close link
		if(args.closable) {
			this.closeLink = dojo.create("a", {href: "#", "class": "closeLink", title: this.messages.closeMessage}, this.messageArea);
			this.connect(this.closeLink, "onclick", "_close");
		}
		
		this._createSummary(args.summary, this.messageArea, "messageSummary");
		
		if (args.details && args.details.length > 0) {				
			//Build detailsLink				
			this.detailsLink = dojo.create("a", { "innerHTML": this.messages.showDetails, "class": "detailsLink", "id" : "details-box" }, this.messageArea);
			this.connect(this.detailsLink, "onclick", "_toggleDetails");
			
			//Holds all of the individual detailed messages
			this.detailsBox = document.createElement("div");
			this.messageArea.appendChild(this.detailsBox);
			dojo.addClass(this.detailsBox,"detailsBox");
			
			//Get the actual details
			for (var i = 0; i < args.details.length; i++) {
				this._createSummary(args.details[i].message, this.detailsBox, "messageDetails");
			}
		}

		if (this._messageFadeAnimation) {
			// If the box is being animated, stop the current 
			// animation and reset the opacity
			this._messageFadeAnimation.stop();
			dojo.style(this.domNode, {opacity: 1});
		}
		dojo.toggleClass(this.domNode, NO_DISPLAY, false);
	}, 
	
	/* hideMessage()
	 * 
	 * Hides the message box. It does not empty the contents of the DIV.
	 *
	 * @param withFadeAnimation Boolean - Fade out the message box with animation when closing
	 */
	
	hideMessage: function (/* boolean */ withFadeAnimation) {
		if (withFadeAnimation) {
			this._doFadeOut();
		} else {
			dojo.toggleClass(this.domNode, NO_DISPLAY, true);
		}
	},
	
	 /*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_animationDone: function() {
		/* Hide the message box, but reset the opacity to 1 so that it can be re-used. */
		dojo.toggleClass(this.domNode, NO_DISPLAY, true);	
		dojo.style(this.domNode, {opacity: 1});
		delete this._messageFadeAnimation;
	},
	
	 /*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_doFadeOut: function() {
		dojo.style(this.domNode, {opacity: 1});
		this._messageFadeAnimation = dojo.fadeOut({
			node: this.domNode,
			duration: 1000,
			onEnd: dojo.hitch(this, this._animationDone)
		});
		this._messageFadeAnimation.play();
	},
	
	/*
	 * Closes the box and hides it in the DOM.
	 * Called automatically when the user clicks on the box's close button.
	 */
	
	_close: function(event) {
		if (event) event.preventDefault();
		if (this._messageFadeAnimation) {
			this._messageFadeAnimation.stop();
			dojo.style(this.domNode, {opacity: 1});
		}
		//Call onClose callback function, if it exists
		if (this.onClose && typeof this.onClose == "function") {
			if (this.onClose() === false) return;
		}
		this.hideMessage();
	},
	
	/*
	 * Returns the current toggle state of the "Show Details" box
	 * Values: "hidden" or "visible"
	 */
	
	getToggleState: function() {
		return this.toggleState;
	},
	
	_createSummary: function(summary, container, classname) {
		if(typeof summary == "string") {
			//Look for a message id, and display it.
			var m = dojo.trim(summary);
			if (m.substr(0, this.messagePrefix.length) == this.messagePrefix) {
				var pos = m.indexOf(" ");
				var msgId = m.substr(0, pos);
				summary = m.substr(pos + 1);
			}
			summary = jazz.util.html.removeScript(summary);
			var divSummary = dojo.create("div", { "innerHTML": summary, "class": classname});
			if(msgId) {
				var spanId = dojo.create("span", { "innerHTML": this.messages.messageID + "  " + msgId, "class": "messageId"});		
				divSummary.appendChild(spanId);		
			}		
			dojo.place(divSummary, container);
			
		} else if(this._isNodeElement(summary)) {	
			dojo.place(summary, container);
		} else {
			console.error(this.messages.summaryInvalid);
			return;
		}
	},
	
	_toggleDetails: function() {
		var isVisible = this.detailsBox.style.display == "block";
		this.detailsBox.style.display = isVisible ? "none" : "block";
		this.detailsLink.innerHTML = isVisible ? this.messages.showDetails : this.messages.hideDetails;
		
		//Set the new state of the toggle before calling the callback
		this.toggleState = isVisible ? "hidden" : "visible";
		
		//Call onToggle callback function, if it exists
		if (this.onToggle && typeof this.onToggle == "function") {
			this.onToggle();
		}
	},
	
	_processErrorObject: function(args) {		
		var error_code = jazz.ui.MessageBox.STATUS_TYPES.ERROR;
		var errorObj = args.errorObject;
		if (!args.type) {
			args.type = error_code;
		}
		
		if (errorObj.status === 0) {
			args.summary = this.messages.serverUnreachable;
		} else {
			// let the caller pass in an optional summary message specific to client code to override the errorObj.message
			args.details = args.details || [];
			if (args.summary && errorObj.message) {
				args.details.push({message: errorObj.message, code: error_code});
			}
			if(!args.summary){
				args.summary = errorObj.message ? errorObj.message : this.messages.unspecifiedServerError;
			}
			if (errorObj.errorClass) {
				args.details.push({message: errorObj.errorClass, code: error_code});
			}
			if (errorObj.exception) {
				var detail = dojo.isArray(errorObj.exception) ? errorObj.exception.join("<br>") : errorObj.exception;
				args.details.push({message: detail, code: error_code});
			}
			if (errorObj.nestedError) {
				var nestedDetails = this._handleNestedError(errorObj);
				args.details = args.details.concat(nestedDetails);
			}			
		}
		return args;
	},
	
	_handleNestedError: function(errorObj, nestedDetails) {
		nestedDetails = nestedDetails || [];
		//This check is needed in addition to the one above due to the recursion.		
		if(errorObj.nestedError) {			
			var error_code = jazz.ui.MessageBox.STATUS_TYPES.ERROR;
			nestedDetails.push({message: "<b>" + this.messages.causedBy + ":</b> ", code: error_code});		
			
			var nestedObj = errorObj.nestedError;
			if (nestedObj.errorClass && nestedObj.errorMessage) {
				nestedDetails.push({message: nestedObj.errorMessage + " (" + nestedObj.errorClass + ")", code: error_code});
			}
			if (nestedObj.errorTrace) {
				var detail = "";
				if(dojo.isArray(nestedObj.errorTrace)) {
					for(var x = 0; x < 4; x++){
						detail += nestedObj.errorTrace[x] + "<br/>";
					}
					detail += " ... " + (nestedObj.errorTrace.length - x) + " " + this.messages.more;
				} else {
					detail = nestedObj.errorTrace;
				}
				nestedDetails.push({message: detail, code: error_code});
			}
			return this._handleNestedError(nestedObj, nestedDetails);			
		} else {
			return nestedDetails;
		}
	},
	
	_isNodeElement: function(elem) {
		if(typeof HTMLElement === "object") {
			return elem instanceof HTMLElement;
		} else {
			return (elem.nodeType === 1 && typeof elem === "object" && typeof elem.nodeName === "string");
		}
	}
});

jazz.ui.MessageBox.STATUS_TYPES = {
	ERROR: "ERROR",
	WARNING: "WARNING", 
	OK: "OK", 
	INFO: "INFO"
};

dojo.mixin(jazz.ui.MessageBox, {
	setDefaultPrefix: function(newPrefix) {
		DEFAULT_MESSAGE_PREFIX = newPrefix;
	}
});


})();