/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.PaginationControl");

dojo.require("dojo.i18n"); 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.toolbar.Button");
dojo.requireLocalization("jazz.ui", "PaginationControlMessages");

(function() {

/*
 * Constants 
 */
var SUBSCRIPTION_TOPIC_PREFIX = "/jazz/ui/PaginationControl/",
	PUBLISH_EVENT_LAST_PAGE = 2,
	PUBLISH_EVENT_NEXT = 1,
	PUBLISH_EVENT_PREVIOUS = -1,
	PUBLISH_EVENT_FIRST_PAGE = -2;

/**
 * Constructs a new PaginationControl widget. All fields may be initialized in the widget constructor.
 * @name jazz.ui.PaginationControl
 * @class <p>A pagination widget for web UIs.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @constructor
 * @extends dijit._Widget
 * @extends dijit._Templated
 * @since 2.0
 */ 
dojo.declare("jazz.ui.PaginationControl", [dijit._Widget, dijit._Templated], /** @lends jazz.ui.PaginationControl.prototype */{
	
	/**
	 * (Required) The total number of results that the user can page through.
	 * @type Number
	 * @default 0
	 */
	total: 0,

	/**
	 * (Required) The number of results that are skipped when the user clicks on "Next" or "Previous". 
	 * @type Number
	 * @default 0
	 */
	increment: 0,
	
	/**
	 * (Optional) The number of results per page. If zero, this will default to the increment value.
	 * @type Number
	 * @default 0
	 */
	pageSize: 0,
	
	/**
	 * (Optional) The initial page index to use when initializing the control (0 based index, e.g. Index 0, 1, ..., n = Page 1, 2, ..., n+1)
	 * @type Number
	 * @default 0
	 */
	startPage: 0,
	
	/**
	 * A user-defined callback function called by the widget when the user clicks "Next". This is typically
	 * used by the client to programmatically show the next page of results. 
	 * @type Function
	 * @param {PaginationControl} widget A reference to this widget instance
	 * @param {Number} start The starting number of the range for the next page of results
	 * @param {Number} endt The ending number of the range for the next page of results
	 */
	onNextClick: null,

	/**
	 * A user-defined callback function called by the widget when the user clicks "Previous". This is typically
	 * used by the client to programmatically show the next page of results. 
	 * @type Function
	 * @param {PaginationControl} widget A reference to this widget instance
	 * @param {Number} start The starting number of the range for the next page of results
	 * @param {Number} endt The ending number of the range for the next page of results
	 */
	onPreviousClick: null,
	
	/**
	 * A user-defined callback function called by the widget to get the href for the "Next" link. The updated href
	 * points to the next page of results
	 * @type Function
	 * @param {PaginationControl} widget A reference to this widget instance
	 * @param {Number} start The starting number of the range for the next page of results
	 * @param {Number} end The ending number of the range for the next page of results
	 * @return {String} The href
	 */
	getNextHref: null,

	/**
	 * A user-defined callback function called by the widget to get the href for the "Previous" link. The updated href
	 * points to the previous page of results
	 * @type Function
	 * @param {PaginationControl} widget A reference to this widget instance
	 * @param {Number} start The starting number of the range for the next page of results
	 * @param {Number} end The ending number of the range for the next page of results
	 * @return {String} The href
	 */
	getPreviousHref: null,
	
	/**
	 * A unique ID used to synchronize pagination state across multiple widget instances. When two or more pagination
	 * controls specify the same ID, any update to one will be reflected in the other.  Note 1: In order to be useful,
	 * the widgets must also share the same totoal, increment and next and previous callbacks. Note 2: The next or previous
	 * callback is fired only once from the widget where the event originated.
	 * @type String
	 * @default null
	 */
	name: null, 
	
	/**
	 * Enable/disable the widget. 
	 * @param {Boolean} enable Flag indicating whether to enable or disable the widget.
	 */
	setEnabled: function(enable) {
		if (enable) {
			// Enable
			this._togglePreviousLinks(this._start > 1);
			this._toggleNextLinks(this._end !== this.total)
				
			if (this._hrefNavigation) {
				
				var pageSize = this._getPageSize();
				// Next
				var nextHref = this.getNextHref(this, this._start, this._end);
				this._setHref(this._next, nextHref);

				// Previous
				var previousHref = this.getPreviousHref(this, this._start, this._end);
				this._setHref(this._previous, previousHref);
				
				// First
				this._firstPageHref = this.getPreviousHref(this, 1+this.increment, 1+this.increment+pageSize);
				this._setHref(this._firstLink, this._firstPageHref);
				
				// Last
				var remaining = this.total % this.increment;
				var e = this.total - remaining - this.increment;
				var s = e - pageSize + 1;
				this._lastPageHref = this.getNextHref(this, s, e);
				this._setHref(this._lastLink, this._lastPageHref);
				
			}
		} else {
			// Disable

			this._toggleNextLinks(false);
			this._togglePreviousLinks(false);
			
			if (this._hrefNavigation) {
				this._setHref(this._next, null);
				this._setHref(this._previous, null);
			}
		}

	},
	/**
	 * Sets the total number of items.
	 * @param {Number} total Total number of items
	 */	
	setTotal: function(total) {
		this.total = total;
		this._initWidget();
	},

	/**
	 * Internals
	 */
	
	_hrefNavigation: false,
	_processEventDelay: 0,
	widgetsInTemplate: true,

	// ------------------------------------------------------------------------>

	/** @private */
	templatePath: dojo.moduleUrl("jazz.ui", "templates/PaginationControl.html"),
	
	/** @private */
	constructor: function(args) {
		this.msg = dojo.i18n.getLocalization("jazz.ui", "PaginationControlMessages");
	},

	/** @private */	
	postCreate: function() {
		this._initWidget();
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initWidget: function() {
		
		if (this.getNextHref &&
			this.getPreviousHref) {
			this._hrefNavigation = true;
		}		
		
		this._updateWidget();
		
		if (!this._initialized && this.total > 0) {	// only run this the first time that we have a total > 0
			
			if (this._hrefNavigation) {
				this._processEventDelay = 10; // ms
			}
		
			this._addSubscriptions();
			this._initialized = true;
		}
	},	

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_updateWidget: function() {
		var start = this.startPage * this.increment + 1;
		var end = this.startPage * this.increment + this._getPageSize();
		
		this._start = start > this.total ? this.total : start;
		this._end = end > this.total ? this.total : end;

		this.setEnabled(true);

		this._setNodeNumber(this._pgStart, this._start);
		this._setNodeNumber(this._pgEnd, this._end);
		this._setNodeNumber(this._total, this.total);
	},

	/*
 	 * @param Element link
 	 * @param String href
 	 * 
 	 * @return void
 	 */		
	_setHref: function(link, href) {
		if (href &&	link.enabled) link.attr("href", href);
	},

	/*
 	 * @param Element node
 	 * @param Number value
 	 * 
 	 * @return void
 	 */		
	_setNodeNumber: function(node, value) {
		dojo.place(document.createTextNode(value), node, "only");
	},
	
	/*
 	 * @param enabled controls whether the next links should be enabled or disabled 
 	 * 
 	 * @return void
 	 */			
	_toggleNextLinks: function(enabled) {
		this._setLinkEnablement(this._next, enabled);
		this._setLinkEnablement(this._lastLink, enabled);
	},
	
	/*
 	 * @param enabled controls whether the previous links should be enabled or disabled
 	 * 
 	 * @return void
 	 */		
	_togglePreviousLinks: function(enabled) {
		this._setLinkEnablement(this._previous, enabled);
		this._setLinkEnablement(this._firstLink, enabled);
	},
	
	/*
 	 * @param prevLinksEnabled should enable previous/first page button controls
 	 * @param nextLinksEnabled should enable next/last page button controls
 	 * 
 	 * @return void
 	 */		
	_updateButtonEnablement: function(/* boolean */ prevLinksEnabled, /* boolean */ nextLinksEnabled) {
		this._togglePreviousLinks(prevLinksEnabled);
		this._toggleNextLinks(nextLinksEnabled);
	},

	/*
 	 * @param Element node
 	 * @param Boolean enable
 	 * 
 	 * @return void
 	 */		
	_setLinkEnablement: function(node, isEnabled) {
		if (node) node.attr("enabled", isEnabled);
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_processNextClickEvent: function() {
		if (!this.domNode) return;
		delete this._timeout;
		var previousHref, nextHref;
		this._updateButtonEnablement(true, this._end !== this.total);		
		if (this._hrefNavigation) {
			previousHref = this.getPreviousHref(this, this._start, this._end);
			this._setHref(this._previous, previousHref);
			nextHref = this.getNextHref(this, this._start, this._end);
			this._setHref(this._next, nextHref);
		} else if (this.onNextClick) {
			this.onNextClick(this, this._start, this._end);
		}
				
		this._firePublishEvent(PUBLISH_EVENT_NEXT, previousHref, nextHref);
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */		
	_processNextEvent: function() {
		if (!this._next.enabled) {
			return false;
		}

		this._start = this._start + this.increment;
		this._end = this._start + this._getPageSize() - 1;
			
		if (this._end > this.total) {
			this._end = this.total;
		}
		
		this._setNodeNumber(this._pgStart, this._start);
		this._setNodeNumber(this._pgEnd, this._end);

		return true;		
		
	},
	
	/*
 	 * @param event
 	 * 
 	 * @return void
 	 */	
	_onNextClick: function(event) {
		if (!this._hrefNavigation && event) {
			dojo.stopEvent(event);
		}
		
		if (!this._timeout && this._processNextEvent()) {
			this._timeout = setTimeout(dojo.hitch(this, this._processNextClickEvent), this._processEventDelay);
		}			

		return true;			
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_processPreviousClickEvent: function() {
		if (!this.domNode) return;
		delete this._timeout;
		var previousHref, nextHref;
		
		this._updateButtonEnablement(this._start !== 1, true);	
		if (this._hrefNavigation) {
			previousHref = this.getPreviousHref(this, this._start, this._end);
			this._setHref(this._previous, previousHref);					
			nextHref = this.getNextHref(this, this._start, this._end);
			this._setHref(this._next, nextHref);
		} else if (this.onPreviousClick) {
			this.onPreviousClick(this, this._start, this._end);
		}

		this._firePublishEvent(PUBLISH_EVENT_PREVIOUS, previousHref, nextHref);
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */		
	_processPreviousEvent: function() {
		if (!this._previous.enabled) {
			return false;					
		}

		this._start = this._start - this.increment;
		if (this._start < 1) {
			this._start = 1;
		}
		this._end = this._start + this._getPageSize() - 1;
		
		this._setNodeNumber(this._pgStart, this._start);
		this._setNodeNumber(this._pgEnd, this._end);
		
		return true;
		
	},
	
	/*
 	 * @param event
 	 * 
 	 * @return void
 	 */	
	_onPreviousClick: function(event) {
		if (!this._hrefNavigation && event) {
			dojo.stopEvent(event);
		}
		if (!this._timeout && this._processPreviousEvent()) {
			this._timeout = setTimeout(dojo.hitch(this, this._processPreviousClickEvent), this._processEventDelay);
		}
		return true;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_processFirstClickEvent: function() {
		if (!this.domNode) return;
		delete this._timeout;
		var nextHref;
		this._updateButtonEnablement(false, true);
		if (this._hrefNavigation) {
			nextHref = this.getNextHref(this, this._start, this._end);
			this._setHref(this._next, nextHref);
		} else if (this.onPreviousClick) {
			this.onPreviousClick(this, this._start, this._end);
		}
		this._firePublishEvent(PUBLISH_EVENT_FIRST_PAGE, undefined, nextHref);
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */		
	_processFirstLinkEvent: function() {

		if (!this._firstLink.enabled) {
			return false;					
		}			
		
		this._start = 1;
		this._end = this._getPageSize();
		
		/* Update the page number */		
		this._setNodeNumber(this._pgStart, this._start);
		this._setNodeNumber(this._pgEnd, this._end);

		return true;
		
	},
	
	/*
 	 * @param event
 	 * 
 	 * @return Boolean
 	 */		
	_onFirstPageClick: function(event) {
		if (!this._hrefNavigation && event) {
			dojo.stopEvent(event);
		}
		if (!this._timeout && this._processFirstLinkEvent()) {
			this._timeout = setTimeout(dojo.hitch(this, this._processFirstClickEvent), this._processEventDelay);
		}
		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_processLastClickEvent: function() {
		if (!this.domNode) return;
		delete this._timeout;
		var previousHref;
		this._updateButtonEnablement(true, false);
		if (this._hrefNavigation) {
			previousHref = this.getPreviousHref(this, this._start, this._end);
			this._setHref(this._previous, previousHref);
		} if (this.onNextClick) {
			this.onNextClick(this, this._start, this._end);
		}
		this._firePublishEvent(PUBLISH_EVENT_LAST_PAGE, previousHref, undefined);
		
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */		
	_processLastLinkEvent: function() {
		if (!this._lastLink.enabled) {
			return false;					
		}			

		var remaining = this.total % this.increment;
		this._end = this.total;
		this._start = this._end - (remaining > 0 ? remaining-1 : this._getPageSize()-1);
		
		/* Update the page number */		
		this._setNodeNumber(this._pgStart, this._start);
		this._setNodeNumber(this._pgEnd, this._end);

		return true;
		
	},
	
	/*
 	 * @param event
 	 * 
 	 * @return Boolean
 	 */	
	_onLastPageClick: function(event) {
		if (!this._hrefNavigation && event) {
			dojo.stopEvent(event);
		}

		if (!this._timeout && this._processLastLinkEvent()) {
			setTimeout(dojo.hitch(this, this._processLastClickEvent), this._processEventDelay);
		}

		return true;
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_addSubscriptions: function() {
		
		if (!this.name) {
			return;
		}
		
		var topic = SUBSCRIPTION_TOPIC_PREFIX + this.name;
		
		this.subscribe(topic, function(event) {
			if (!event) {
				// Error
				return;
			} else if (this.id === event.sourceID) {
				// Ignore self published events
				return;
			} else if (event.type === PUBLISH_EVENT_NEXT) {
				this._processNextEvent();
				this._updateButtonEnablement(true, this._end !== this.total);
			} else if (event.type === PUBLISH_EVENT_PREVIOUS) {
				this._processPreviousEvent();
				this._updateButtonEnablement(this._start !== 1, true);
			} else if (event.type === PUBLISH_EVENT_LAST_PAGE) {
				this._processLastLinkEvent();
				this._updateButtonEnablement(true, false);
			} else if (event.type === PUBLISH_EVENT_FIRST_PAGE) {
				this._processFirstLinkEvent();
				this._updateButtonEnablement(false, true);
			}

			if (event.previousHref) {
				this._setHref(this._previous, event.previousHref);
			}
			if (event.nextHref) {
				this._setHref(this._next, event.nextHref);
			}
		});
	},

	/*
 	 * @param Number eventType
 	 * @param String previousHref
 	 * @param String nextHref
 	 * 
 	 * @return void
 	 */		
	_firePublishEvent: function(eventType, previousHref, nextHref) {
		
		if (!this.name) {
			return;
		}

		var topic = SUBSCRIPTION_TOPIC_PREFIX + this.name;
		var parameter = {
    		type: eventType,
			sourceID: this.id
		};
		
		if (previousHref) {
			parameter.previousHref = previousHref;
		}
		if (nextHref) {
			parameter.nextHref = nextHref;
		}
		
		dojo.publish(topic, [parameter]);
		
	},
	
	_getPageSize: function() {
		return this.pageSize || this.increment;
	}
});

})();