/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.SearchBox");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.Menu");
dojo.require("jazz.util.i18n");
dojo.require("jazz.util.html");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.ui.PaginationControl")
dojo.require("jazz.ui.ResourceLink");

dojo.require("jazz.ui.MegaMenu");

dojo.requireLocalization("jazz.ui", "SearchBoxMessages");

(function() {

var _MESSAGES = dojo.i18n.getLocalization("jazz.ui", "SearchBoxMessages");
var _DELAY = 600;
var _DEFAULT_TEXT = _MESSAGES.defaultText;

/**
 * A customizable search box with a text field, search button, and scope selection
 * menu. Provides a number of parameters in the constructor for the component-
 * specific behaviors, such as updating the results, performing the search,search
 * scopes, etc. There are two modes of searching:
 * 
 * 1) Search as you type - A results drop down will appear below the search
 *    results and will be updated as you type. This feature is enabled by supplying
 *    an onUpdate function in the constructor.
 * 2) Traditional - Type in the query and press enter or click on the search
 *    button. Enabled by providing an onExecute function in the constructor.
 * 
 * At least one of onUpdate and onExecute must be supplied. Both are allowed
 * simultaneously.
 * 
 * NOTE: This is provisional API. Clients using this API do so with the
 * awareness that this class may be removed or substantially changed at
 * future times without warning.
 * 
 * @since 0.7
 */
dojo.declare("jazz.ui.SearchBox", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/SearchBox.html"),
	
	/**
	 * {object[]} An array of objects containing information about each scope type
	 * to provide for the search box. Each object must have 3 properties:
	 * 	- id (string): A unique string representing the scope
	 * 	- label (string): A human-readable label for the scope
	 * 	- icon (string): A path to a 16x16 image for the scope icon
	 * e.g.:
	 * 	[
	 * 		{ id: "scope1", label: "First Scope", icon: dojo.moduleUrl("my.module", "images/scope1.gif") },
	 * 		{ id: "scope2", label: "Second Scope", icon: dojo.moduleUrl("my.module", "images/scope2.gif") },
	 * 	]
	 */
	scopes: null,
	
	/**
	 * {function} An optional function that, if specified, will be called when the
	 * search results drop-down (search as you type) needs to be updated. The function
	 * will be given the following parameters:
	 * 	- query (string): The text to search for
	 * 	- scope (string): The id of the currently selected scope (see scopes param)
	 * 	- callback (function): The callback function for returning the search
	 * 		results. The function takes one parameter, an array of objects
	 * 		containing the search results, each with a label parameter containing
	 * 		HTML formatted markup for the result, a handler function (optional)
	 * 		to be executed when the result is selected, and an array of child
	 * 		results (optional), which will be grouped under the parent.
	 * 		e.g.: 
	 * 			[ 
	 * 				{ label: "<b>First</b> result", handler: function() {...} }
	 * 				{ label: "<b>Second</b> result", children: [ {...}, {...}] }
	 * 			]
	 * 	- errorCallback (function): The callback to use for reporting errors.
	 * 		Accepts one string parameter containing the error message.
	 * 
	 * One of onUpdate, onExecute must be specified.
	 */
	onUpdate: null,
	
	/**
	 * {function} An optional function that, if specified, will be called when the user
	 * clicks on the search button or presses enter. It accepts the following parameters:
	 * 	- query (string): The text to search for
	 * 	- scope (string): The id of the currently selected scope (see scopes param)
	 * If not specified, onUpdate will be called instead, and the first result with
	 * a handler will be executed.
	 * 
	 * One of onUpdate, onExecute must be specified.
	 */
	onExecute: null,
	
	/**
	 * {string} The tooltip text to show for the search field on mouse hover.
	 */
	tooltip: null,
	
	/**
	 * {string} Optional default text to show in the search box when empty, e.g.
	 * "Type to search".
	 */
	defaultText: null,
	
	/**
	 * {string} An optional HTML-formatted label (string) to show in the update results
	 * when no results were found, e.g. "No results found".
	 */
	noResultsLabel: null,
	
	/**
	 * {number} An optional minimum width (integer) in pixels for the drop down results
	 * shown as you type.
	 */
	minResultsWidth: null,
	
	/**
	 * Constructs a new SearchBox from the given args listed above. For example:
	 * 
	 * var searchBox = new SearchBox({
	 *    scopes: [
	 *       {id: "scope1", label: "Scope 1", icon: dojo.moduleUrl("my.bundle", "path/scope1.gif")},
	 *       {id: "scope2", label: "Scope 2", icon: dojo.moduleUrl("my.bundle", "path/scope2.gif")},
	 *    ],
	 *    onExecute: _onExecute,
	 *    onUpdate: _onUpdate,
	 *    tooltip: "Search",
	 *    noResultsLabel: "No items found"
	 * });
	 * containerDiv.appendChild(searchBox.domNode);
	 * 
	 * @param {object} args the constructor arguments (listed above).
	 */
	constructor: function(/** object */ args) {
		this._messages = _MESSAGES;
		
		// to skip stale responses that come out of order
		this._sequenceCounter = 0;
		this._lastResponseReceived = -1;
		this._isHoveringSearchButton = false;
	},
	
	postCreate: function() {
		if (this.delay) {
			dojo.deprecated("jazz.ui.SearchBox.delay","This parameter is no longer supported.")
		}
		if (this._bannerSearch) {
			dojo.addClass(this.domNode,"jazz-ui-BannerSearchBox")
			dojo.addClass(this.domNode,"jazz-ui-BannerSearchBox-areaInactive");
			this.connect(this,"_onFocus",this._showBannerSearchArea);
			this.connect(this,"_onBlur",this._hideBannerSearchArea);
			this._isAdvanced = false;
			for (var i = 0; i < this.providers.length; i++) {
				this.connect(this.providers[i],"_setScopesAttr",this._parseProviders);
			}
			this._parseProviders();
		} else if (this.provider) {
			dojo.deprecated("jazz.ui.SearchBox#provider","The SearchBox no longer supports instantiation using a jazz.ui._SearchProvider, some functionality may not work using this mechanism");
			this._setScope({
				provider: this.provider,
				scope: this.provider.scopes[0],
				focusText: false
			});
		} else {
			this._setScope({
				scope: this.scopes[0],
				focusText: false
			});
		}
		dijit.setWaiRole(this._scopeMenu, "button");
		dijit.setWaiState(this._scopeMenu, "expanded", false);
		this._searchButtonSpan.appendChild(document.createTextNode(_MESSAGES.searchButtonAltText));
		
		this.connect(this._searchText, "onkeyup", this._onchange);
		this.connect(this._searchText, "oninput", this._onchange);
		this.connect(this._searchText, "onkeypress", this._onkeypress);
		this.connect(this._searchText, "onfocus", this._onfocus);
		this.connect(this._searchText, "onblur", this._onblur);
		this.connect(this._scopeMenu, "onclick", this._openMenu);
		this.connect(this._scopeMenu, "onkeypress",this._sniffKey);
		this.connect(this._searchButton, "onclick",this._onclickSearchButton);
		this.connect(this._searchButton, "onmouseover",this._onmouseoverSearchButton);
		this.connect(this._searchButton, "onmouseout",this._onmouseoutSearchButton);
	},
	
	_parseProviders: function() {
		var provider = this._provider || this.defaultProvider, scope;
		if (!this._isProviderValid(provider)) {
			provider = null;
			for (var i = 0; i < this.providers.length; i++) {
				if (this._isProviderValid(this.providers[i])) {
					provider = this.providers[i];
					break;
				}
			}
		}
		if (!provider) {
			this.domNode.style.display = "none";
		} else {
			this.domNode.style.display = "";
			var scopeId = (this._scope && this._scope.id) || this.defaultScope;
			if (scopeId) {
				for (var i = 0; i < provider.scopes.length; i++) {
					if (scopeId === provider.scopes[i].id) {
						scope = provider.scopes[i];
						break;
					}
				}
			} 
			if (!scope)
				scope = provider.scopes[0];
			this._setScope({
				provider: provider,
				scope: scope,
				focusText: false
			});
		}
	},
	
	_isProviderValid: function(provider) {
		return provider && provider.scopes && provider.scopes.length;
	},
	
	destroy: function() {
		if (this._results) {
			this._results.destroyRecursive();
			delete this._results;
		}
		if (this._menu) {
			this._menu.destroyRecursive();
			delete this._menu;
		}
		this.inherited(arguments);
	},
	
	_getProperty: function(property) {
		var result, source;
		if (this._scope && this._scope[property]) {
			result = this._scope[property];
			source = this._scope;
		}
		if (this._provider && this._provider[property]) {
			result = this._provider[property];
			source = this._provider;
		}
		if (!result) {
			result = this[property];
			source = this;
		}
		return typeof result === "function" ? dojo.hitch(source,result) : result;
	},
	
	_getDefaultText: function() {
		return this._getProperty("defaultText") || (this._provider && this._provider.name ? dojo.string.substitute(_MESSAGES.tooltip, [ this._provider.name ]) : _DEFAULT_TEXT);
	},
	
	_onchange: function(e) {
		var query = dojo.trim(this._searchText.value);		
		if (query != this._currentQuery) {
			this._currentQuery = query;
			if (this._timeout) {
				clearTimeout(this._timeout);
				delete this._timeout;
			}
			this._timeout = setTimeout(dojo.hitch(this, this._update, false, 0, false), _DELAY);
		}
		this._updateSearchButton();
	},
	
	_onkeypress: function(e) {
		if (e && e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER) {
			if (e.keyCode == dojo.keys.DOWN_ARROW && this._results) {
				// down arrow from text field jumps to drop down
				this._results.focus();
			}
			if (e.keyCode == dojo.keys.ENTER) {
				this._performExecute();
			}
			dojo.stopEvent(e);
		}
		if (e && e.keyCode == dojo.keys.TAB && !e.shiftKey && this._isLegacyResultsOpen) {
			this._results.focus();
			dojo.stopEvent(e);
		}
	},
	
	_onfocus: function(e) {
		if (this._searchText.value == this._getDefaultText()) {
			this._searchText.value = "";
		}
		dojo.removeClass(this._searchForm, "inactive");
		this._closeMenu();
	},

	_onblur: function(e) {
		// kill the periodic update
		if (this._timeout) {
			clearTimeout(this._timeout);
			delete this._timeout;
		}
		// show default help text if empty
		if (dojo.trim(this._searchText.value) == "") {
			dojo.addClass(this._searchForm, "inactive");
			this._searchText.value = this._getDefaultText();
		}
	},
	
	_onclickSearchButton: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		this._performExecute();
	},
	
	_onmouseoverSearchButton: function() {
		this._isHoveringSearchButton = true;
		this._updateSearchButton();
	},
	
	_onmouseoutSearchButton: function() {
		this._isHoveringSearchButton = false;
		this._updateSearchButton();
	},
	
	_performExecute: function() {
		if ((this._currentQuery && this._currentQuery.length > 0) || this._bannerSearch) {
			var onExecute = this._getProperty("onExecute"), actionTaken = false;
			if (onExecute) {
				// stop updating
				if (this._timeout) {
					clearTimeout(this._timeout);
					delete this._timeout;
				}
				this._closeLegacyResults();
				if (this._provider) {
					actionTaken = !onExecute({
						query: this._currentQuery || "",
						scope: this._scope.id,
						advanced: this._isAdvanced
					});
				} else {
					onExecute(this._currentQuery, this._scope.id);
					actionTaken = true;
				}
			}
			// no global execute handler, or provider indicated this query should be handled with onUpdate
			if (!actionTaken && this._getProperty("onUpdate")) {
				this._update(true);
			}
			if (this._bannerSearch && actionTaken && !this._getProperty("resultsArea")) {
				// if we fired an execute or a default update action and the provider does not use the results area, hide the focus area
				this._doHideBannerSearchArea(null, true);
				this._clearResults(true);
			}
		}
	},
	
	_showErrorLegacy: function(/* string */ errorMsg) {
		this._showResultsLegacy([{label: errorMsg}], this.minResultsWidth);
	},
	
	_showResultsLegacy: function(/* object[] */ results, /* number */ minWidth) {
		if (this._isResultsShowing) {
			// if the new sytle loading message is shown, hide it when we show legacy resutls
			this._clearResults(true);
		}
		if (!results || results.length == 0) {
			results = [{label: this._getProperty("noResultsLabel") || _MESSAGES.noResultsLabel}];
		}
		if (!this._results) {
			this._results = new net.jazz.web.ui.internal.SearchResults({predecessor: this._searchText, successor: this.domNode.nextSibling, onSelect: dojo.hitch(this, this._onSelectResult)});
			this._resultsCloseOnBody = this.connect(this, "_onBlur", this._closeLegacyResults);
		}
		
		var pos = dojo.position(this._searchWrapper);
		
		// use client specified min width if available, else size according to search box
		var width = minWidth;
		if (!width || width < pos.w) {
			width = pos.w;
		}
		
		this._results.setContent(results, width);
		
		// open if needed
		if (this._currentQuery.length > 0) {
			if (this._bannerSearch && dojo.isIE && dojo.isIE < 8) {
				// workaround a bug in IE7 where the content wraps if it's up against the right side
				this._results.domNode.style.width = "100%";
				dijit.placeOnScreenAroundNode(this._results.domNode, this._searchWrapper, {"BR": "TR"});
				this._results.domNode.style.width = "";
			}
			dijit.placeOnScreenAroundNode(this._results.domNode, this._searchWrapper, this._bannerSearch ? {"BR": "TR"} : {"BL": "TL"});
			// prevent _onBlur from firing when keyboarding into the results dropdown
			dojo.attr(this._results.domNode, {dijitPopupParent: this.id});
			this._isLegacyResultsOpen = true;
		}
	},
	
	_onSelectResult: function(e) {
		if (e) {
			dojo.stopEvent(e);
			if (e.currentTarget) {
				e.currentTarget.blur();
			}
		}
		this._closeLegacyResults();
	},
	
	_closeLegacyResults: function(){
		if (this._results) {
			dijit.popup.close(this._results);
			if (this._results) {
				this._results.destroyRecursive();
				delete this._results;
			}
			this._isLegacyResultsOpen = false;
		}
	},
	
	_handlePagination: function(pagination, start, end) {
		var page = Math.ceil(start/10);
		this._update(true,page);
	},
	
	_showResults: function(results, page, query) {
		this._clearResults();
		if (!results.totalResults) {
			var label = this._getProperty("noResultsLabel") || _MESSAGES.noResultsLabelWithTerm;
			this._showMessage(dojo.string.substitute(label, [ query ]));
		} else if (!results.paging) {
			var showing = Math.min(10,results.totalResults);
			this._bannerSearchArea._statusInfo.innerHTML = dojo.string.substitute(_MESSAGES.itemsShowing, [ showing, results.totalResults, query ]);
			if (this._getProperty("onExecute")) {
				var showAll = dojo.create("a",{
					innerHTML: "Show All >>",
					href: "#"
				},this._bannerSearchArea._statusControls,"only");
				this._resultConnects.push(this.connect(showAll,"onclick",this._onclickSearchButton));
			}
		} else if (!page) {
			// if this was not a result of changing pages, populate the header
			this._bannerSearchArea._statusInfo.innerHTML = dojo.string.substitute(_MESSAGES.resultsFound, [ results.totalResults, query ]);
			var onPagination = dojo.hitch(this, this._handlePagination);
			var pagination = new jazz.ui.PaginationControl({
				total: results.totalResults,
				startPage: 0,
				increment: 10,
				onNextClick: onPagination,
				onPreviousClick: onPagination
			});
			dojo.place(pagination.domNode,this._bannerSearchArea._statusControls,"only");
		}
		this._bannerSearchArea._statusArea.style.display = "block";
		var root = this._bannerSearchArea._resultsBody;
		for (var i = 0; i < results.results.length; i++) {
			var result = results.results[i];
			var div = dojo.create("div",{className: "search-result"});
			var icon = dojo.create("div",{className: "icon"},div);
			if (result.icon)
				dojo.create("img",{src:result.icon},icon);
			var link = dojo.create("div",{className: "link"},div);
			var a = dojo.create("a",{href: result.uri || "#", innerHTML: result.label},link);
			if (result.uri) {
				var rlArgs = {retainLinkText: true};
				if (result.presentationUri) rlArgs.presentationUri = result.presentationUri;
				var rl = new jazz.ui.ResourceLink(rlArgs, a);
				this._resultConnects.push(this.connect(rl.popup,"onOpen",function() {
					this._isResourceLinkOpen = true;
				}));
				this._resultConnects.push(this.connect(rl.popup,"onClose",function() {
					this._isResourceLinkOpen = false;
				}));
				this._resultWidgets.push(rl);
			}
			if (result.description)
				dojo.create("div",{innerHTML: result.description},div);
			root.appendChild(div);
			this._resultConnects.push(this.connect(a,"onclick", function(e){
				e.target.blur();
				if (!result.uri)
					e.preventDefault();
				if (result.onclick)
					result.onclick(e);
			}));
		}
		this._isResultsShowing = true;
	},
	
	_showLoading: function() {
		this._showMessage(_MESSAGES.loading);
	},
	
	_showError: function(errorMsg) {
		this._showMessage(errorMsg);
		dojo.addClass(this._bannerSearchArea._statusInfo,"status-error");
	},
	
	_showMessage: function(msg) {
		this._clearResults();
		this._bannerSearchArea._statusArea.style.display = "block";
		this._bannerSearchArea._statusInfo.innerHTML = msg;
		dojo.empty(this._bannerSearchArea._statusControls);
		this._isResultsShowing = true;
	},
	
	_clearResults: function(hideStatus) {
		if (this._resultConnects) {
			for (var i = 0; i < this._resultConnects.length; i++)
				this.disconnect(this._resultConnects[i]);
		}
		this._resultConnects = [];
		if (this._resultWidgets) {
			for (var i = 0; i < this._resultWidgets.length; i++)
				this._resultWidgets[i].destroy();
		}
		this._resultWidgets = [];
		this._bannerSearchArea._resultsBody.innerHTML = "";
		dojo.removeClass(this._bannerSearchArea._statusInfo,"status-error");
		if (hideStatus) {
			this._isResultsShowing = false;
			this._bannerSearchArea._statusArea.style.display = "";
		}
	},
	
	_update: function(/* boolean */ force, /* number */ page, /* boolean */ forceLoading) {
		var onUpdate = this._getProperty("onUpdate");
		if (onUpdate && ((this._currentQuery != this._lastQuery) || force)) {
			this._lastQuery = this._currentQuery;
			if (this._currentQuery && this._currentQuery.length > 0) {
				if (this._bannerSearch && (forceLoading || (!this._isResultsShowing && !this._isLegacyResultsOpen))) {
					this._showLoading();
				}
				/*
				 * Use the sequence counter to make sure we never show an old
				 * stale response when we've already received a more recent one.
				 */
				var callbackObj = {
					_self: this,
					_sequenceCounter: ++this._sequenceCounter,
					_query: this._currentQuery,
					callback: function(/* object */ results) {
						// only show responses newer than the previous; ignore older ones
						if (this._sequenceCounter > this._self._lastResponseReceived) {
							this._self._lastResponseReceived = this._sequenceCounter;
							if (dojo.isArray(results))
								this._self._showResultsLegacy(results, this._self._getProperty("minResultsWidth"));
							else
								this._self._showResults(results, page, "'<strong>" + jazz.util.html.escape(this._query) + "</strong>'");
						}
					}
				};
				if (this._provider) {
					onUpdate({
						query: this._currentQuery,
						scope: this._scope.id,
						advanced: this._isAdvanced,
						page: page || 1,
						maxResults: 10,
						callback: dojo.hitch(callbackObj, callbackObj.callback),
						errback: dojo.hitch(this, this._showError)
					});
				} else {
					onUpdate(this._currentQuery, this._scope.id, dojo.hitch(callbackObj, callbackObj.callback), dojo.hitch(this, this._showErrorLegacy));
				}
			}
			else {
				// ignore any pending searches by syncing up the counters
				this._lastResponseReceived = this._sequenceCounter;
				if (this._bannerSearchArea) {
					// if the banner search area already exists, get rid of any existing results
					this._closeLegacyResults();
					this._clearResults(true);
				}
			}
		}
		else {
			delete this._timeout;
		}
	},
	
	_updateSearchButton: function() {
		var hasText = (this._currentQuery && this._currentQuery.length > 0);
		this._searchButton.className = "SearchButton " + (hasText ? "SearchButtonEnabled" : "SearchButtonDisabled");
	},
	
	_sniffKey: function(e){
		if (e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER) {
			this._openMenu();
			dojo.stopEvent(e);
		}
	},
	
	_openMenu: function(e) {
		if (e){
			dojo.stopEvent(e);
		}
		this._closeLegacyResults();
		if (this._menuOpen) {
			this._closeMenu();
			return;
		}
		
		if (this._menu)
			this._menu.destroyRecursive();
		var menu;
		if (this._bannerSearch) {
			menu = new jazz.ui.MegaMenu();
			var providers = this.providers;
			for (var i = 0; i < providers.length; i++) {
				menu.addTitle(providers[i].name);
				for (var j = 0; j < providers[i].scopes.length; j++) {
					var scope = providers[i].scopes[j];
					var item = menu.addItemByArgs({
						icon: scope.icon,
						label: scope.label,
						onClick: dojo.hitch(this, this._setScope, {scope: scope, focusText: true, provider: providers[i]})
					});
					if (scope === this._scope)
						var selected = item;
				}
				if (selected)
					menu.setSelected(selected);
			}
		} else {
			menu = new jazz.ui.Menu();
			var scopes = this.scopes || this._provider.scopes;
			for (var i=0;i<scopes.length;i++) {
				menu.addChild(this._createMenuItem(scopes[i]));
			}
			dojo.addClass(menu.domNode, "jazz-ui-SearchBox-ScopeMenu");
		}

		dojo.addClass(this._scopeMenu, "ScopeMenu-selected");
		dijit.setWaiState(this._scopeMenu, "expanded", true);
		dijit.popup.open({ popup: menu, parent: this, around: this._scopeMenu, onExecute: dojo.hitch(this,this._closeMenu), onCancel: dojo.hitch(this,this._closeMenu), orient: this._bannerSearch ? {"BR": "TR"} : {"BL": "TL"} });
		if (this._bannerSearch) {
			var func = function() {
				if (menu.domNode) {
					var node = menu.domNode.parentNode;
					if (!node) return;
					var pos = dojo.position(node, true);
					var aroundPos = dojo.position(this._scopeMenu, true);
					var left = Math.max(0, aroundPos.x + aroundPos.w - pos.w);
					node.style.left = left + "px";
				}
			};
			this.connect(menu,"onReflow", func);
			dojo.hitch(this,func)();
		}
		//register event handler that causes menu to close
		this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
		this._menuKeyHandler = this.connect(menu.domNode,"onkeypress",this._onMenuKeyPress);
		this._menu = menu;

		menu.focus();
		this._menuOpen = true;
	},
	
	_createMenuItem: function(scope) {
		var label = "<img class=\"itemIcon\" src=\"" + scope.icon + "\"><span class=\"itemLabel\">" + scope.label + "</span>";
		var hitchArgs = {scope: scope, focusText: true};
		var args = {label: label, onClick: dojo.hitch(this, this._setScope, hitchArgs )};
		if(scope === this._scope) {
			args.iconClass = "selectedItem";
		}
		var item = new dijit.MenuItem(args);
		dijit.setWaiRole(item.domNode, "menuitemradio");
		return item;
	},
	
	_closeMenu: function(e){
		if (e && typeof e == "object") {
			dojo.stopEvent(e);
		}
		if (this._menuOpen) {
			this._menuOpen = false;
			dojo.removeClass(this._scopeMenu, "ScopeMenu-selected");
			dijit.setWaiState(this._scopeMenu, "expanded", false);
			this.disconnect(this._menuCloseOnBody);
			this.disconnect(this._menuKeyHandler);
			if (this._bannerSearch)
				this.disconnect(this._menuBlur);
			dijit.popup.close(this._menu);
		}
	},
	
	_onMenuKeyPress: function(e) {
		if (e.keyCode == dojo.keys.TAB) {
			if (e.shiftKey) {
				// previous item is menu drop down
				this._scopeMenu.focus();
			}
			else {
				// next item is text field
				this._searchText.focus();
			}
		}
	},
	
	_setScope: function(/* object */ args) {
		if (!args.scope) return;
		if (args.provider)
			this._provider = args.provider;
		this._scope = args.scope;
		this._scopeMenu.title = dojo.string.substitute(_MESSAGES.tooltip, [ this._scope.label ]);
		if (this._bannerSearchArea) {
			this._bannerSearchArea.scopeMessage.innerHTML = this._scope.label;
		}
		this._searchText.title = dojo.string.substitute(_MESSAGES.tooltip, [ this._scope.label ])
		if (dojo.hasClass(this._searchForm, "inactive")) {
			// if we are going to focus the box anyway, just clear the value now to avoid flicker
			this._searchText.value = args.focusText ? "" : this._getDefaultText();
		}
		this._menuImg.src = this._scope.icon;
		if (args.focusText) {
			this._searchText.focus();
		}
		this._updateBannerSearchArea(true);
		this._update(true,null,true);
	},
	
	_showBannerSearchArea: function() {
		var initial = false;
		if (!this._bannerSearchArea) {
			dojo.addClass(this.domNode, "jazz-ui-BannerSearchBox-areaInactive");
			this._bannerSearchArea = new net.jazz.web.ui.internal.BannerSearchArea();
			this._bannerSearchArea.scopeMessage.innerHTML = this._scope.label;
			this.connect(this._bannerSearchArea.moreOptions,"onclick",this._toggleMoreOptions);
			dojo.place(this._bannerSearchArea.domNode, this.domNode, "after");
			initial = true;
		}
		this._updateBannerSearchArea(initial);
		
		if (!this._searchAreaBlur)
			this._searchAreaBlur = this.connect(this._bannerSearchArea,"_onBlur",this._hideBannerSearchArea);
		if (!this._searchAreaEscape)
			this._searchAreaEscape = this.connect(dojo.body(), "onkeypress", this._onKeyPress);
		// dijit._Widget._onBlur doesn't always fire properly when clicking out, listen for body clicks
		if (!this._onBodyClick) {
			this._onBodyClick = this.connect(dojo.body(),"onclick",this._doHideBannerSearchArea);
		}
		dojo.removeClass(this.domNode, "jazz-ui-BannerSearchBox-areaInactive");
		dojo.style(this._bannerSearchArea.domNode, "opacity", 0);
		if (this._closeAnim)
			this._closeAnim.stop();
		var animArgs = {
			node: this._bannerSearchArea.domNode,
			duration: 250,
			properties: {
				opacity: {start: 0, end: 1}
			},
			onEnd: dojo.hitch(this,function() {
				dojo.style(this._bannerSearchArea.domNode, "opacity", 1);
				if (dojo.isIE === 8)
					dojo.style(this._bannerSearchArea.scopeText, "opacity", 1);
				this._openAnim = null;
			})
		};
		this._openAnim = dojo.animateProperty(animArgs);
		if (dojo.isIE === 8) {
			// Have to animate the scope message in IE8 because it doens't inherit the opacity for some reason
			dojo.style(this._bannerSearchArea.scopeText, "opacity", 0);
			animArgs.node = this._bannerSearchArea.scopeText;
			this._openAnim = dojo.fx.combine([this._openAnim, dojo.animateProperty(animArgs)]);
		}
		this._openAnim.play(250);
		
		this._bannerSearchArea.domNode.style.display = "";
		this._setResultsAreaSizing(true);
	},
	_updateBannerSearchArea: function(isNewScope) {
		if (!this._bannerSearchArea)
			return;
		
		var hasAdvanced = this._getProperty("createAdvancedOptions");
		var defaultAdvanced = this._getProperty("defaultAdvancedOptions");
		this._bannerSearchArea.moreOptions.style.display = hasAdvanced ? "inline" : "";
		
		if (isNewScope) {
			if (hasAdvanced && this._isAdvanced && (this._deliberateState || defaultAdvanced)) {
				// advanced options is open and defaults to open OR was opened by the user
				this._populateAdvancedOptions();
			} else if ((this._isAdvanced && (!hasAdvanced || (!this._deliberateState && !defaultAdvanced))) ||
					(!this._isAdvanced && hasAdvanced && defaultAdvanced && !this._deliberateState)) {
				// advanced options is open but isn't supported OR is open but defaults to closed OR
				// is closed but defaults to open
				this._toggleMoreOptions();
			}
		}

		var resultsArea = this._getProperty("resultsArea");
		if (resultsArea) {
			dojo.deprecated("jazz.ui._SearchProvider#resultsArea","The Foundation is now responsible for rendering inline results, use the callback provided to jazz.ui._SearchProvider#onUpdate instead");
			if (!this._getProperty("onUpdate")) {
				this._removeChildren(this._bannerSearchArea._resultsAreaWrapper);
				this._bannerSearchArea._resultsAreaWrapper.appendChild(resultsArea);
			}
			if (dojo.isIE === 7)
				dojo.style(this._bannerSearchArea._resultsAreaWrapper,{display:"block"});
		} else if (dojo.isIE === 7) {
			dojo.style(this._bannerSearchArea._resultsAreaWrapper,{display:""});
		}
	},
	_updateResultsAreaHeight: function() {
		var v = dijit.getViewport();
		var p = dojo.position(this._bannerSearchArea._resultsAreaWrapper);
		if (!v.h || !p.y)
			return;
		this._bannerSearchArea._resultsAreaWrapper.style.maxHeight = Math.max(v.h - p.y - 50, 200) + "px";
	},
	_setResultsAreaSizing: function(enabled) {
		if (enabled) {
			this._updateResultsAreaHeight();
			this._resizeNotifierId = jazz.util.ResizeNotifier.register({
				context: this,
				callback: this._updateResultsAreaHeight,
				force: true,
				publishEvents: false
			});
		} else if (this._resizeNotifierId){
			jazz.util.ResizeNotifier.unregister(this._resizeNotifierId);
		}
	},
	
	_onKeyPress: function(e) {
		if(e.type == "keypress" && e.keyCode == dojo.keys.ESCAPE)
			this._doHideBannerSearchArea(null, true);
	},
	_hideBannerSearchArea: function(e) {
		this._hideTimeout = setTimeout(dojo.hitch(this,this._doHideBannerSearchArea),0);
	},
	_doHideBannerSearchArea: function(e, force) {
		var target = (e && e.target) || (dijit.getFocus() && dijit.getFocus().node);
		if (!force && target &&
				(dojo.isDescendant(target, this._bannerSearchArea.domNode) ||
				dojo.isDescendant(target, this.domNode) ||
				(this._menu && dojo.isDescendant(target, this._menu.domNode)) ||
				(this._isAdvanced && this._getProperty("isAdvancedDescendant") && this._getProperty("isAdvancedDescendant")(target)) || 
				(this._isResourceLinkOpen && dojo.isDescendant(target, jazz.ui._getMasterPopup().domNode))))
			return;
		this._setResultsAreaSizing(false);
		clearTimeout(this._hideTimeout);
		delete this._hideTimeout;
		this.disconnect(this._searchAreaBlur);
		delete this._searchAreaBlur;
		this.disconnect(this._searchAreaEscape);
		delete this._searchAreaEscape;
		if (this._onBodyClick) {
			this.disconnect(this._onBodyClick);
			delete this._onBodyClick;
		}
		dojo.style(this._bannerSearchArea.domNode, "opacity", 1);
		if (dojo.isIE === 8)
			dojo.style(this._bannerSearchArea.scopeText, "opacity", 1);
		var onEnd = dojo.hitch(this,function() {
			this._bannerSearchArea.domNode.style.display = "none";
			dojo.style(this._bannerSearchArea.domNode, "opacity", 0);
			if (dojo.isIE === 8)
				dojo.style(this._bannerSearchArea.scopeText, "opacity", 0);
			this._closeAnim = null;
		});
		dojo.addClass(this.domNode, "jazz-ui-BannerSearchBox-areaInactive");
		if (this._openAnim) {
			this._openAnim.stop();
			onEnd();
		} else {
			var animArgs = {
				node: this._bannerSearchArea.domNode,
				duration: 250,
				properties: {
					opacity: {start: 1, end: 0}
				},
				onEnd: onEnd
			};
			this._closeAnim = dojo.animateProperty(animArgs);
			if (dojo.isIE === 8) {
				animArgs.node = this._bannerSearchArea.scopeText;
				this._closeAnim = dojo.fx.combine([this._closeAnim, dojo.animateProperty(animArgs)]);
			}
			this._closeAnim.play();
		}
	},
	_toggleMoreOptions: function(e) {
		if (e) {
			dojo.stopEvent(e);
			this._deliberateState = true;
		} else {
			this._deliberateState = false;
		}
		this._isAdvanced = !this._isAdvanced;
		if (this._isAdvanced) {
			this._bannerSearchArea.moreOptions.innerHTML = _MESSAGES.fewerOptions;
			this._populateAdvancedOptions();
			this._bannerSearchArea._advancedOptions.style.display = "block";
			dojo.addClass(this._bannerSearchArea.moreOptions,"fewer-options");
		} else {
			this._bannerSearchArea.moreOptions.innerHTML = _MESSAGES.moreOptions;
			this._bannerSearchArea._advancedOptions.style.display = "";
			dojo.removeClass(this._bannerSearchArea.moreOptions,"fewer-options");
		}
		this._updateResultsAreaHeight();
	},
	_populateAdvancedOptions: function() {
		this._removeChildren(this._bannerSearchArea._advancedOptions);
		if (this._provider && this._provider.createAdvancedOptions) {
			if (!this._provider.createAdvancedOptions.__domNode)
				this._provider.createAdvancedOptions.__domNode = this._provider.createAdvancedOptions.apply(this._provider);
			this._bannerSearchArea._advancedOptions.appendChild(this._provider.createAdvancedOptions.__domNode);
		} else {
			this._removeChildren(this._bannerSearchArea._advancedOptions);
		}
		if (this._scope && this._scope.createAdvancedOptions) {
			if (!this._scope.createAdvancedOptions.__domNode)
				this._scope.createAdvancedOptions.__domNode = this._scope.createAdvancedOptions.apply(this._provider);
			this._bannerSearchArea._advancedOptions.appendChild(this._scope.createAdvancedOptions.__domNode);
		}
	},
	// safely removes children without destroying child nodes
	// in IE dojo.empty destroys child nodes
	_removeChildren: function(domNode) {
		while(domNode.hasChildNodes())
			domNode.removeChild(domNode.firstChild);
	}
});

/*
 * The drop-down search result widget.
 */
dojo.declare("net.jazz.web.ui.internal.SearchResults",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"net-jazz-web-ui-internal-SearchResults\"></div>",
	
	constructor: function(args) {
		this._predecessor = args.predecessor;
		this._successor = args.successor;
		this._onSelect = args.onSelect;
	},
	
	postCreate: function() {
		this._bgIframe = new dijit.BackgroundIframe(this.domNode);
	},
	
	focus: function() {
		this._menu.focus();
	},
	
	performDefault: function() {
		if (this._defaultHandler) {
			this._defaultHandler();
			return true;
		}
		return false;
	},
	
	setContent: function(results, width) {
		delete this._defaultHandler;
		this._results = results;
		this._width = width;

		// create a new menu
		var menu = new net.jazz.web.ui.internal.CustomMenu({predecessor: this._predecessor, successor: this._successor});
		for (var i=0;i<results.length;i++) {
			this._addItem(menu, results[i], 0);
			if (!this._defaultHandler && results[i].handler) {
				this._defaultHandler = results[i].handler;
			}
		}
		
		// create container
		var content = document.createElement("div");
		content.className = "content";
		content.style.minWidth = (this._width - 2) + "px";
		content.appendChild(menu.domNode);
		
		// swap in new container
		if (this._content) {
			this.domNode.replaceChild(content, this._content);
		}
		else {
			this.domNode.appendChild(content);
		}
		this._content = content;
		
		// destroy old menu
		if (this._menu) {
			this._menu.destroyRecursive();
		}
		this._menu = menu;
	},
	
	_addItem: function(menu, result, indent) {
		var buf = ["<div class=\"entry\" style=\"padding-left: "];
		buf.push((indent*1.25));
		buf.push("em\">");
		buf.push(result.label);
		buf.push("</div>");
		if (result.handler) {
			var handlerObj = {
				_self: this,
				_result: result,
				handle: function(e) {
					this._self._onSelect(e);
					this._result.handler();
				}
			};
			var item = new dijit.MenuItem({label: buf.join(""), onClick: dojo.hitch(handlerObj, handlerObj.handle)});
		}
		else {
			var item = new net.jazz.web.ui.internal.DisabledMenuItem({label: buf.join("")});
		}
		menu.addChild(item);
		
		// add children recursively
		var children = result.children;
		if (children) {
			var childIndent = indent + 1;
			for (var i=0;i<children.length;i++) {
				this._addItem(menu, children[i], childIndent);
			}
		}
	},
	
	destroy: function() {
		this._bgIframe.destroy();
		this.inherited(arguments);
	}
});

/*
 * A specialized menu that handles up/down keys differently than normal.
 * If you press down arrow and you're on the last node, it stays instead of
 * wrapping. If you're on the first node and you press up arrow, it will leave
 * the menu and go to a designated predecessor node.
 */
dojo.declare("net.jazz.web.ui.internal.CustomMenu",
	dijit.Menu, {
	
	/*
	 * Constructs the menu with the given predecessor node that will receive
	 * focus when you press up arrow on the first node.
	 */
	constructor: function(args) {
		this._predecessor = args.predecessor;
		this._successor = args.successor;
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this.connect(this, "onkeypress", this._onKeyPress);
	},
	
	/*
	 * Override to prevent wrapping on the last node.
	 */
	focusNext: function() {
		if (this.focusedChild) {
			var children = this.getChildren();
			if (this.focusedChild == children[children.length-1]) {
				// prevent wrapping
				return;
			}
		}
		this.inherited(arguments);
	},

	/*
	 * Override to jump to predecessor node if on first node.
	 */	
	focusPrev: function() {
		if (this.focusedChild) {
			var children = this.getChildren();
			if (this.focusedChild == children[0]) {
				// jump to the menu's predecessor
				this._onChildBlur(this.focusedChild);
				this.focusedChild = null;
				this._predecessor.focus();
				return;
			}
		}
		this.inherited(arguments);
	},
	
	onItemHover: function(/* MenuItem */ item) {
		dojo.addClass(item.domNode, "dijitMenuItemHover");
	},
	
	onItemUnhover: function(/* MenuItem */ item) {
		dojo.removeClass(item.domNode, "dijitMenuItemHover");
	},
	
	_onKeyPress: function(e) {
		if (e.keyCode == dojo.keys.TAB) {
			if (e.shiftKey) {
				this._predecessor.focus();
			}
			else {
				this._successor.focus();
			}
		}
	}
});

/*
 * A grayed and non-functional menu item. You can't click or highlight it, and
 * up/down arrow keys will skip it.
 */
dojo.declare("net.jazz.web.ui.internal.DisabledMenuItem",
	dijit.MenuItem, {

	/*
	 * Override to disable the menu item.
	 */
	postCreate: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "disabled");
		dojo.setSelectable(this.domNode, false);
	},
	
	/*
	 * Never receives focus.
	 */
	isFocusable: function(){
		return false;
	}
});

dojo.declare("net.jazz.web.ui.internal.BannerSearchArea",[dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("jazz.ui", "templates/BannerSearch.html"),
	messages: _MESSAGES,
	postCreate: function() {
		
		this.scopeMessage = jazz.util.i18n.addMessage(this.scopeText,_MESSAGES.scopeText)[0];
		dojo.addClass(this.scopeMessage,"scope-text");
	}
});

})();