/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.StyledTable");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.util.html");

(function() {

/*
 * Generic descriptions of styles:
 * 		PRIMARY - Dark column headings
 * 		SECONDARY - Light column headings
 * 		COLLAPSABLE - Include a top level header than can be clicked to show/hide the table
 * 		FIXED - Include a top level header that is fixed and can not be clicked
 * 		HEADLESS - Do no include a top level header
 * 
 * Descriptions of the styles in terms of the Jazz common style guide:
 * https://jazz.net/wiki/bin/view/Main/CommonStyle#Tables
 * 		PRIMARY_COLLAPSABLE: SC1 header with T1 column header combination
 * 		PRIMARY_HEADLESS: T1 column header alone
 * 		SECONDARY_COLLAPSABLE: SC1 header with T2 column header combination 
 * 		SECONDARY_HEADLESS: T2 column header alone
 * 		SECONDARY_FIXED: T1 header with T2 column header combination
 */
var TrimStyles = {
	PRIMARY_COLLAPSABLE: 3,
	PRIMARY_HEADLESS: 4,
	SECONDARY_COLLAPSABLE: 5,
	SECONDARY_HEADLESS: 6,
	SECONDARY_FIXED: 7
}

// These styles are deprecated, use the ones above
jazz.ui.styledTable = {
	TrimStyles: {
		GOLD: 0,
		BLUE: 1,
		GREY: 2
	}
};
var OldTrimStyles = jazz.ui.styledTable.TrimStyles;

dojo.declare("jazz.ui.StyledTable", [dijit._Widget, dijit._Templated], {
	// summary:
	//		A simple widget that creates a table with common Jazz web ui styles.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		The StyledTable provides 3 common trim styles used throughout Jazz web UIs: gold, blue
	//		and grey. In addition, all styles support rendering an arrow on the current sort column
	//		as well as tracking the current sorting settings. A callback can be provided to react to
	//		the user's header clicks and ensure the sorting remains in sync with the table. By
	//		default the table renders a border above the columns that matches the trim style in
	//		color, though this can be disabled.
	//
	//		The widget populates the table heading and applies css to style the rows appropriatly.
	//		The consumer is reponsible for actually populating the table by accessing the <tbody>
	//		element through the tbody attribute of the widget (e.g. myTable.tbody.appendChild(myRow)).
	//
	//		Basic CSS is applied to the table to give it the style specified, but there is a well
	//		defined css class on the root of the table to allow consumers to size it according to
	//		needs. This class is jazz-ui-StyledTable. Styles can also be applied to the td and tr
	//		elements under jazz-ui-StyledTable to adjust paddings of the cells etc.
	// columns (required):
	//		An array of objects to represent each of the columns in the table. Must be passed in the
	//		contructor and cannot be changed during the lifecycle of the widget. Each object in the
	//		array supports the following properties:
	//			label (required):
	//				A string or domNode reference to be used as the label for the heading of the table
	//				column. If a string is provided it will be escaped and interpreted as pure text and
	//				all spaces will be converted to "&nbsp;" to prevent wrapping. If a more complex
	//				column heading is required, a reference to a live domNode can be provided instead
	//				and it will be placed in the dom as is. Note that if the column is sortable, the
	//				sort order arrow will still be added to the right of the label.
	//			width (optional):
	//				A string which, if specified, will be applied to the th cell as its width in css
	//				direcly as provided (units must be included).
	//			className (optional):
	//				A string which, if specified, will be added to the table headings as a class to
	//				allow custom styling for the column.
	//			sortable (optional):
	//				A boolean to indicate if the table can be sorted by this column. If true, the
	//				heading of the column will have a hover style to indicate the sortability. Clicks
	//				on sortable columns will update the rendering in the header to indicate the
	//				current sort order. They will also cause the sortBy and sortByAscending attributes
	//				to be updated followed by a call to handleSort.
	// heading (optional):
	//		A string or domNode reference to be used as the top heading for all TrimStyles that use a
	//		heading. If a string is provided, it will be used as innerHTML and if a domNode is provided
	//		it will be placed directly on the header cell. (e.g. trim styles that are not HEADLESS)
	// trim (optional):
	//		An integer representing one of the predefined table styles. Must be one of the constants
	//		defined in jazz.ui.StyledTable.TrimStyles. (default TrimStyles.SECONDARY_HEADLESS)
	// sortBy (optional):
	//		An integer representing the index of the current sort column (based on the array indeces of
	//		the columns array). Can be passed in the constructor to indicate the default sort order,
	//		but after that it becomes a read only parameter to be read from handleSort to react to
	//		changes in the sort order. Note that setting this value in the constructor will not trigger
	//		a call to handleSort. The consumer is responsible for ensuring that initial contents match
	//		the initial sort order.
	// sortByAscending (optional):
	//		A boolean indicating if the current sort is ascending or decending. Will be toggled when the
	//		user clicks on the currently sorted column heading. Like sortBy, this can be set in the
	//		constructor but is then read only for the remainder of the widget's lifecycle. (default true)
	// showHover (optional):
	//		A boolean indicating if table rows should be highlighted on hover.
	// since:
	//		0.7.1

	columns: null,
	
	heading: "",
	trim: TrimStyles.SECONDARY_HEADLESS,
	
	sortBy: null,
	sortByAscending: true,
	
	showHover: false,

	clearBody: function() {
		// summary:
		//		Removes all the contents of the table's body.
		if (dojo.isIE) {
			// replace the tbody and update the reference
			var newBody = document.createElement("tbody");
			newBody.className = this.tbody.className;
			this.domNode.replaceChild(newBody, this.tbody);
			this.tbody = newBody;
		} else {
			this.tbody.innerHTML = "";
		}
	},

	handleSort: function() {
		// summary:
		//		A callback to allow consumers to react to changes in the table sorting.
		// description:
		//		To be overriden by consumers.
		//
		//		Will be called when the user clicks the heading of a sortable column. Before the
		//		function is invoked, the sortBy and sortByAscending variables will be updated so
		//		the consumer can react to the new settings accordingly.
	},

	setTrim: function(trim) {
		// summary:
		//		Updates the trim style of the table.
		// trim:
		//		One of the constants defined in jazz.ui.styledTable.TrimStyles.
		if (this.trim == trim)
			return;
		var styleClass = this._getStyleClass();
		dojo.removeClass(this.tbody, styleClass);
		dojo.removeClass(this._thead, styleClass);
		dojo.removeClass(this._thead, this._getHeaderClass());
		this.trim = trim;
		this._applyStyle();
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/StyledTable.html"),
	
	postCreate: function() {
		if (typeof this.topBorder !== "undefined")
			dojo.deprecated("jazz.ui.StyledTable#topBorder","top border is always rendered");
		this._applyStyle();
		this._sortNodes = [];
		this._globalLineTop.colSpan = this.columns.length;
		this._globalLineBottom.colSpan = this.columns.length;
		this._globalHeader.colSpan = this.columns.length;
		for (var i = 0; i < this.columns.length; i++) {
			var heading = this.columns[i];
			var cellArgs = {className: "headerColumn" + (heading.className ? " " + heading.className : "")};
			if (heading.width) 
				cellArgs.style = { width: heading.width };
			var cell = dojo.create("th",cellArgs,this._headerRow);
			if (typeof heading.label === "string") {
				cell.innerHTML = "<div>" + jazz.util.html.escape(heading.label).replace(/ /g,"&nbsp;") + "</div>";
			} else {
				var wrapper = dojo.create("div",{},cell);
				wrapper.appendChild(heading.label);
			}
			if (heading.sortable) {
				this._sortNodes[i] = cell;
				dojo.addClass(cell,"sortableColumn");
				dojo.create("div",{className: "arrow"},cell);
				this.connect(cell,"onclick",dojo.hitch(this,this._onSort,i));
				if (this.sortBy == i) {
					this._sortNode = cell;
					dojo.addClass(cell, this.sortByAscending ? "sortAscending" : "sortDescending");
				}
			}
			var lineArgs = {className: "headerLine headerColumnLine" + (heading.className ? " " + heading.className : "")};
			if (heading.width)
				lineArgs.style = { width: heading.width };
			dojo.create("th",lineArgs,this._headerLineTop);
			dojo.create("th",lineArgs,this._headerLineBottom);
		}
	},
	
	_onSort: function(index) {
		if (this._sortNode)
			dojo.removeClass(this._sortNode, this.sortByAscending ? "sortAscending" : "sortDescending");
		this._sortNode = this._sortNodes[index];
		if (index == this.sortBy)
			this.sortByAscending = !this.sortByAscending;
		else {
			this.sortBy = index;
			this.sortByAscending = true;
		}
		dojo.addClass(this._sortNode, this.sortByAscending ? "sortAscending" : "sortDescending");
		this.handleSort();
	},
	
	_applyStyle: function() {
		if (this.trim === OldTrimStyles.GREY) {
			dojo.deprecated("jazz.ui.styledTable.TrimStyles.GREY", "use jazz.ui.StyledTable.TrimStyles.SECONDARY_HEADLESS instead");
			this.trim = TrimStyles.SECONDARY_HEADLESS;
		} else if (this.trim === OldTrimStyles.GOLD) {
			dojo.deprecated("jazz.ui.styledTable.TrimStyles.GOLD", "use jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS instead");
			this.trim = TrimStyles.PRIMARY_HEADLESS;
		} else if (this.trim === OldTrimStyles.BLUE) {
			dojo.deprecated("jazz.ui.styledTable.TrimStyles.BLUE", "use jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS instead");
			this.trim = TrimStyles.PRIMARY_HEADLESS;
		}
		var styleClass = this._getStyleClass()
		dojo.addClass(this.tbody, styleClass);
		dojo.addClass(this._thead, styleClass);
		dojo.addClass(this._thead, this._getHeaderClass());
		if (!this._isCollapsable()) {
			this._setCollapsed(false);
		}
	},
	
	_onHeaderClick: function(e) {
		if (this._isCollapsable()) {
			this._toggleCollapsed();
		}
	},
	
	_isCollapsable: function() {
		return this.trim === TrimStyles.SECONDARY_COLLAPSABLE || this.trim === TrimStyles.PRIMARY_COLLAPSABLE;
	},
	
	_toggleCollapsed: function() {
		this._setCollapsed(!dojo.hasClass(this.tbody,"table-collapsed"));
	},
	
	_setCollapsed: function(collapsed) {
		if (collapsed) {
			dojo.addClass(this.tbody,"table-collapsed");
			dojo.addClass(this._thead,"table-collapsed");
		} else {
			dojo.removeClass(this.tbody,"table-collapsed");
			dojo.removeClass(this._thead,"table-collapsed");
		}
	},
	
	_getStyleClass: function() {
		switch (this.trim) {
			case TrimStyles.PRIMARY_COLLAPSABLE:
			case TrimStyles.PRIMARY_HEADLESS:
				return "table-primary";
			case TrimStyles.SECONDARY_COLLAPSABLE:
			case TrimStyles.SECONDARY_HEADLESS:
			case TrimStyles.SECONDARY_FIXED:
				return "table-secondary";
		}
	},
	
	_getHeaderClass: function() {
		switch (this.trim) {
			case TrimStyles.PRIMARY_HEADLESS:
			case TrimStyles.SECONDARY_HEADLESS:
				return "table-headless";
			case TrimStyles.PRIMARY_COLLAPSABLE:
			case TrimStyles.SECONDARY_COLLAPSABLE:
				return "table-collapsable";
			case TrimStyles.SECONDARY_FIXED:
				return "table-fixed";
		}
	},
	
	_setHeadingAttr: function(heading) {
		this.heading = heading;
		if (typeof heading === "string")
			this._globalHeader.innerHTML = heading || "&nbsp;";
		else {
			this._globalHeader.innerHTML = "";
			this._globalHeader.appendChild(heading);
		}
	},
	
	_setShowHoverAttr: function(showHover) {
		this.showHover = showHover;
		if (showHover) {
			dojo.addClass(this.tbody, "table-hover");
		} else {
			dojo.removeClass(this.tbody,"table-hover");
		}
	}

});
jazz.ui.StyledTable.TrimStyles = TrimStyles;

})();
