/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.Tree");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.tree.AbstractItem");

(function() {

var AbstractItem = jazz.ui.tree.AbstractItem;

dojo.declare("jazz.ui.Tree", [dijit._Widget, dijit._Templated], {
	// summary:
	//		A primitive tree widget with fully customizable node structure and functionality.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		The Tree is intended to be subclassed, but it can be used as is also. There are
	//		empty functions that can be overriden to add functionality and change behaviour.
	//		The constructor accepts a single argument. It should be an object containing any
	//		of the following parameters:
	//		
	//			indent: The number of pixels to indent each subsequent tree level's nodes.
	//			selection: Indicates if the tree should track selection, can be true (default)
	//				for single-select, "multi" for multi-select or false for no selection.
	//
	//		Any of the functions intended to be overriden can also be passed as constructor
	//		arguments. For example: new jazz.ui.Tree({selection: "multi", init: myFunction()};
	// since:
	//		0.7
	
	// PUBLIC API
	
	indent: 15,			// The number of pixels to indent each subsequent tree level's nodes.
	selection: true,	// Indicates if the tree should track selection, can be true (default)
						//		for single-select, "multi" for multi-select or false for no selection.
	
	addChild: function(item, index) {
		// summary:
		//		Adds a new item to the root of the tree (e.g. a new root).
		// description:
		//		Adds the provided item as a new tree root. If the index is specified, the item will
		//		be added in the desired position relative to the rest of the roots.
		// item:
		//		A subclass of jazz.ui.tree.AbstractItem that represents the node to be added.
		// index:
		//		The index to insert the item relative to the other roots.
		this._root.addChild(item,index);
		if (!this._focus)
			this._setFocus(item, false);
	},
	
	getChildren: function() {
		// summary:
		//		Returns the root nodes of the tree.
		// description:
		//		Returns an array of jazz.ui.tree.AbstractItem instances that represent the roots of
		//		the tree. The array will be ordered the same as the nodes appear in the tree.
		return this._root.getChildren();
	},
	
	getItem: function(id) {
		// summary:
		//		Returns the indicated tree root.
		// description:
		//		Returns an instance of jazz.ui.tree.AbstractItem with the indicated id. If the
		//		provided id does not correspond to one of the tree roots, the return will be undefined.
		// id:
		//		The id of the desired tree root.
		var b = this._branches[id];
		if (b)
			return b.item;
	},
	
	getSelection: function() {
		// summary:
		//		Returns the current tree selection.
		// description:
		//		Returns an array of jazz.ui.tree.AbstractItem instances representing the current
		//		selection of the tree. If selection is disabled, the return will be undefined. If the
		//		selection is empty, an empty array is returned. If the tree is single select, the array
		//		will be at most length 1.
		if (this.selection) {
			var result = [];
			for (var i in this._selection)
				result.push(this._selection[i]);
			return result;
		}
	},
	
	onDblClick: function(item) {
		// summary:
		//		Called when a tree item is double clicked.
		// description:
		//		Can be overriden to provide double custom double click functionality. The default
		//		implementation toggles the state of non-leaf nodes. To preserve this functionality
		//		when overriding, subclasses must call the superclass.
		// item:
		//		The jazz.ui.tree.AbstractItem that was double clicked.
		if (!item.isLeaf())
			item.toggleExpanded();
	},
	
	onEnter: function() {
		// summary:
		//		Called when enter is pressed while the tree has focus.
		// description:
		//		Can be overriden to provide double custom functionality on enter.
	},
	
	onSelectionChanged: function(selection) {
		// summary:
		//		Called when the tree's selection changes from a user action.
		// description:
		//		Can be overriden to react to selection changes.
		// selection
		//		The new selection of the tree using the same structure as the getSelection method.
	},
	
	refresh: function() {
		// summary:
		//		Refreshes the entire tree.
		// description:
		//		Refreshes all root nodes and all their children.
		this._refreshItem(this._root);
	},
	
	removeChild: function(child, preserve) {
		// summary:
		//		Removes a root from the tree.
		// description:
		//		Removes the indicated root node from the tree.
		// child:
		//		A reference to the jazz.ui.tree.AbstractItem that should be removed. This must be
		//		a valid root of the tree.
		// preserve:
		//		A boolean indicating if the tree item should be preserved. If this is not true,
		//		the removed item will be destroyed. This should be set to true if the item will be
		//		held to add back to the tree later.
		this._root.removeChild(child, preserve);
	},
	
	setSelection: function(item) {
		// summary:
		//		Updates the tree's selection.
		// description:
		//		Sets the tree's current selection to the indicated item. Currently this function does
		//		not support setting the selection to multiple items (for multi select trees) or
		//		clearing the selection.
		// item:
		//		A reference to the jazz.ui.tree.AbstractItem that should be selected.
		if (!item)
			return;
		this._doSelection({}, item, true);
	},
	
	// INTERNAL PROPERTIES AND FUNCTIONS
	
	templateString: "<div class='jazz-ui-Tree'/>",
	
	postCreate: function() {
		this._root = new AbstractItem();
		this._root.tree = this;
		this._branches = [];
		if (this.selection)
			this._selection = {};
		this.connect(this.domNode,"onkeypress",this._handleKeypress);
		
		dijit.setWaiRole(this.domNode, "tree");
		
		if (this.selection === "multi") {
			dijit.setWaiState(this.domNode, "multiselectable", "true");
		} else {
			dijit.setWaiState(this.domNode, "multiselectable", "false");
		}
	},
	
	_setExpanded: function(item, bool){
		var root = this._branches[item.getId()].root;
		if (bool) {
			dojo.removeClass(root, "closed");
			dojo.addClass(root, "open");
			dijit.setWaiState(item.getFocusNode(), "expanded", "true");
		} else {
			dojo.addClass(root, "closed");
			dojo.removeClass(root, "open");
			dijit.setWaiState(item.getFocusNode(), "expanded", "false");
			if (this.selection) {
				var s = this.getSelection();
				for (var i = 0; i < s.length; i++) {
					if (!s[i].isVisible()) {
						this._doSelection({ctrlKey: true}, s[i], true);
						var node = s[i];
						while (!node.isVisible())
							node = node._parent;
						if (!this._selection[node.getId()])
							this._doSelection({ctrlKey: true}, node, true);
					}
				}
			}
		}
	},
	
	_isLeaf: function(item) {
		return item.getChildren().length == 0;
	},

	_refreshItem: function(item) {
		if (item.getId() == this._root.getId()) {
			for (var i = 0; i < item.getChildren().length; i++) {
				this._refreshItem(item.getChildren()[i]);
			}
			return;
		}
		var b = this._branches[item.getId()]
		if (!b) {
			if (item.getFocusNode())
				this.connect(item.getFocusNode(), "onfocus", dojo.hitch(this, this._setFocus, item, false));
			b = {};
			var r = document.createElement("div");
			b.root = r;
			r.className = 'treeBranch closed';
			r.innerHTML = "<div class='row'><a hideFocus='true' class='twistie'></a></div><div class='body'></div>";
			var row = r.firstChild;
			b.row = row;
			b.twist = row.firstChild
			b.body = row.nextSibling;
			row.appendChild(item.domNode);
			this.connect(b.twist, "onclick", function(e) {
				e.preventDefault();
				this._handleTwistie(item);
			});
			// prevent text selection on twistie double click, see work item 66428
			this.connect(b.twist, "onmousedown", function(e) {e.preventDefault(); return false;});
			this.connect(b.twist, "onselectstart", function() {return false;});
			b.item = item;
			this._branches[item.getId()] = b;
			if (item.isRoot()) {
				this.domNode.appendChild(b.root);
			}
		}
		if (this.selection && !b.handler) {
			b.handler = this.connect(item.domNode, "onclick", function(e){
				this._doSelection(e, item);
			});
		}
		if (!b.dblHandler) {
			b.dblHandler = this.connect(item.domNode, "ondblclick", function(e){
				this.onDblClick(item);
			});
		}
		this._updateBranch(b);
		item.update();
		b.row.style.paddingLeft = (item.getDepth() * this.indent) + "px";
		for (var i = 0; i < item.getChildren().length; i++) {
			this._refreshItem(item.getChildren()[i]);
			b.body.appendChild(this._branches[item.getChildren()[i].getId()].root);
		}
	},
	
	_handleTwistie: function(item) {
		item.toggleExpanded();
		if (this.selection && this.getSelection().length == 0) {
			this._setFocus(item, false);
		}
		if (this._focus) {
			var node = this._focus;
			while (!node.isVisible())
				node = node._parent;
			this._focus = node;
		}
		// force browser focus to the twistie after click so the tree will handle key presses
		var twist = this._branches[item.getId()].twist;
		twist.tabIndex = "0";
		twist.focus();
		if (!this._onTwistBlur) {
			this._onTwistBlur = this.connect(twist,"onblur",dojo.hitch(this,function() {
				twist.tabIndex = "-1";
				this.disconnect(this._onTwistBlur);
				delete this._onTwistBlur;
			}));
		}
	},
	
	_doSelection: function(e, item, supressBrowserFocus) {
		var b = this._branches[item.getId()];
		if (this.selection == "multi" && e.ctrlKey) {
			if (this._selection[item.getId()]) {
				dojo.removeClass(b.row, "j-selected");
				dijit.setWaiState(item.getFocusNode(), "selected", "false");	
				delete this._selection[item.getId()];
			} else {
				dojo.addClass(b.row, "j-selected");
				dijit.setWaiState(item.getFocusNode(), "selected", "true");	
				this._selection[item.getId()] = item;
			}
		} else {
			var s = this.getSelection();
			for (var i = 0; i < s.length; i++) {
				dojo.removeClass(this._branches[s[i].getId()].row, "j-selected");
				var currentFocusNode = s[i].getFocusNode();
				dijit.setWaiState(currentFocusNode, "selected", "false");	
				delete this._selection[s[i].getId()];
			}
			this._selection[item.getId()] = item;
			dojo.addClass(b.row, "j-selected");
			dijit.setWaiState(item.getFocusNode(), "selected", "true");
		}
		this._setFocus(b.item, !supressBrowserFocus);
		this.onSelectionChanged(this.getSelection());
	},
	
	_setFocus: function(item, browserFocus) {
		var d = document.createElement("div");
		if (this._focus) {
			this._focus.getFocusNode().setAttribute("tabIndex", "");
		}
		item.getFocusNode().setAttribute("tabIndex", "0");
		if (browserFocus)
			item.getFocusNode().focus();
		this._focus = item;
	},
	
	_handleKeypress: function(e) {
		if (e.altKey)
			return;
		var k = dojo.keys;
		switch (e.charOrCode) {
			case k.ENTER:
				this.onEnter(e);
				break;
			case k.LEFT_ARROW:
				this._onLeft(e);
				break;
			case k.RIGHT_ARROW:
				this._onRight(e);
				break;
			case k.UP_ARROW:
				this._onUp(e);
				break;
			case k.DOWN_ARROW:
				this._onDown(e);
				break;
			case k.HOME:
				this._onHome(e);
				break;
			case k.END:
				this._onEnd(e);
				break;
		}
	},
	_onLeft: function() {
		if (this._focus.isExpanded())
			this._focus.setExpanded(false);
		else {
			if (this._focus.isRoot())
				return;
			this._doKeyboardFocus(this._focus.getParent());
		}
	},
	_onRight: function() {
		if (this._focus.isLeaf())
			return;
		if (!this._focus.isExpanded())
			this._focus.setExpanded(true);
		else
			this._doKeyboardFocus(this._focus.getChildren()[0]);
	},
	_onUp: function(e) {
		dojo.stopEvent(e);
		var prev = this._focus.getPreviousSibling();
		if (prev) {
			if (prev.isExpanded())
				this._findEnd(prev.getChildren())
			else
				this._doKeyboardFocus(prev);
		}
		else if (!this._focus.isRoot())
			this._doKeyboardFocus(this._focus.getParent());
	},
	_onDown: function(e) {
		dojo.stopEvent(e);
		if (this._focus.isExpanded())
			this._doKeyboardFocus(this._focus.getChildren()[0])
		else {
			var n = this._focus.getNextSibling();
			if (n)
				this._doKeyboardFocus(n);
			else
				this._findParentNext(this._focus);
		}
	},
	_onHome: function() {
		var roots = this.getChildren();
		this._doKeyboardFocus(roots[0]);
	},
	_onEnd: function() {
		this._findEnd(this.getChildren());
	},
	_findParentNext: function(item) {
		var p = item.getParent();
		if (!p)
			return;
		var n = p.getNextSibling();
		if (n)
			this._doKeyboardFocus(n);
		else
			this._findParentNext(p);
	},
	_findEnd: function(arr) {
		var last = arr[arr.length-1];
		if (last.isLeaf() || !last.isExpanded())
			this._doKeyboardFocus(last);
		else
			this._findEnd(last.getChildren());
	},
	_doKeyboardFocus: function(item) {
		if (this.selection)
			this._doSelection({}, item)
		else
			this._setFocus(item, true);
	},
	
	_removeItem: function(item) {
		var b = this._branches[item.getId()];
		if (b) {
			if (b.root.parentNode)
				b.root.parentNode.removeChild(b.root);
			var p = item.getParent();
			if (p) {
				b = this._branches[p.getId()];
				if (b)
					this._updateBranch(b);
			}
			item.destroy();
		}
	},
	
	_updateBranch: function(branch) {
		if (branch.item.isLeaf()) {
			dojo.addClass(branch.row,"leaf");
		} else {
			dojo.removeClass(branch.row,"leaf");
		}
	},
	
	destroy: function() {
		this._root.destroy();
		this.inherited(arguments);
	}
});

})();
