/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.WidgetPanel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.string");

dojo.require("jazz.ensemble.internal.WidgetContainer");
dojo.require("jazz.ensemble.internal.LayoutConverter");
dojo.require("jazz.ensemble.internal.LayoutFactory");
dojo.require("jazz.ensemble.internal.Schema");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");

dojo.requireLocalization("jazz.ensemble.internal", "Messages");

(function() {

var LayoutConverter = jazz.ensemble.internal.LayoutConverter;
var LayoutFactory = jazz.ensemble.internal.LayoutFactory;
var Schema = jazz.ensemble.internal.Schema;
var WidgetContainer = jazz.ensemble.internal.WidgetContainer;
var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

var messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "Messages");

/**
 * A container that hosts widgets. It must have a layout set, which determines
 * how the widgets are arranged within it.
 * 
 * @since 3.0
 */
dojo.declare("jazz.ensemble.WidgetPanel", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/templates/WidgetPanel.html"),
	
	/**
	 * {boolean} Whether or not to allow widgets to be maximized. This should only
	 * be true when there is enough real estate allocated for the widget panel to
	 * make maximized widgets useful.
	 */
	allowMaximize: true,
	
	/**
	 * {function} An optional listener callback that, if specified, will be
	 * called when the widget panel is modified in a way that changes its state.
	 */
	onStateChange: function() {},
	
	/**
	 * {object} An optional object containing arbitrary widget type-specific
	 * properties, used to attach extra information to this widget panel that
	 * can be read by the widget implementations. Not persisted.
	 */
	properties: {},
	
	/**
	 * {boolean} Whether or not the widget panel is read-only (cannot be modified
	 * by the user).
	 */
	readOnly: false,
	
	/**
	 * {boolean} Whether or not the widget panel's state is shared among multiple
	 * users. Widgets may use this information to decide whether or not to allow
	 * user-specific personalization of the state.
	 */
	shared: true,
	
	/**
	 * {Element} An XML DOM element containing the state of the widget panel to
	 * display.
	 */
	state: null,
	
	/**
	 * {string} An opaque unique id for the WidgetPanel. This will be used to construct
	 * a universally unique id for each widget in the panel.
	 * 
	 * @since 3.0.1.1
	 */
	uuid: null,
	
	// internal variables
	_layout: null,
	_started: false,
	_state: null,
	_widgetContainers: [],
	
	postCreate: function() {
		if (this.state) {
			this.setState(this.state);
		}
	},
	
	/**
	 * Add a widget to the panel, given an initial widget state.
	 * 
	 * @param {object} state an object describing the widget to add
	 * @param {string} (Optional) the id of the widget to insert before
	 */
	addWidget: function(/** Element */ widgetElem, /** string? */ refId) {
		var widgetContainer = this._addWidget.apply(this,arguments);

		// make the widget short and hidden for fade effect
		widgetContainer.domNode.style.height = "1px";
		dojo.style(widgetContainer.domNode,{
			opacity: 0
		});
		
		// setup fade
		var args = {node: widgetContainer.domNode, duration: 200};
		var animation = dojo.fx.chain([dojo.fx.wipeIn(args), dojo.fadeIn(args)]);
		var fadeEvent = dojo.connect(animation,"onEnd",widgetContainer,function() {
			dojo.disconnect(fadeEvent);
			var s = this.domNode.style;
			s.overflow = "";
			s.opacity = "";
			if (s.removeAttribute) {
				// workaround IE cleartype bug.
				s.removeAttribute("filter");
			}
		});
		
		this._stateChanged();
		animation.play();
	},
	
	/**
	 * Creates a new widget element to be configured and subsequently added to
	 * the widget panel via addWidget().
	 */
	/** Element */ createWidget: function() {
		var widgetsCollection = this._state.getWidgets();
		var widget = widgetsCollection.createWidget();
		return widget.elem;
	},
	
	/**
	 * Destroys the panel, the layout, and all widgets in it.
	 */
	uninitialize: function() {
		if (this._layout) {
			this._layout.destroy();
		}
		for (var i=0;i<this._widgetContainers.length;i++) {
			this._widgetContainers[i].destroy();
		}
	},
	
	/**
	 * Returns true if the widget panel has no widgets.
	 * 
	 * @return {boolean} whether or not the panel has no widgets
	 */
	/** boolean */ isEmpty: function() {
		return (this._widgetContainers.length == 0);
	},
	
	/**
	 * To be called when the WidgetPanel is shown (becomes visible) on the page.
	 * @see _Widget#onShow
	 */
	onShow: function() {
		for (var i=0;i<this._widgetContainers.length;i++) {
			this._widgetContainers[i].onShow();
		}
	},
	
	/**
	 * To be called when the WidgetPanel is hidden, e.g. on a background tab.
	 * @see _Widget#onHide
	 */
	onHide: function() {
		for (var i=0;i<this._widgetContainers.length;i++) {
			this._widgetContainers[i].onHide();
		}
	},
	
	/**
	 * Refreshes the widget panel by refreshing all of its widgets.
	 */
	refresh: function() {
		for (var i=0;i<this._widgetContainers.length;i++) {
			this._widgetContainers[i].refresh();
		}
	},
	
	/**
	 * Removes the widget with the given id from the panel.
	 * 
	 * @param {string} the id (rdf:nodeID) of the widget to remove
	 */
	removeWidget: function(/** string */ id) {
		for (var i=0;i<this._widgetContainers.length;i++) {
			var widgetContainer = this._widgetContainers[i];
			if (widgetContainer.widgetId == id) {
				this._removeWidgetContainer(widgetContainer);
				break;
			}
		}
	},
	
	/**
	 * Sets a new layout on the panel, which determines how the widgets
	 * are arranged. Replaces any existing layout.
	 * 
	 * @param {string} type the dc:type of layout to set
	 */
	setLayout: function(/** string */ type) {
		var layoutState = this._state.getLayout().getLayout();
		var oldType = layoutState.getType();
		if (type != oldType) {
			layoutState.setType(type);
			var layoutDataElem = layoutState.getData().getLayoutData();
			LayoutConverter.convert(oldType, type, layoutDataElem);
			var oldLayout = this._layout;
			this._setLayout(this._createLayout(layoutState));
			if (oldLayout) {
				oldLayout.destroy();
			}
			this._stateChanged();
		}
	},
	
	/**
	 * Sets a new state for the widget panel, which includes the widgets
	 * and the layout. This replaces and discards all current content.
	 * 
	 * @param {Element} state the XML DOM element describing the state of the widget panel
	 */
	setState: function(/** Element */ state) {
		this.state = state;
		
		// work with XML via binding objects
		this._state = XmlBindingFactory.createBinding(state, Schema.WIDGET_PANEL);
		
		// destroy old content
		for (var i=0;i<this._widgetContainers.length;i++) {
			this._widgetContainers[i].destroy();
		}
		
		// create and add new content
		this._widgetContainers = [];
		var widgetsCollection = this._state.getWidgets();
		var widgetStates = widgetsCollection.getWidgets();
		for (var i=0;i<widgetStates.length;i++) {
			var widgetState = widgetStates[i];
			var id = widgetState.getId();
			this._widgetContainers.push(this._createWidgetContainer(widgetState));
		}
		var layoutProp = this._state.getLayout();
		this._setLayout(this._createLayout(layoutProp.getLayout()));
		
		/**
		 * since widgets are being added in bulk call their 
		 * startup method after they are all added
		 * this could be added to the layout contract
		 */
		dojo.forEach(this._widgetContainers, function(w) {w.startup();});
	},
	
	/**
	 * Initializes the panel. Must be called immediately after it
	 * is added to the live DOM.
	 */
	startup: function() {
		this.inherited(arguments);
		this._started = true;
		if (this._layout) {
			this._layout.startup();
		}
	},
	
	/*--------------------------------*/
	/* Internal methods, do not call. */
	/*--------------------------------*/
	
	/*
	 * Adds a widget to the panel by XML state object
	 */
	_addWidget: function(/** Element */ widgetElem, /** string|node? */ ref, /** string|number? */pos){
		// create widget container from state
		var widgetState = XmlBindingFactory.createBinding(widgetElem, Schema.WIDGET, this._state.indent + "\t");
		widgetState.setId(this._generateWidgetId());
		var widgetContainer = this._createWidgetContainer(widgetState);
		// add widget to layout
		this._addWidgetContainer(widgetContainer, ref, pos);
		return widgetContainer;
	},
	
	/*
	 * Adds the given widget container to the panel.
	 */
	_addWidgetContainer: function(/* WidgetContainer */ widgetContainer, /* string|node? */ ref, /* string|number? */ pos) {
		this._widgetContainers.push(widgetContainer);
		this._layout.add(this._itemizeWidget(widgetContainer), ref, pos);
		widgetContainer.startup();

		// update state
		this._updateWidgets();
	},
	
	/*
	 * Creates a layout initialized from the given state.
	 */
	_createLayout: function(/* object */ state) {
		var items = [];
		for (var i=0;i<this._widgetContainers.length;i++) {
			items.push(this._itemizeWidget(this._widgetContainers[i]));
		}
		var layout = LayoutFactory.createLayout(state, items, this.readOnly);
		layout.onStateChange = dojo.hitch(this, this._stateChanged);
		layout.onAddWidget = dojo.hitch(this, this._onAddWidget);
		layout.onRemoveWidget = dojo.hitch(this, this._onRemoveWidget);
		return layout;
	},
	
	/*
	 * Creates a widget container initialized from the given state.
	 */
	_createWidgetContainer: function(/* object */ state) {
		var widgetContainer = new WidgetContainer({
			readOnly: this.readOnly,
			shared: this.shared,
			state: state,
			onClose: dojo.hitch(this, this._onWidgetClose, state.getId()),
			onStateChange: dojo.hitch(this, this._stateChanged),
			properties: this.properties,
			panelId: this.uuid
		});
		if (this.allowMaximize) {
			widgetContainer.setMaximizeContainerHandler(dojo.hitch(this, this._handleMaximize, widgetContainer));
		}
		if (!this.readOnly) {
			widgetContainer.setDuplicateHandler(dojo.hitch(this, this._handleDuplicateWidget, widgetContainer));
		}
		return widgetContainer;
	},
	
	/*
	 * Returns a new widget id unique to this panel.
	 */
	/* string */ _generateWidgetId: function() {
		// find highest integer id + 1
		var max = 0;
		for (var i=0;i<this._widgetContainers.length;i++) {
			try {
				var widgetId = parseInt(this._widgetContainers[i].widgetId.substring(1));
				if (widgetId > max) {
					max = widgetId;
				}
			}
			catch (/* Error */ e) {
				// not an integer; skip
			}
		}
		return "_" + (max + 1);
	},
	
	_handleDuplicateWidget: function(/* WidgetContainer */ widgetContainer) {
		var clonedElem = widgetContainer.state.elem.cloneNode(true);
		this.addWidget(clonedElem, widgetContainer.widgetId);
	},
	
	_handleMaximize: function(/* WidgetContainer */ widgetContainer, /* boolean */ maximize) {
		this._layout.setMaximized(widgetContainer.widgetId, maximize);
	},
	
	_itemizeWidget: function(widget) {
		return {
			id: widget.widgetId,
			domNode: widget.domNode,
			handle: widget._headerText
		};
	},
	
	/*
	 * Called when the user clicks to close a widget. The panel is in
	 * charge of closing/removing.
	 */
	_onWidgetClose: function(/* string */ id) {
		for (var i=0;i<this._widgetContainers.length;i++) {
			var widgetContainer = this._widgetContainers[i];
			if (widgetContainer.widgetId == id) {
				var widgetTitle = widgetContainer.primaryTitle || messages.undefinedTitle;
				if (confirm(dojo.string.substitute(messages.confirmDeleteWidget, [widgetTitle]))) {
					var args = {
						node: widgetContainer.domNode,
						duration: 250
					};
					var animation = dojo.fx.chain([dojo.fadeOut(args), dojo.fx.wipeOut(args)]);
					var fadeEvent = dojo.connect(animation, "onEnd", this, function(){
						dojo.disconnect(fadeEvent);
						this._removeWidgetContainer(widgetContainer);
						widgetContainer.destroy();
						widgetContainer = null;
						this._stateChanged();
					});
					animation.play();
				}
				break;
			}
		}
		// veto styledbox close
		return false;
	},
	
	/*
	 * Removes the widget container from the panel.
	 */
	_removeWidgetContainer: function(/* WidgetContainer */ widgetContainer) {
		var index = dojo.indexOf(this._widgetContainers, widgetContainer);
		this._widgetContainers.splice(index, 1);
		
		// update layout and state
		this._layout.remove(widgetContainer.widgetId);
		this._updateWidgets();
	},
	
	/*
	 * Called when a widget is removed by the layout
	 */
	_onRemoveWidget: function(container) {
		this._removeWidgetContainer(container);
		container.destroy();
		this._stateChanged();
	},
	
	/*
	 * Called when a widget is added by the layout.
	 */
	_onAddWidget: function() {
		this._addWidget.apply(this, arguments);
		this._stateChanged();
	},
	
	/*
	 * Sets the given layout on the panel, replacing any existing layout
	 * (but not destroying it).
	 */
	_setLayout: function(/* Layout */ layout) {
		// remove old layout
		var oldLayout = this._layout;
		if (oldLayout) {
			oldLayout.destroy();
			oldLayout = null;
		}
		// add and start new layout
		dojo.place(layout.domNode, this.domNode, "first");
		if (this._started) {
			layout.startup();
		}
		this._layout = layout;
	},
	
	/*
	 * Fires the onStateChange event handler.
	 */
	_stateChanged: function() {
		this.onStateChange && _callContributedCode(this.onStateChange);
	},
	
	/*
	 * Updates the list of widgets in this panel; should be called
	 * when one is added or removed.
	 */
	_updateWidgets: function() {
		var widgets = [];
		var arr = [];
		for (var i=0;i<this._widgetContainers.length;i++) {
			widgets[i] = this._widgetContainers[i].state;
			arr[i] = this._widgetContainers[i].domNode;
		}
		// update widget collection
		this._state.getWidgets().setWidgets(widgets);
	}
});

/* private static */

/*
 * Safely calls a contributed function, and reports any errors.
 */
function _callContributedCode(/* function */ func) {
	try {
		func();
	}
	catch (/* Error */ e) {
		console.error(e);
	}
}

})();
