/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app._ComponentMenu");

dojo.require("jazz.ui.MegaMenu");
dojo.require("jazz.ui.internal.MegaMenu"); //temporary require to support use of internal wrapHint constants for now.
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("jazz.app.internal", "ComponentMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("jazz.app.internal", "ComponentMenuMessages"); //$NON-NLS-1$ //$NON-NLS-2$
/**
 * @class A base-class to use for implementing component menus. 
 * 
 * In addition to add, remove and clear, convenience functions are provided
 * for adding various types of menu items such as dijit.MenuItems (addItem), 
 * titles, separators, and loading.
 * 
 */
dojo.declare("jazz.app._ComponentMenu", [dijit._Widget, dijit._Templated, dijit._Container], {
	
	/**
	 * Determines whether a filter box should be shown at the top of the component menu.
	 * It searches the text values of all menu items and hides the ones that don't match.
	 */
	showFilter: false,
	
	/**
	 * To be implemented by subclasses to point out the currently selected item.
	 * Is called everytime the menu is opened.
	 * @return {dijit.MenuItem|index}
	 */
	getSelected: function() { },
	
	/**
	 * Call this to force the menu to re-validate the currently selected item. 
	 */
	updateSelected: function() {
		this._menu.setSelected(this.getSelected());
	},
	
	/**
	 * Adds menu items, strings or domNodes to the component menu. Items must be compatible with dijit.Menu.
	 * @param {dijit.MenuItem|string|domNode} item
	 * @param {Number|object} [insertIndex|args]  args: insertIndex, wrapHint
	 */
	add: function(item, args) {
		return this._menu.add(item, args);
	},

	/**
	 * Removes a menu item from the component menu.
	 * @param {Object|Number} item
	 */
	remove: function(/*index or item*/ item) {
		this._menu.removeChild(item);
		return item;
	},
	
	/**
	 * Adds a title menu item to the menu.
	 * 
	 * @param {String} title
	 * @param {Number|object} [insertIndex|args] args: insertIndex, wrapHint
	 */
	addTitle: function(title, args) {
		return this._menu.addTitle(title, args);
	},

	/**
	 * Adds a menu item to the menu by args.  Supports extra stuff that vanilla dijit.MenuItems do not (i.e. href and icon).
	 * @param {Object} args - Accepted values: 	icon (path to icon)
	 * 											iconClass (css class for iconNode) 
	 * 											href (href to visit on click)
	 * 											onClick
	 * 											label
	 * 											id
	 * 											wrapHint (See jazz.ui.MegaMenu.WrapHints for constants - NO_WRAP, AUTO_WRAP, FORCE_WRAP)
	 * 											title
	 * @param {Number} [insertIndex]
	 */
	addItemByArgs: function(args, insertIndex) {
		return this._menu.addItemByArgs(args, insertIndex);
	},
	
	/**
	 * Adds a loading message at the specified index or at end of menu.
	 * @param {Number} [insertIndex]
	 */
	addLoading: function(insertIndex) {
		var loading = this._menu.add(messages.loading, insertIndex);
		dojo.addClass(loading.domNode, "jazz-ui-MegaMenu-loading"); 
		return loading;
	},
	
	/**
	 * Adds a separator at the specified index or at end of menu.
	 * @param {Object} [insertIndex]
	 */
	addSeparator: function(insertIndex){
		return this._menu.addSeparator(insertIndex);
	},

	/**
	 * Gets all of the items in the menu
	 */
	getItems: function() {
		return this._menu.getChildren();
	},

	/** 	
	 * Clears the menu.
	 */
	clear: function() {
		this._menu.destroyDescendants();
	},
	
	/**
	 * Closes the menu
	 */	
	close: function() {
		this.onCancel();
	},
	
	/**
	 * Called to determine whether it's safe to close the menu.
	 * 
	 * @return boolean
	 */
	canDismiss: function() {
		return this._menu.canDismiss();
	},
	
	/**
	 * Called after a reflow happens.  Outer functions should connect to this.
	 */
	onReflow: function() { },
	
	/* Internal */
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("jazz.app","internal/templates/_ComponentMenu.html"),
	buildRendering: function() {
		this.inherited(arguments);
		this.connect(this._menu,"onExecute",dojo.hitch(this,function() {
			this.onExecute();
		}));
		this.connect(this._menu,"onCancel",dojo.hitch(this,function() {
			this.onCancel();
		}));
		this._menu.onReflow = dojo.hitch(this,function() {
			this.onReflow();
		});
	},
	onOpen: function() {
		this._menu.setSelected(this.getSelected());
		this._menu.onOpen();
	},
	focus: function() {
		this._menu.focus();
	},
	onClose: function() {
		this._menu.onClose();
	},
	onExecute: function() {
		//stub for onExecute connect (see dijit.popup API)
	},
	onCancel: function(closeAll) { 
		//stub for onCancel connect (see dijit.popup API)
	},
	_setShowFilterAttr: function(showFilter) {
		this._menu.attr("showFilter", showFilter);
	},
	_queueReflow: function() {
		this._menu._queueReflow();
	}
});

})();