/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.ui._SearchProvider"); //$NON-NLS-1$

dojo.require("dijit._Widget");

/**
 * Abstract base class used to define a provider for the {@link jazz.ui.SearchBox}
 * @name jazz.ui._SearchProvider
 * @class <p>An abstract collection of arguments for a jazz.ui.SearchBox. This can be used to define
 *	the behaviour of a SearchBox while leaving its instantiation and lifecycle up to a
 *	third party. Provides a number of parameters that can be overriden for the component-
 *	specific behaviors, such as updating the results, performing the search, search
 *	scopes, etc. There are two modes of searching:</p>
 *	<ol>
 *	  <li>Search as you type - A results drop down will appear below the search
 *    results and will be updated as you type. This feature is enabled by supplying
 *    an onUpdate function in the constructor.</li>
 *    <li>Traditional - Type in the query and press enter or click on the search
 *    button. Enabled by providing an onExecute function in the constructor.</li>
 *	</ol>
 *  <p>At least one of onUpdate and onExecute must be supplied. Both are allowed
 *  simultaneously.</p>
 * 	<p>PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * 	future with minimal or no warning.</p>
 * @extends dijit._Widget
 * @abstract
 */ 
dojo.declare("jazz.ui._SearchProvider",[dijit._Widget],/** @lends jazz.ui._SearchProvider.prototype */{
	/** @private */
	postCreate: function() {
		if (this.delay) {
			dojo.deprecated("jazz.ui._SearchProvider.delay","This parameter is no longer supported.")
		}
	},
	
	/**
	 * An optional string giving the heading name to use for this provider's
	 * scopes in the search scope mega menu. If not specified, will default to the name
	 * of the page that registered the provider.
	 * @field
	 * @type String
	 */
	name: null,
	
	/**
	 * An array of objects containing information about each scope type
	 * to provide for the search box. Each object must have 3 properties:
	 * <ol>
	 * 	<li>String id: A unique string representing the scope</li>
	 * 	<li>String label: A human-readable label for the scope</li>
	 * 	<li>String icon: A path to a 16x16 image for the scope icon</li>
	 * </ol>
	 * <p>The provider's scopes can be dynamically updated after construction by
	 * calling the get method inherited from dijit._Widget ({@link dijit._Widget#get}).</p>
	 * @field
	 * @type Object[]
	 * @example
	 * 	[
	 * 		{
	 * 			id: "scope1",
	 * 			label: "First Scope",
	 * 			icon: dojo.moduleUrl("my.module", "images/scope1.gif")
	 * 		},
	 * 		{
	 * 			id: "scope2",
	 * 			label: "Second Scope",
	 * 			icon: dojo.moduleUrl("my.module", "images/scope2.gif")
	 * 		}
	 * 	]
	 */
	scopes: null,
	
	/**
	 * <p>An optional function that, if specified, will be called when the
	 * search results drop-down (search as you type) needs to be updated.
	 * The function will be called with 7 parameters.</p>
	 * <p>One of onUpdate or onExecute must be specified.</p>
	 * @field
	 * @type Function
	 * @param {String} query The text to search for
	 * @param {String} scope The id of the currently selected scope (see {@link jazz.ui._SearchProvider#scopes})
	 * @param {Boolean} advanced A flag indicating if the advanced search options are enabled.
	 *  	If this is true, the consumer should honor the user's settings in the advanced
	 *  	search options.
	 * @param {Number} page The page number to return results for. For providers that do not support
	 * 		paging this will always be 1.
	 * @param {Number} maxResults The maximum number of results to return. For providers that support
	 * 		paging, this should be used as the page size.
	 * @param {Function} callback The callback function for returning the search
	 * 		results. The function takes one parameter, an object containing a 'results' array
	 * 		representing the search results, a 'paging' boolean indicating if the provider
	 * 		supports paging, and a 'totalResults' integer giving the total number of results the query
	 * 		returned. Each result in the results array must be an object containing, a 'label'
	 * 		string containing the text to show for the result link, an 'icon' string (optional) which
	 * 		gives the path to the icon for the result, a 'description' string (optional) which provides
	 * 		extra details about the result below its link, and either a 'uri' string or an 'onClick'
	 * 		function that provides the click handling behaviour of the result. An optional 'presentationUri'
	 * 		can also be provided and will replace the 'uri' as the href on the link. In this case the 'uri'
	 * 		will only be used as an argument to the ResourceLink to fetch the compact rendering for the
	 * 		resource.
	 * 		e.g.: 
	 * 			{
	 * 				paging: false,
	 * 				totalResults: 2,
	 * 				results: [
	 * 					{
	 * 						label: "First result",
	 * 						icon: "http://sample.com/myicon.png",
	 * 						description: "This is the first result.",
	 * 						uri: "http://sample.com/firstResult",
	 * 						presentationUri: "http://sample.com/presentation/firstResult"
	 * 					},
	 * 					{
	 * 						label: "Second result",
	 * 						icon: "http://sample.com/myothericon.png",
	 * 						onclick: function() {...}
	 * 					}
	 * 				]
	 * 			}
	 * @param {Function} errback The callback to use for reporting errors.
	 * 		Accepts one string parameter containing the error message.
	 */
	onUpdate: null,
	
	/**
	 * <p>An optional function that, if specified, will be called when the user
	 * clicks on the search button or presses enter. The function will be called with 3 parameters.</p>
	 * <p>One of onUpdate or onExecute must be specified. If not specified, onUpdate will be called instead.
	 * Consumers that wish to use onUpdate behaviour for certain calls to onExectue can return true
	 * from the function to trigger this behaviour.</p>
	 * @field
	 * @type Function
	 * @param {String} query The text to search for
	 * @param {String} scope The id of the currently selected scope (see {@link jazz.ui._SearchProvider#scopes})
	 * @param {Boolean} advanced A flag indicating if the advanced search options are enabled.
	 *  	If this is true, the consumer should honor the user's settings in the advanced
	 *  	search options.
	 * @return {Boolean} if true, the call to onExecute will be ignored and the onUpdate API will be used instead.
	 */
	onExecute: null,
	
	/**
	 * The tooltip text to show for the search field on mouse hover.
	 * @field
	 * @type String
	 */
	tooltip: null,
	
	/**
	 * Optional default text to show in the search box when empty. e.g.
	 * "Type to search".
	 * @field
	 * @type String
	 */
	defaultText: null,
	
	/**
	 * An optional HTML-formatted label (string) to show in the update results
	 * when no results were found. e.g. "No results found". Optionally, ${0}
	 * can be included in this string to have the search framework inject the user's
	 * search term in quotes and bold. e.g. "No results found for ${0}" will become
	 * "No results found for '<strong>My Search</strong>'" when the user searches for
	 * "My Search".
	 * @field
	 * @type String
	 */
	noResultsLabel: null,
	
	/**
	 * An optional minimum width in pixels for the drop down results
	 * shown as you type.
	 * @field
	 * @type Number
	 */
	minResultsWidth: null,
	
	/**
	 * An optional domNode to be placed in the search area that will be used
	 * to show the results. It will be appended to the search area immediately and will
	 * always be visible. Consumers are responsible for controlling the content and hiding
	 * it when necessary.
	 * @field
	 * @type DomNode
	 */
	resultsArea: null,
	
	/**
	 * An optional function that must return a domNode containing advanced search
	 * options. If defined, the function will be called the first time the user enters
	 * advanced search mode. From then on, the dom will be maintained and the consumer is
	 * responsible for honoring the users selections in the advanced search area.
	 * @field
	 * @type Function
	 */
	createAdvancedOptions: null,
	
	/**
	 * Indicates if the advanced options panel should be open by default. Defaults
	 * to false.
	 * @field
	 * @type Boolean
	 */
	defaultAdvancedOptions: false,
	
	/**
	 * A function that accepts one parameter, a domNode reference. This function
	 * should return true if the provided domNode is a descendant of a control related to the
	 * provider's advanced options area and false otherwise. Note that this function does not
	 * need to take into account controls that are direct descendants of the advanced area
	 * domNode itself as this is handled by the framework. This API is used for situations
	 * where the advanced search area spawns controls that are not direct dom descendants of
	 * the quick search area (like dialogs, popup menus, etc.).
	 * @field
	 * @type Function
	 */
	isAdvancedDescendant: null,
	
	// no-op, SearchBox listens for these calls
	_setScopesAttr: function(scopes) {
		this.scopes = scopes;
	}
});

})();
