/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.auth");

dojo.require("jazz.app.events");
dojo.require("jazz.app.internal.AuthenticationInterceptor");
dojo.require("jazz.app.internal.HeadlessCredentialProvider");
dojo.require("jazz.app.internal.LoginDialog");
dojo.require("jazz.app.internal.OAuthResponseInterceptor");
dojo.require("net.jazz.ajax.ui.internal.OpenSocialJazz");
dojo.require("jazz.client.xhr");

var loginDialog = new jazz.app.internal.LoginDialog();
jazz.app.internal.AuthenticationInterceptor.setCredentialProvider(loginDialog);

// Apply Jazz authentication logic to responses received through jazz.client.xhr
jazz.client.internal.addResponseInterceptor(jazz.app.internal.AuthenticationInterceptor);

if (!dojo.getObject("gadgets.io.makeRequest"))
	jazz.client.internal.addResponseInterceptor(jazz.app.internal.OAuthResponseInterceptor);
else {
	jazz.client.internal.addResponseInterceptor(net.jazz.ajax.ui.internal.OpenSocialJazz);
	jazz.client.internal.addRequestInterceptor(net.jazz.ajax.ui.internal.OpenSocialJazz.RequestInterceptor);
}

/**
 * Attempts to log in to the local server using either the supplied user ID and
 * password, or by prompting the user for credentials. Note: This function will
 * first call the log out method before attempting to log in.
 * </p><p>
 * This function takes two forms:
 * </p><ul>
 * <li>Zero argument form: Prompts the user to enter credentials.</li>
 * <li>One argument form: If the single argument is an object containing a
 *   property 'userId' with a string value and a property 'password' with a
 *   string value, attempts to log in with those credentials. If either of these
 *   properties are missing or contain a non-string value, uses the zero
 *   argument behavior (prompts for credentials).</li>
 * </ul><p>
 * On a successful log in, the system calls the Deferred's callback chain with
 * an unspecified return type; clients may only assume that the log in attempt
 * succeeded based on the fact that the callback chain was called. Note that a
 * successful log in also fires the jazz.app.events.LOGIN event.
 * </p><p>
 * On an unsuccessful log in attempt, the system calls the Deferred's errback
 * chain with an Error object containing information about the error that
 * occurred.
 * </p><p>
 * Note: The system will automatically challenge a user for credentials as long
 * as some client has dojo.require'd this module. It is unnecessary to manually
 * call log in unless you want to invoke an explicit log in (e.g. when a user
 * clicks on a "Log in" command).
 * </p>
 * 
 * @return {dojo.Deferred} a dojo.Deferred object indicating success or failure
 *  of the log out attempt.
 * @since 0.7
 */
jazz.app.auth.login = function(args) {
	
	// ignore auth events so we don't fire USER_CHANGE after the intermediate
	// logout step
	monitorAuthEvents = false;
	
	var logoutDeferred = jazz.app.auth.logout();
	
	var clientDeferred = new dojo.Deferred(function(deferred) {
		logoutDeferred.cancel();
	});
	
	logoutDeferred.addErrback(function(logoutError) {
		// begin monitoring auth events once again
		monitorAuthEvents = true;
		clientDeferred.errback(logoutError);
	});
	
	logoutDeferred.addCallback(function(logoutResult) {
		if(dojo.isObject(args) && args.userId && args.password) {
			// if the user specified credentials, switch the auth interceptor to use
			// those credentials rather than prompting the user
			var headlessCredentialProvider = new jazz.app.internal.HeadlessCredentialProvider(args.userId, args.password);
			jazz.app.internal.AuthenticationInterceptor.setCredentialProvider(headlessCredentialProvider);
		}
		
		var dfd = jazz.client.xhrPost({
			// TODO: Explore using a standard URI to force log in
			url: net.jazz.ajax._contextRoot + "/web/loggedInUser"
		});
		
		dfd.addBoth(function(result) {
			if(headlessCredentialProvider) {
				// only use a headless provider once
				jazz.app.internal.AuthenticationInterceptor.setCredentialProvider(loginDialog);
			}
			checkedLoggedInUser(result);
			// begin monitoring auth events once again
			monitorAuthEvents = true;
			return result;
		});
		
		dfd.addCallback(function(result) {
			// hide response body
			clientDeferred.callback(null);
		});
		
		dfd.addErrback(function(loginError) {
			clientDeferred.errback(loginError);
		});
	});
	
	return clientDeferred;
};

/**
 * Causes a log in challenge to occur if the user is not already logged in. This
 * is useful because jazz.app.auth.login always first calls jazz.app.auth.logout
 * which may not always be desirable to the caller.
 * 
 * @return {dojo.Deferred} a dojo.Deferred object indicating success or failure
 *  of the log in attempt.
 * @since 0.7
 */
jazz.app.auth.loginIfNecessary = function() {
	var loginDeferred = jazz.client.xhrPost({
		// TODO: Explore using a standard URI to force log in
		url: net.jazz.ajax._contextRoot + "/web/loggedInUser"
	});
	
	var clientDeferred = new dojo.Deferred(function(deferred) {
		loginDeferred.cancel();
	});
	
	loginDeferred.addCallback(function(result) {
		// hide response body
		clientDeferred.callback(null);
	});
	
	loginDeferred.addErrback(function(loginError) {
		clientDeferred.errback(loginError);
	});
	
	return clientDeferred;
};

// Default logout URL
var logoutUrl = net.jazz.ajax._contextRoot + "/auth/logout";

/**
 * Attempts to log out of the local server.
 * <p>
 * On a successful log out, the system calls the Deferred's callback chain with
 * an unspecified return type; clients may only assume that the log in attempt
 * succeeded based on the fact that the callback chain was called. Note that a
 * successful log in also fires the jazz.app.events.LOGOUT event.
 * </p><p>
 * On an unsuccessful log out attempt, the system calls the Deferred's errback
 * chain with an Error object containing information about the error that
 * occurred.
 * </p>
 * 
 * @return {dojo.Deferred} a dojo.Deferred object indicating success or failure
 *  of the log out attempt.
 * @since 0.7
 */
jazz.app.auth.logout = function() {
	var dfd = jazz.client.xhrPost({
		url: logoutUrl
	});
	
	dfd.addCallback(function(result) {
		// hide response body
		return null;
	});
	
	return dfd;
};

jazz.app.auth.setLogoutUrl = function(url) {
	logoutUrl = url;
}

jazz.app.auth.getUserRoles = function () {
	var dfd = new dojo.Deferred();	
	var xhrArgs = {
        url: net.jazz.ajax._contextRoot + "/authenticated/identity",
        content: { },
		handleAs: "json",
		headers: { 
			"X-com-ibm-team-configuration-versions": "LATEST",
			"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID") 
		},
		handle: dojo.hitch(this, function(response, ioArgs){
			var userId = response.userId||loggedInUser;
			if (userId) {
				dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", userId);
			}
			if (response instanceof Error) {
				var errorObj = {
					status: ioArgs.xhr.status
				};
				// format detailed message (unescape URI-encoding formatting, remove leading dashes, and trim whitespace)
				errorObj.message = dojo.trim(unescape(ioArgs.xhr.statusText).replace(/\+/g, " ").replace(/^(-*)/g, ""));
				dfd.errback(errorObj);
			} else {
				dfd.callback(response);
			}
	    })
    };
	jazz.client.xhrGet(xhrArgs);
	return dfd;
};

// internal state tracking logged-in user
var loggedInUser;

/*
 * Checks to see if the logged-in user has changed. Initially the logged-in user
 * is unknown. If someone either logs in or logs out, the logged-in user will
 * become known and this method will publish the jazz.app.events.USER_CHANGE
 * event.
 */
function checkedLoggedInUser(userId) {
	if(!userId) {
		userId = null;
	}
	if(userId !== loggedInUser) {
		loggedInUser = userId;
		dojo.publish(jazz.app.events.USER_CHANGE, [userId]);
	}
}

// Whether or not to monitor authentication events. This is necessary so that
// when a client calls login() we don't observe two user changes (from a valid
// userId to <null> and from <null> to a valid userId). 
var monitorAuthEvents = true;

function onAuthEvent(userId) {
	if(monitorAuthEvents) {
		checkedLoggedInUser(userId);
	}
}

dojo.subscribe(jazz.app.events.LOGIN, onAuthEvent);
dojo.subscribe(jazz.app.events.LOGOUT, onAuthEvent);

})();