/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.util.date");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("jazz.util", "dateMessages");

(function() {
	
var NEGATIVE_TIME_STRING = "--";
var HOURS_PER_DAY = 24;
var MILLISECONDS_PER_SECOND = 1000;
var MILLISECONDS_PER_MINUTE = MILLISECONDS_PER_SECOND * 60;
var MILLISECONDS_PER_HOUR = MILLISECONDS_PER_MINUTE * 60;
var MILLISECONDS_PER_DAY = MILLISECONDS_PER_HOUR * HOURS_PER_DAY;

jazz.util.date = {
	
	// summary:
	//		Provides supplemental date/time-related functions not provided by dojo.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// usage notes:
	//		- access this object via 'static' methods (e.g.
	//		  jazz.util.date.formatDif())
	//		- do not instantiate
	// since:
	//		0.7

	
	// summary:
	//		Formats the difference between the current time and the provided time
	//		in a human-readable string
	// date:
	//		The Date object representing the time to use for comparison
	// format:
	// 		Arguments used to specify the desired return format. Must be of the 
	//		following form:
	//			{
	//				approximate: [ true | false ]
	//			}
	//		Setting the approximate flag to true will return a approximate result
	//		(eg 2 Days ago) while settings the approximate flag to false will 
	//		return an exact result (eg 2 Days, 3 Hours, 34 Minutes ago)
	// return: 
	//		a human-readable string representing the change in time between the 
	//		current time and the 'date' parameter
	//
	/* string */ formatDif: function(/* Date */ date, /* object */ format) {
		// Backwards compatibility check: A call to the old API had the paramters [Date, boolean, boolean]
		// Check for this condition and fall back to the old behaviour if we're in that case
		var oldApi = (arguments.length == 3 
			&& typeof arguments[1] == 'boolean'
			&& typeof arguments[2] == 'boolean');
		if (oldApi) {
			return _formatDiffLegacy(date, arguments[1], arguments[2]);
		} else {
			var isApproximate = format.approximate;
			if (typeof (format.approximate) == 'undefined') {
				isApproximate = true;
			} 
			if (isApproximate) {
				return _getApproxReadableTimeDif(date, format.currentTimeInMilliseconds);	
			} else {
				return _getFullReadableTimeDif(date, format.currentTimeInMilliseconds);	
			}
		}
	},

	// summary:
	//		Formats the provided duration into a specified format
	// duration:
	//		The duration which is to be formatted (in milliseconds)
	// format:
	// 		Arguments used to specify the desired return format. Must be of the 
	//		following form:
	//		{
	//			style : [ "full" | "stopwatch" | "abbreviated" ], 
	//			approximate : [ true | false ]
	//		}
	//
	//		If the style is set to "full", the result will be formatted in a 
	//		human-readable string. The "approximate" flag is used to specify whether a 
	// 		complete or approximate string is returned. Example:
	//			formatDuration(14797000, {style : "full", approximate : true}) 
	//				- returns "4 hours" 
	//			formatDuration(14797000, {style : "full", approximate : false})
	//				- returns "4 hours 6 minutes 37 seconds"
	//
	//		If the style is set to "stopwatch", the result will be formatted in a
	//		24-hour stopwatch time (eg 20:34:34). The 'approximate' flag is not used,
	//		all all of the following function calls are equivalent:
	//			formatDuration(367237000, {style : "stopwatch")							
	//			formatDuration(367237000, {style : "stopwatch", approximate : true})	
	//			formatDuration(367237000, {style : "stopwatch", approximate : false})
	//
	//		If the style is set to "abbreviated", the result will be formatted with
	//		abbreviated time components. Example output:
	//      		 	     45 s
	//      			 1 m 45 s
	//     				18 m 02 s
	// 				1 h 20 m 05 s
	//
	//		The 'approximate' flag is used when the provided duration is less than 1 
	//		second. If the flag is set to false, the ms component will be printed. Example:
	//			formatDuration(342, {style : "abbreviated", approximate : true)
	//				- returns "0 s"
	//			formatDuration(342, {style : "abbreviated", approximate : true)
	//				- returns "342 ms"
	// return:
	// 		A string representing the duration in the requested format.
	//
	/* string */ formatDuration: function(/* int */ duration, /* object */ format) {
		if (typeof (format) == 'undefined') { 
   			// Backwards compatibility: if the format var is undefined, this is likely
			// a call to the old API so fall back on the old behaviour
			return _formatApproxDuration(duration);
		}
		
		if (format.style == "abbreviated") {
			var isAppox = format.approximate;
			if (typeof(isAppox) == 'undefined') {
				// approximate flag is not provided - default to true
				isAppox = true;
			}
			return _formatAbbreviatedTime(duration, isAppox);
		} else if (format.style == "stopwatch") {
			return _formatStopwatchTime(duration);
		} else if (format.style == "full") {
			var isApprox = format.approximate;
			if (isApprox) {
				return _formatApproxDuration(duration);
			} else {
				return _formatFullDuration(duration);					
			}
		}
	}
};

//
// Internal; do not call or reference
//

var _messages = dojo.i18n.getLocalization("jazz.util", "dateMessages");

var _getDayOfYear = function(/* Date */ date){
	return dojo.date.difference(new Date(date.getFullYear(), 0, 1), new Date(date.getFullYear(), date.getMonth(), date.getDate())) + 1;
};

var _formatStopwatchTime = 	/* string */ function(/* int */duration) {
	// summary:
	//		Format the given time in milliseconds as an elapsed time in "stopwatch"
	//		format. Only days, hours, minutes and seconds are shown. e.g.:
	//
	//           00:00:01
	//           00:24:13
	//           01:00:01
	//           23:59:59
	//    1 day, 00:24:13
	//   9 days, 08:15:37
	// 847 days, 08:15:37
	//
	// duration:
	//		The time in milliseconds to be formatted 
	// return: 
	//		a string representing the given time in stopwatch format
	
	var startTimeInSeconds = duration / 1000;
	var seconds = Math.floor(startTimeInSeconds % 60);
	var startTimeInMinutes = startTimeInSeconds / 60;
	var minutes = Math.floor(startTimeInMinutes % 60);
	var startTimeInHours = startTimeInMinutes / 60;
	var hours = Math.floor(startTimeInHours % 24);		
	var startTimeInDays = Math.floor(startTimeInHours / 24);
	
	var secondsString = seconds < 10 ? _messages.paddedResult : "${0}"; 
	var minutesString = minutes < 10 ? _messages.paddedResult : "${0}";
	var hoursString = hours < 10 ? _messages.paddedResult : "${0}"; 
	
	hoursString = dojo.string.substitute(hoursString, [ hours ]);
	minutesString = dojo.string.substitute(minutesString, [ minutes ]);
	secondsString = dojo.string.substitute(secondsString, [ seconds ]);
	
	if (startTimeInDays > 0) {
		var dayString = startTimeInDays == 1 ? _messages.day : dojo.string.substitute(_messages.days, [ startTimeInDays ]);
		return dojo.string.substitute(_messages.stopwatchTimeWithDay, [ dayString, hoursString, minutesString, secondsString ]);
	} else {
		return dojo.string.substitute(_messages.stopwatchTime, [ hoursString, minutesString, secondsString ]);
	}
}

var _formatAbbreviatedTime = /* string */ function(/* int */ duration, /* boolean */ showMilliseconds) {
	// summary:
	//	Format the given time in milliseconds as a string with abbreviated time
	//	components. If the given time is less than 1 second, the string will show
	//	milliseconds unless <code>showMilliseconds</code> is false, in which
	//	case the string will show 0 seconds. A zero will be prepended to single
	//	digit numbers (unless it is the leftmost number) to allow for good
	//	alignment results in aligned columns. Example formatted strings:
	//
	// 			4 ms
	//        885 ms
	//           1 s
	//          45 s
	//      1 m 45 s
	//     18 m 02 s
	// 1 h 20 m 05 s
	// duration:
	// 		the time in milliseconds to be formatted
	// showMilliseconds:
	//		Optional. Whether or not to snow milliseconds if time is less than 1 second.
	//		Default value is true 
	// return:
	//		a string representing the given time in the abbreviated format
	
	if (showMilliseconds === undefined) showMilliseconds = true;
	
	if (duration < 0) {
		return NEGATIVE_TIME_STRING;
	}
	
	var timeInSeconds = duration / 1000;
	var seconds = Math.floor(timeInSeconds % 60);
	var timeInMinutes = timeInSeconds / 60;
	var minutes = Math.floor(timeInMinutes % 60);		
	var timeInHours = timeInMinutes / 60;
	var hours = Math.floor(timeInHours);		
	
	var secondsString = seconds < 10 && duration >= MILLISECONDS_PER_MINUTE ? 
			_messages.paddedResult : "${0}"; 
	var minutesString = minutes < 10 && duration >= MILLISECONDS_PER_HOUR ? 
			_messages.paddedResult : "${0}";
	var hoursString = "${0}"; 

	if (duration < 1000 && showMilliseconds) {
		return dojo.string.substitute(_messages.millisecsShort, [duration]);
	} else if (duration < MILLISECONDS_PER_MINUTE) {
		secondsString = dojo.string.substitute(secondsString, [ seconds ]);
		return dojo.string.substitute(_messages.onePartAbbreviatedTime, [secondsString]);
	} else if (duration < MILLISECONDS_PER_HOUR) {
		secondsString = dojo.string.substitute(secondsString, [ seconds ]);
		minutesString = dojo.string.substitute(minutesString, [ minutes ]);
		return dojo.string.substitute(_messages.twoPartAbbreviatedTime, [minutesString, secondsString]);
	} else {
		secondsString = dojo.string.substitute(secondsString, [ seconds ]);
		minutesString = dojo.string.substitute(minutesString, [ minutes ]);
		hoursString = dojo.string.substitute(hoursString, [ hours ]);
		return dojo.string.substitute(_messages.threePartAbbreviatedTime, [hoursString, minutesString, secondsString]);
	}
	
};

var _formatApproxDuration = /* string */ function(/* int */ duration) {
	// summary:
	//		Formats the given number of milliseconds into an easy to read,
	//		approximate duration. e.g.:
	//		
	//		5000: 5 seconds
	//		200000: 3 minutes, 20 seconds
	//		99000111: 1 day, 4 hours
	//	duration:
	//		the number of milliseconds for the duration
	//	return:
	//		a string representing the given date in the specified format
	
	var timeLeft = Math.round(duration / 1000);
	var seconds = timeLeft % 60;
	timeLeft = Math.floor(timeLeft / 60);
	if (timeLeft > 0) {
		var minutes = timeLeft % 60;
		timeLeft = Math.floor(timeLeft / 60);
		if (timeLeft > 0) {
			var hours = timeLeft % 24;
			timeLeft = Math.floor(timeLeft / 24);
			if (timeLeft > 0) {
				var days = timeLeft;
			}
		}
	}
	
	// collect string segments
	var segments = [];
	var units = [
		[days, _messages.day, _messages.days],
		[hours, _messages.hour, _messages.hours],
		[minutes, _messages.min, _messages.mins],
		[seconds, _messages.sec, _messages.secs]
	];
	for (var i=0;i<units.length && segments.length<2;i++) {
		var unit = units[i];
		var value = unit[0];
		if (value) {
			segments.push(value == 1 ? unit[1] : dojo.string.substitute(unit[2], [value]));
		}
	}
	
	// format string
	switch (segments.length) {
		case 0:
			return _messages.lessThanSec;
		case 1:
			return segments[0];
		case 2:
			return dojo.string.substitute(_messages.twoPartDuration, segments);
	}
};

var _formatFullDuration = /* string */ function(/* int */duration) {
	// summary:
	//		Format the specified time into a string showing hours, minutes and
	//		seconds, or, if the total time is less than 1 second, a string showing
	//		just milliseconds.
	//		
	//		The units of time (hours, minutes, seconds, milliseconds) are written out
	//		fully - they are not abbreviated.
	//
	//		Milliseconds is shown iff the total time is less than 1 second.
	//
	//		The string will not contain any leading portion that would be "0"; for
	//		instance, "0 hours, 59 minutes, 59 seconds" would be "59 minutes, 59
	//		seconds".
	//	duration:
	//		the number of milliseconds for the duration
	//	return:
	//		a string representing the given date in the specified format
	
	if ( duration < 0 ) {
		return NEGATIVE_TIME_STRING;
	} else if ( duration == 0 ) {
		return dojo.string.substitute(_messages.secs, [ duration ]);		
	} else if ( duration < 1000 ) {
		return ( duration == 1) ? _messages.millisec : dojo.string.substitute(_messages.millisecs, [ duration ]);
	}
	
	var seconds = Math.floor(duration / 1000 % 60);
	var secondsStr = (seconds == 1) ? _messages.sec : dojo.string.substitute(_messages.secs, [ seconds ]); 
	var minutes = Math.floor(duration / 60000 % 60);
	var minutesStr = (minutes == 1) ? _messages.min : dojo.string.substitute(_messages.mins, [ minutes ]);
	var hours = Math.floor(duration / 3600000);
	var hoursStr = (hours == 1) ? _messages.hour : dojo.string.substitute(_messages.hours, [ hours ]);
	
	if (hours > 0) {
		return dojo.string.substitute(_messages.threePartDuration, [hoursStr, minutesStr, secondsStr]);
	}
	if (minutes > 0) {
		return dojo.string.substitute(_messages.twoPartDuration, [minutesStr, secondsStr]);
	}
	return secondsStr;
};

var _getApproxReadableTimeDif = function(/* Date */ date, /* int? */ currentTimeInMilliseconds) {
	// This function is very similar to the public getReadableTimeDif() function, the 
	// only difference is that this function provides an approximate string, while the 
	// getReadableTimeDif() function provides the complete difference
	//
	// For example, suppose the time difference was 150 seconds (2 minutes, 30 seconds). 
	// This method will return "2 minutes ago"
	// getReadableTimeDif() will return "2 minutes and 30 seconds ago"
	if (!date)
		return "";
		
	var now = currentTimeInMilliseconds;
	if (typeof(currentTimeInMilliseconds) == 'undefined') {
		now= new Date().getTime();
	}  
	var then= date.getTime();
	var dif= now - then;

	var minuteIntervals= [ 1, 2, 3, 5, 10, 15, 20, 30, 45, 60 ];
	var hourIntervals= [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24 ];

	/* Event is Now (dif < 60 Seconds) */
	if (dif > -MILLISECONDS_PER_MINUTE && dif < MILLISECONDS_PER_MINUTE)
		return _messages.now;

	/* Event is in the Future */
	else if (dif < 0) {
		dif*= -1;

		/* Minutes */
		if (dif < MILLISECONDS_PER_MINUTE * 2)
			return _messages.inMin;

		for (var i= 2; i < minuteIntervals.length; i++) {
			if (dif < minuteIntervals[i] * MILLISECONDS_PER_MINUTE)
				return dojo.string.substitute(_messages.inMins, [ minuteIntervals[i-1] ]);
		}

		/* Hours */
		if (dif < MILLISECONDS_PER_HOUR * 2)
			return _messages.inHour;

		for (var i= 1; i < hourIntervals.length; i++) {
			if (dif < hourIntervals[i] * MILLISECONDS_PER_HOUR)
				return dojo.string.substitute(_messages.inHours, [ hourIntervals[i-1] ]);
		}

		if (dif < 2 * MILLISECONDS_PER_DAY) {
			return _messages.inDay;
		}
		return _messages.inSeveralDays;
	}

	/* Event is in the Past */
	else {

		/* Minutes */
		if (dif < MILLISECONDS_PER_MINUTE * 2)
			return _messages.pastMin;

		for (var i= 2; i < minuteIntervals.length; i++) {
			if (dif < minuteIntervals[i] * MILLISECONDS_PER_MINUTE)
				return dojo.string.substitute(_messages.pastMins, [ minuteIntervals[i-1] ]);
		}

		/* Hours */
		if (dif < MILLISECONDS_PER_HOUR * 2)
			return _messages.pastHour;

		for (var i= 2; i < hourIntervals.length; i++) {
			if (dif < hourIntervals[i] * MILLISECONDS_PER_HOUR)
				return dojo.string.substitute(_messages.pastHours, [ hourIntervals[i-1] ]);
		}

		if (dif < 2 * MILLISECONDS_PER_DAY) {
			return _messages.pastDay;
		}
		return _messages.pastSeveralDays;
	}
};

var	_getFullReadableTimeDif = /* string */ function(/* Date */ date, /* int? */ currentTimeInMilliseconds) {
		// summary:
		// 		Compares the given Date with the current Time and generates a Label
		//		describing the difference in specified locale format. The given Date can either 
		//		be in the future or in the past. Some sample results are: "1 minute ago", 
		//		"3 hours ago" or "in 5 minutes". Dates more than one day before or after 
		//		the current time generate "in more than one day" or "more than one day ago".
		// dateToCompare:
		//		The Date to compare with the current Time. Can either be a
		//      future date or a past date. The Date's difference in Time to
		//      the current one should not be greater of 1 Day.
		// currentTimeInMilliseconds:
		// 		Optional. The time against which to compare dateToCompare, in milliseconds. If
		//		not supplied, the time will be calculated using new Date().getTime() 
		// loc:
		//		Display date in specified locale format
		// return: 
		//		a string that represents the difference in time in a readable format
		//		For example, "1 Minute ago" "3 Hours ago" or "In 5 Minutes"
		
		if (!date)
			return "";
		
		var now = currentTimeInMilliseconds;
		if (typeof(currentTimeInMilliseconds) == 'undefined') {
			now = new Date().getTime();
		} 
		
		var dif = now - date.getTime();
		var isPast = dif > 0;
		if (!isPast) 
			dif *= -1;
	
		var numHours = Math.floor(dif / MILLISECONDS_PER_HOUR);
		var difMinutes = dif - numHours * MILLISECONDS_PER_HOUR;
		var numMinutes = Math.floor(difMinutes / MILLISECONDS_PER_MINUTE);
		var difSeconds = difMinutes - numMinutes * MILLISECONDS_PER_MINUTE;
		var numSeconds = Math.floor(difSeconds / MILLISECONDS_PER_SECOND);
		
	  	if (numHours == 0 && numMinutes == 0 && numSeconds == 0) 
			return _messages.now;
		
		/* Event is in the Future */
		else if (!isPast) {
			if (numHours * MILLISECONDS_PER_HOUR + numMinutes * MILLISECONDS_PER_MINUTE + numSeconds
			        * MILLISECONDS_PER_SECOND > MILLISECONDS_PER_DAY) {
					return dojo.string.substitute(_messages.inSeveralDaysWithDate, [ dojo.date.locale.format(date, {formatLength: "medium"}) ]);		        	
			} else if (numHours > 1) {
				if (numMinutes > 1) {
					return dojo.string.substitute(_messages.inHoursAndMins, [ numHours, numMinutes ]);				
				} else if (numMinutes == 1) {
					return dojo.string.substitute(_messages.inHoursAndMin, [ numHours ]);				
				} else { // numMinutes == 0
					return dojo.string.substitute(_messages.inHours, [ numHours ]);
				}
			} else if (numHours == 1) {
				if (numMinutes > 1) {
					return dojo.string.substitute(_messages.inHourAndMins, [ numMinutes ]);
				} else if (numMinutes == 1) {
					return _messages.inHourAndMin;
				} else { // numMinutes == 0
					return _messages.inHour;
				}
			} else { // numHours == 0
				if (numMinutes > 1) {
					return dojo.string.substitute(_messages.inMins, [ numMinutes ]);
				} else if (numMinutes == 1) {
					return _messages.inMin;
				} else { // numMinutes == 0
					return _messages.inLessThanMin;
				}
			}
		}
			
		/* Event is in the Past */
		else {
	
			if (numHours * MILLISECONDS_PER_HOUR + numMinutes * MILLISECONDS_PER_MINUTE + numSeconds
					* MILLISECONDS_PER_SECOND > MILLISECONDS_PER_DAY) {
				return dojo.string.substitute(_messages.pastSeveralDaysWithDate, [ dojo.date.locale.format(date, {formatLength: "medium"}) ]);
			} else if (numHours > 1) {
				if (numMinutes > 1) {
					return dojo.string.substitute(_messages.pastHoursAndMins, [ numHours, numMinutes ]);
				} else if (numMinutes == 1) {
					return dojo.string.substitute(_messages.pastHoursAndMin, [ numHours ]);
				} else { // numMinutes == 0
					return dojo.string.substitute(_messages.pastHours, [ numHours ]);
				}
			} else if (numHours == 1) {
				if (numMinutes > 1) {
					return dojo.string.substitute(_messages.pastHourAndMins, [ numMinutes ]);
				} else if (numMinutes == 1) {
					return _messages.pastHourAndMin;
				} else { // numMinutes == 0
					return _messages.pastHour;
				}
			} else { // numHours == 0
				if (numMinutes > 1) {
					return dojo.string.substitute(_messages.pastMins, [ numMinutes ]);
				} else if (numMinutes == 1) {
					return _messages.pastMin;
				} else { // numMinutes == 0
					return _messages.lessThanMinAgo;
				}			
			} 
		}
	};

var _formatDiffLegacy = /* string */ function(/* Date */ date, /* boolean */ showAbsoluteDate, /* boolean */ showRelativeDate) {
	// Legacy code - Do not use! This function is still included for backwards compatibility
	// summary:
	//		Formats the given date into a string indicating the difference compared
	//		to the current time (past or future). At least one of showAbsoluteDate
	//		and showRelativeDate is required.
	//
	//		Examples of calling this method with true supplied for
	//		showAbsoluteDate and showRelativeDate:
	//
	//		Now + 5 Seconds: 12:15 PM (In 1 minute)
	//		Now - 5 Minutes: 12:10 PM (5 minutes ago)
	//		Now + 5 Hours: 5:15 PM (In 5 hours)
	//		Now + 1 Day: Sep 30 12:26 PM (Tomorrow)
	//		Now - 2 Days: Sep 27 12:26 PM (2 days ago)
	//		Now + 1 Week: Oct 06 12:26 PM (Next Week)
	//		Now + 3 Weeks: Oct 20 12:26 PM
	//		Now - 1 Year: Sep 29 2005 12:26 PM
	//		
	//		Examples for passing false for showAbsoluteDate:
	//		
	//		Now + 5 Seconds: In 5 minutes
	//		Now - 2 Hours: 2 hours ago
	//	date:
	//		the Date to compare to the current time.
	//	showAbsoluteDate:
	//		if true, the specific date is made clear in the result
	//	showRelativeDate:
	//		if true, a rough approximation of the time difference is indicated.
	//	return:
	//		a string representing the given date in the specified format
	
	if (!date)
		return "";
	
	var absoluteStr;
	var relativeStr;
	
	if (showAbsoluteDate) {
		var dateStr = dojo.date.locale.format(date, { selector: "date", formatLength: "medium" });
		var timeStr = dojo.date.locale.format(date, { selector: "time", formatLength: "short" });
		absoluteStr = dojo.string.substitute(_messages.dateAndTime, [dateStr, timeStr]);
	}
	if (showRelativeDate) {
		var fgCalBase = new Date();
		var fgCalActual = date;
		
		var baseDay = _getDayOfYear(fgCalBase);
		var actualDay = _getDayOfYear(fgCalActual);
		
		var baseWeek = _getWeekOfYear(fgCalBase);
		var actualWeek = _getWeekOfYear(fgCalActual);
		
		var baseYear = fgCalBase.getFullYear();
		var actualYear = fgCalActual.getFullYear();
		
		/* Same Day */
		if (baseDay == actualDay && baseYear == actualYear) {
			var currentTimeInMilliseconds = fgCalBase.getTime();
			relativeStr = _getApproxReadableTimeDif(date);
		}
		
		/* Yesterday */
		else if ((baseDay - actualDay) == 1 && baseYear == actualYear) {
			relativeStr = _messages.yesterday;
		}
		
		/* Tomorrow */
		else if ((actualDay - baseDay) == 1 && baseYear == actualYear) {
			relativeStr = _messages.tomorrow;
		}
		
		/* Same Week */
		else if (baseWeek == actualWeek && baseYear == actualYear) {
			var dayDif= actualDay - baseDay;
			if (dayDif * dayDif == 1) {
				relativeStr = (dayDif > 0) ? _messages.inDay : _messages.pastDay;
			}
			else {
				relativeStr = (dayDif > 0) ? dojo.string.substitute(_messages.inDays, [dayDif]) : dojo.string.substitute(_messages.pastDays, [-dayDif]);
			}
		}
		
		/* Last Week */
		else if ((baseWeek - actualWeek) == 1 && baseYear == actualYear) {
			relativeStr = _messages.lastWeek;
		}
		
		/* Next Week */
		else if ((actualWeek - baseWeek) == 1 && baseYear == actualYear) {
			relativeStr = _messages.nextWeek;
		}
	}
	if (!absoluteStr && !relativeStr) {
		// neither was requested or generated; fall back to date
		absoluteStr = dojo.date.locale.format(date, { selector: "date", formatLength: "medium" });
	}
	
	// build final string
	if (absoluteStr && relativeStr) {
		// show combined format
		return dojo.string.substitute(_messages.absoluteAndRelative, [absoluteStr, relativeStr]);
	}
	return absoluteStr || relativeStr;
};

var _getWeekOfYear = function(/* Date */ date){
	var firstDayOfYear = new Date(date.getFullYear(), 0, 1).getDay();
	var adj = (firstDayOfYear + 7) % 7;
	var week = Math.floor((_getDayOfYear(date) + adj - 1) / 7);
	if(firstDayOfYear == 0){ week++; }
	return week;
};

})();
