/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.entryPoints");

dojo.require("jazz.app.internal.schema.UIEntryPoints");
dojo.require("jazz.app.internal.util.DiscoveredService");
dojo.require("jazz.ensemble.internal.util.Xml");

(function() {
	
var DiscoveredService = jazz.app.internal.util.DiscoveredService;
var UIEntryPoints = jazz.app.internal.schema.UIEntryPoints;
var namespace = jazz.ensemble.internal.util.Xml.namespace;
var urls = {};

/**
 * Discovers Jazz UI entry points from the discovery service. Useful for discovering the url 
 * for Personal Dashboards, Lifecycle Project Admin, etc.
 * 
 * @param {String|Array} value A string or an array of strings containing UI entry point names.
 * @return {dojo.Deferred} The deferred callback will be passed the url or an array of urls (the 
 * 		same order you passed in).	i.e. if you passed in a String, it will pass the url string. 
 * 		If you passed in an array of strings, it will return you an array of url strings.  If any
 * 		entry point doesn't exist, null will be returned in its place.
 * 
 * Known UI entry point names:
 *   - PersonalDashboards
 *   - LifecycleProjectAdmin
 *   - Users
 *   - JazzTeamServerHome
 * 
 * Example usage:
 * 
 * var deferred = jazz.app.entryPoints.get(["PersonalDashboards", "LifecycleProjectAdmin"]);
 * deferred.addCallback(function(urls) {
 * 		var dashboardsUrl = urls[0];
 * 		var lpaUrl = urls[1];
 * 		//do stuff.
 * });
 * 
 */
jazz.app.entryPoints.get = function(value) {
	var deferred = new dojo.Deferred();
	var typesToFetch = [], valuesToFetch = [];
	if (dojo.isString(value)) {
		if (!urls[value]) {
			typesToFetch.push(namespace.juentry + value);
			valuesToFetch.push(value);
		}
	} else if (dojo.isArray(value)) {
		for (var i = 0; i < value.length; i++) {
			if (!urls[value[i]]) {
				valuesToFetch.push(value[i]);
				typesToFetch.push(namespace.juentry + value[i]);
			}
		}
	}
	if (typesToFetch.length > 0) {
		var discoveryDeferred = DiscoveredService.loadDiscoveryServices(typesToFetch, UIEntryPoints.getSchema(valuesToFetch));
		discoveryDeferred.addBoth(function(doc){
			if (dojo.isString(value)) {
				var type = doc[value];
				if (type && type.url) {
					deferred.callback(type.url);
					urls[value] = type.url;
				} else {
					deferred.callback(null);
				}
			} else if (dojo.isArray(value)) {
				var retArr = [];
				for (var j = 0; j < value.length; j++) {
					var type = doc[value[j]];
					if (type && type.url) {
						retArr.push(type.url);
						urls[value[j]] = type.url;
					} else {
						var url = urls[value[j]];
						retArr.push(url||null);
					}
				}
				deferred.callback(retArr);
			}
			return doc;
		});
	} else {
		if (dojo.isString(value)) {
			deferred.callback(urls[value]);
		} else if (dojo.isArray(value)) {
			var retArr = []; 
			for (var k = 0; k < value.length; k++) {
				retArr.push(urls[value[k]]);
			}
			deferred.callback(retArr);
		}
	}
	return deferred;
}
})();