/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.WidgetContainer");

dojo.require("dijit._Contained");
dojo.require("dijit.MenuSeparator");

dojo.require("dojo.i18n");
dojo.require("dojox.xml.parser");

dojo.require("jazz.ensemble.WidgetContext");
dojo.require("jazz.ensemble.internal.WidgetFactory");
dojo.require("jazz.ensemble.internal.settings.AppearancePanel");
dojo.require("jazz.ensemble.internal.util.ImgMenuItem");
dojo.require("jazz.ui.Menu");
dojo.require("jazz.ui.StyledBox");

dojo.requireLocalization("jazz.ensemble.internal", "Messages");

(function() {

var AppearancePanel = jazz.ensemble.internal.settings.AppearancePanel;
var ImgMenuItem = jazz.ensemble.internal.util.ImgMenuItem;
var StyledBox = jazz.ui.StyledBox;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var WidgetContext = jazz.ensemble.WidgetContext;
var WidgetFactory = jazz.ensemble.internal.WidgetFactory;

var DEFAULT_TRIM = "blue";
var TRIM_BY_ID = {
	"blue": TrimStyles.BLUE,
	"gray": TrimStyles.GRAY,
	"gold": TrimStyles.GOLD,
	"green": TrimStyles.GREEN,
	"teal": TrimStyles.TEAL,
	"purple": TrimStyles.PURPLE,
	"orange": TrimStyles.ORANGE,
	"none": TrimStyles.TRIMLESS
};

var messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "Messages");

/*
 * The container with trim that houses a Widget, and functions as its
 * WidgetContext.
 */
dojo.declare("jazz.ensemble.internal.WidgetContainer", [WidgetContext, StyledBox, dijit._Contained], {
	
	"class": "jazz-ensemble-internal-WidgetContainer",
	collapsed: false,
	onStateChange: null,
	panelId: null,
	readOnly: false,
	shared: true,
	state: null,
	trim: TRIM_BY_ID[DEFAULT_TRIM],
	type: null,
	uuid: null,
	widget: null,
	widgetId: null,
	
	_appearancePanel: null,
	_definition: null,
	_editContentHandler: null,
	_loadingDiv: null,
	_maximizeHandler: null,
	_maximizeContainerHandler: null,
	_memento: null,
	_preferences: null,
	_preMaximizeHandler: null,
	_settingsHandler: null,
	_started: false,
	
	constructor: function(/* object */ args) {
		var state = args.state;
		this.closable = !args.readOnly;
		this.collapsed = state.isCollapsed() || false;
		this.iconPath = state.getIcon();
		this.primaryTitle = state.getTitle();
		this.showBackground = state.getShowBackground() || false;
		var trim = state.getTrim();
		if (trim) {
			this.trim = TRIM_BY_ID[trim];
		}
		this.type = state.getType();
		this.widgetId = state.getId();
		if (args.panelId) {
			this.uuid = args.panelId + "|" + this.widgetId;
		}
		this._definition = state.getDefinition();
		
		// create container div and show "Loading..." initially
		this.contentNode = dojo.create("div", {className: "jazz-ensemble-internal-WidgetContainer-content"});
		this._loadingDiv = dojo.create("div", {className: "loading-text", innerHTML: messages.loading}, this.contentNode);
	},
	
	postCreate: function() {
		this.inherited(arguments);
		var deferred = WidgetFactory.createWidget(this.state, this, this.readOnly, this.uuid);
		deferred.addCallback(dojo.hitch(this, this._createContents));
		deferred.addErrback(dojo.hitch(this, this._showError));
		this._isInitialized = true;
	},
	
	/*
	 * Gets the default title for the widget.
	 * 
	 * @return {string} the widget's default title
	 */
	getDefaultTitle: function() {
		return this.widget ? this.widget.getDefaultTitle() : "";
	},
	
	/*
	 * Gets the widget's 16x16 icon URL, or undefined if there is none.
	 * 
	 * @return {string} url the 16x16 icon url
	 */
	getIcon: function() {
		return this.iconPath;
	},
	
	/*
	 * Returns the memento, which is a string containing arbitrary persisted data
	 * for the widget. Provided by the container if exists. Can be set via setMemento.
	 * 
	 * @return {string} the widget memento
	 */
	getMemento: function() {
		if (!this._memento) {
			this._memento = this.state.getMemento();
		}
		return this._memento;
	},
	
	/*
	 * Returns an object with string properties representing preference key-value
	 * pairs. Provided by the container. Can be set via setPreferences.
	 * 
	 * @return {object} the widget preferences
	 */
	getPreferences: function() {
		if (!this._preferences) {
			this._preferences = {};
			var prefsCollection = this.state.getPreferences();
			if (prefsCollection) {
				var prefArray = prefsCollection.getPreferences();
				for (var i=0;i<prefArray.length;i++) {
					var pref = prefArray[i];
					this._preferences[pref.getName()] = pref.getValue();
				}
			}
		}
		return this._preferences;
	},
	
	getSecondaryTitle: function() {
		return this.secondaryTitle;
	},
	
	getTitle: function() {
		return this.primaryTitle;
	},
	
	/*
	 * Returns the widget container's toolbar, where additional toolbar items can
	 * be added, or undefined if the container has no toolbar.
	 * 
	 * @return {jazz.ui.SimpleToolbar} the container's toolbar
	 */
	getToolbar: function() {
		return this.toolbar;
	},
	
	/*
	 * Returns whether or not the widget state is shared among multiple users or not.
	 * Widgets may use this information to decide whether or not to allow user-
	 * specific personalization of the state.
	 * 
	 * @return {boolean} whether or not the widget state is shared or not
	 */
	isShared: function() {
		return this.shared;
	},
	
	/*
	 * Called when the user clicks to open the menu.
	 */
	menuProvider: function() {
		if (!this._widgetMenu) {
			var menu = this._widgetMenu = new jazz.ui.Menu();
			dojo.addClass(menu.domNode, "jazz-ensemble-internal-WidgetContainer-menu");
			if (!this.readOnly) {
				menu.addChild(new ImgMenuItem({
					label: messages.menuAppearance,
					iconClass: "jazz-ensemble-internal-WidgetContainer-menu-appearance",
					onClick: dojo.hitch(this, this._handleAppearance)
				}));
			}
			if (this._editContentHandler && !this.readOnly) {
				menu.addChild(new ImgMenuItem({
					label: messages.menuEditContent,
					iconClass: "jazz-ensemble-internal-WidgetContainer-menu-edit-content",
					onClick: dojo.hitch(this, this._handleEditContent)
				}));
			}
			if (this._settingsHandler) {
				menu.addChild(new ImgMenuItem({
					label: messages.menuSettings,
					iconClass: "jazz-ensemble-internal-WidgetContainer-menu-settings",
					onClick: dojo.hitch(this, this._handleSettings)
				}));
			}
			if (menu.getChildren().length > 0) {
				menu.addChild(new dijit.MenuSeparator());
			}
			var contribs = this.menuContributions;
			if (contribs && contribs.length > 0) {
				for (var i=0;i<contribs.length;i++) {
					menu.addChild(new ImgMenuItem({
						label: contribs[i].label,
						icon: contribs[i].icon,
						onClick: contribs[i].onClick
					}));
				}
				menu.addChild(new dijit.MenuSeparator());
			}
			if (!this.maximized) {
				menu.addChild(new ImgMenuItem({
					label: messages.menuCopy,
					iconClass: "jazz-ensemble-internal-WidgetContainer-menu-copy",
					onClick: dojo.hitch(this, this._handleCopy)
				}));
				if (this._duplicateHandler) {
					menu.addChild(new ImgMenuItem({
						label: messages.menuDuplicate,
						iconClass: "jazz-ensemble-internal-WidgetContainer-menu-duplicate",
						onClick: dojo.hitch(this, this._handleDuplicate)
					}));
				}
				menu.addChild(new dijit.MenuSeparator());
			}
			if (!this.maximized) {
				if (this.collapsed) {
					menu.addChild(new ImgMenuItem({
						label: messages.menuExpand,
						iconClass: "jazz-ensemble-internal-WidgetContainer-menu-expand",
						onClick: dojo.hitch(this, this.setCollapsed, false)
					}));
				}
				else {
					menu.addChild(new ImgMenuItem({
						label: messages.menuCollapse,
						iconClass: "jazz-ensemble-internal-WidgetContainer-menu-collapse",
						onClick: dojo.hitch(this, this.setCollapsed, true)
					}));
				}
				menu.addChild(new dijit.MenuSeparator());
			}
			menu.addChild(new ImgMenuItem({
				label: messages.menuRefresh,
				iconClass: "jazz-ensemble-internal-WidgetContainer-menu-refresh",
				onClick: dojo.hitch(this, this._handleRefresh)
			}));
			menu.startup();
		}
		return this._widgetMenu;
	},
	
	/*
	 * Called when the StyledBox is collapsed.
	 */
	onCollapse: function() {
		if (!this.readOnly && !this.maximized) {
			this.state.setCollapsed(true);
			this._onMenuChange();
			this.onStateChange();
		}
	},
	
	/*
	 * Called when the StyledBox is expanded.
	 */
	onExpand: function() {
		if (!this.readOnly && !this.maximized) {
			this.state.setCollapsed(null);
			this._onMenuChange();
			this.onStateChange();
		}
	},
	
	/*
	 * @see _Widget#onShow
	 */
	onShow: function() {
		this.widget&&this.widget.onShow();
	},
	
	/*
	 * @see _Widget#onHide
	 */
	onHide: function() {
		this.widget&&this.widget.onHide();
	},
	
	refresh: function() {
		this.widget&&this.widget.refresh();
	},
	
	/*
	 * Sets the callbackhandler to be called when the user requests that the
	 * widget be maximized. The container of this widget must enlarge it so
	 * that it maximizes its content area. The function will be given one boolean
	 * parameter, which is whether the widget should be maximized or not.
	 */
	setMaximizeContainerHandler: function(/* function */ handler) {
		if (handler != this._maximizeContainerHandler) {
			this._maximizeContainerHandler = handler;
		}
	},
	
	/*
	 * Sets the callback handler to be called when the user requests that the
	 * widget be duplicated. The control(s) for duplicating will only be shown if
	 * a handler is set. This functionality is the responsibility of the container
	 * of this WidgetContainer.
	 */
	setDuplicateHandler: function(/* function */ handler) {
		if (handler != this._duplicateHandler) {
			this._onMenuChange();
			this._duplicateHandler = handler;
		}
	},
	
	/*
	 * Sets the callback handler to be called by the container when the user
	 * requests to edit content. The control(s) for editing content will only be
	 * shown if a handler is set.
	 * 
	 * @param {function} handler a function to be called when user requests
	 * 		to edit content
	 */
	setEditContentHandler: function(/* function */ handler) {
		if (handler !== this._editContentHandler) {
			// add toolbar button
			if (!this.readOnly && !this._editButton) {
				this._editButton = this.toolbar.addButton(this, this._handleEditContent,{
					className: "trim-edit-button",
					index: this.toolbar.items.length - 1,
					imgSrc: dojo.moduleUrl('jazz.ensemble', "internal/graphics/trim/images/edit-contents.gif").toString(),
					label: messages.trimEditImgAlt
				});
				this._queueTruncate(true);
			}
			this._onMenuChange();
			this._editContentHandler = handler;
		}
	},
	
	/*
	 * Sets the callback handler to be called by the container when the user
	 * requests to maximize the widget.
	 * 
	 * @param {function} handler a function to be called when user requests
	 * 		to maximize. The function will be passed one boolean param that will
	 *      be true if this is a request to maximize, and false otherwise.
	 */
	setMaximizeHandler: function(/* function */ handler) {
		if (handler !== this._maximizeHandler) {
			// add toolbar button
			if (this._maximizeContainerHandler && !this._maximizeButton) {
				this._maximizeButton = this.toolbar.addButton(this, this._handleMaximize,{
					className: "trim-maximize-button",
					index: this.toolbar.items.length - (this.readOnly ? 0 : 1),
					imgSrc: dojo.moduleUrl('jazz.ensemble', "internal/graphics/trim/images/maximize.gif").toString(),
					label: messages.trimMaximizeImgAlt
				});
				this._queueTruncate(true);
			}
			this._maximizeHandler = handler;
		}
	},
	
	/*
	 * Sets the widget memento value to the given one. A memento is arbitrary
	 * text to be persisted.
	 */
	setMemento: function(/* string */ memento) {
		this._memento = memento;
		this.state.setMemento(memento);
		this.onStateChange();
	},
	
	/*
	 * Sets the given menu items to show in the container menu. This is in addition
	 * to the default menu items provided by the container.
	 */
	setMenuContributions: function(/* object[] */ items) {
		if (items !== this.menuContributions) {
			this._onMenuChange();
			this.menuContributions = items;
		}
	},
	
	/*
	 * Sets the new preference values for the widget. To be called by widgets
	 * when their settings are changed by the user.
	 * 
	 * @param {object} prefs an object with string properties representing
	 * 		preference key-value pairs.
	 */
	setPreferences: function(/* object */ preferences) {
		// ensure the collection exists
		var prefsCollection = this.state.getPreferences();
		if (!prefsCollection) {
			prefsCollection = this.state.createPreferences();
			prefsCollection.setParseType("Collection");
			this.state.setPreferences(prefsCollection);
		}
		var prefs = prefsCollection.getPreferences();
		var newPrefs = [];
		
		// add existing prefs
		var prefsByName = {};
		for (var i=0;i<prefs.length;i++) {
			var pref = prefs[i];
			var value = preferences[pref.getName()];
			prefsByName[pref.getName()] = pref.getValue();
			if (value) {
				pref.setValue(value);
				newPrefs.push(pref);
			}
		}
		// add newly added prefs
		for (var i in preferences) {
			if (!prefsByName[i]) {
				var value = preferences[i] ? preferences[i] : "";
				var newPref = prefsCollection.createPreference();
				newPref.setName(i);
				newPref.setValue(value);
				newPrefs.push(newPref);
			}
		}
		// set new prefs
		prefsCollection.setPreferences(newPrefs);
		this._preferences = dojo.clone(preferences);
		this.onStateChange();
	},
	
	/*
	 * Sets the callback handler to be called by the container when the container
	 * is about to start a maximize or restore operation.
	 * 
	 * @param {function} handler a function to be called when container is about to
	 *      be maximized or restored. The function will be passed one boolean param
	 *      that will be true if the container is about to maximize, and false
	 *      otherwise.
	 */
	setPreMaximizeHandler: function(/* function */ handler) {
		if (handler !== this._preMaximizeHandler) {
			this._preMaximizeHandler = handler;
		}
	},
	
	/*
	 * Sets the callback handler to be called by the container when the user
	 * requests to edit settings. The control for editing settings will only be
	 * shown if a handler is set.
	 * 
	 * @param handler a javascript function to be called when settings were opened
	 */
	setSettingsHandler: function(/* function */ handler) {
		if (handler != this._settingsHandler) {
			this._onMenuChange();
			this._settingsHandler = handler;
		}
	},
	
	/*
	 * Sets the title of the widget on the trim and in the state. This is a
	 * silent change to the state so it will not dirty the widget panel and will
	 * only be persisted if a save occurs for another change.
	 * 
	 * @param {string} title the new widget title.
	 */
	setTitle: function(/* string */ title) {
		if (this._isInitialized) {
			if (!this.state.isCustomTitle()) {
				this.state.setTitle(title);
				this.inherited(arguments);
			}
			this._latestTitle = title;
		}
		else {
			this.inherited(arguments);
		}
	},
	
	/*
	 * Override StyledBox#setTrim to not set it while maximized.
	 */
	setTrim: function(/* string */ trim) {
		if (!this.maximized) {
			this.inherited(arguments);
		}
	},
	
	startup: function() {
		this.widget && this.widget.startup();
		this._started = true;
	},
	
	uninitialize: function() {
		if (this._widgetMenu) {
			this._widgetMenu.destroyRecursive();
		}
		if (this.widget) {
			this.widget.destroy();
		}
	},
	
	_createContents: function(/* Widget */ widget) {
		this.widget = widget;
		this.contentNode.replaceChild(widget.domNode, this._loadingDiv);
		if (this._started) {
			widget.startup();
		}
		return widget;
	},
	
	_handleAppearance: function() {
		// if not already open
		if (!this._appearancePanel) {
			this._appearancePanel = new AppearancePanel({
				onCancel: dojo.hitch(this, this._handleAppearanceCancel),
				onOk: dojo.hitch(this, this._handleAppearanceOk),
				container: this
			});
			dojo.place(this._appearancePanel.domNode, this.contentNode, "first");
			this._appearancePanel.open();
			this.collapsed && this.setCollapsed(false);
		}
	},
	
	_handleAppearanceCancel: function() {
		if (this._appearancePanel.hasSettingChanged('trim')) {
			this.setTrim(TRIM_BY_ID[this.state.getTrim() || DEFAULT_TRIM]);
		}
		if (this._appearancePanel.hasSettingChanged('showBackground')) {
			this.setBackground(this.state.getShowBackground() || false);
		}
		this._appearancePanel.close(dojo.hitch(this, this._handleAppearanceClose));
	},
	
	_handleAppearanceClose: function() {
		this.contentNode.removeChild(this._appearancePanel.domNode);
		this._appearancePanel.destroyRecursive();
		delete this._appearancePanel;
	},
	
	_handleAppearanceOk: function() {
		var hasChanged = false;
		if (this._appearancePanel.hasSettingChanged('title')) {
			var customTitle = this._appearancePanel.getSettingValue('title');
			var title = customTitle || this._latestTitle || this.getDefaultTitle();
			this.inherited("setTitle",[title]); // call the StyledBox setTitle directly
			this.state.setTitle(title);
			this.state.setCustomTitle(customTitle ? true : false);
			hasChanged = true;
		}
		if (this._appearancePanel.hasSettingChanged('trim')) {
			this.state.setTrim(this._appearancePanel.getSettingValue('trim'));
			hasChanged = true;
		}
		if (this._appearancePanel.hasSettingChanged('showBackground')) {
			this.state.setShowBackground(this._appearancePanel.getSettingValue('showBackground') === "true" ? true : false);
			hasChanged = true;
		}
		this._appearancePanel.close(dojo.hitch(this, this._handleAppearanceClose));
		if (hasChanged)
			this.onStateChange();
	},
	
	_handleCopy: function() {
		// store the widget XML in the local storage clipboard
		try {
			localStorage["net-jazz-ajax-clipboard"] = dojox.xml.parser.innerXML(this.state.elem);
		} catch (e) {
			alert(messages.dlgNotSupported);
		}
	},
	
	_handleDuplicate: function(/* Event */ event) {
		// this is the responsibility of the higher container
		if (event) {
			dojo.stopEvent(event);
		}
		this._duplicateHandler && this._duplicateHandler();
	},
	
	_handleEditContent: function(/* Event */ event) {
		if (event) {
			dojo.stopEvent(event);
		}
		this._editContentHandler && this._editContentHandler();
	},
	
	_handleMaximize: function(/* Event */ event) {
		// this is partially the responsibility of the higher container
		if (event) dojo.stopEvent(event);
		var maximize = !this.maximized;
		// notify that we're about to maximize/restore
		this._preMaximizeHandler && this._preMaximizeHandler(!this.maximized);
		if (!maximize) {
			if (this.state.isCollapsed()){
				this.setCollapsed(true, false);
			}
			this.maximized = false;
		}
		this.setClosable(!this.readOnly && !maximize);
		this.setTrim(maximize ? "" : TRIM_BY_ID[this.state.getTrim() || DEFAULT_TRIM]);
		if (maximize) {
			this.maximized = true;
			if (this.collapsed) {
				this.setCollapsed(false, false);
			}
		}
		this._maximizeContainerHandler(this.maximized);
		this._maximizeHandler(this.maximized);
		var args = this.maximized ? {
			label: messages.trimRestoreImgAlt,
			className: "trim-restore-button",
			imgSrc: dojo.moduleUrl('jazz.ensemble', "internal/graphics/trim/images/restore.gif").toString()
		} : {
			label: messages.trimMaximizeImgAlt,
			className: "trim-maximize-button",
			imgSrc: dojo.moduleUrl('jazz.ensemble', "internal/graphics/trim/images/maximize.gif").toString()
		};
		this.toolbar.updateButton(this._maximizeButton, args);
		this._truncate(true);
	},
	
	_handleRefresh: function() {
		this.widget.refresh();
	},
	
	_handleSettings: function() {
		if (this._settingsHandler) {
			this._settingsHandler();
			if (this.collapsed) {
				this.setCollapsed(false);
			}
		}
	},
	
	_onMenuChange: function() {
		// clear cache, rebuild menu next time
		if (this._widgetMenu) {
			this._widgetMenu.destroyRecursive();
			delete this._widgetMenu;
		}
	},
	
	_showError: function(/* Error */ e) {
		// TODO: show nice error msg
		console.error(e);
	}
});

})();
