/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("jazz.ensemble.internal.WidgetEnsemble");

dojo.require("jazz.ensemble.WidgetPanel");
dojo.require("jazz.ensemble.internal.client.HttpConstants");
dojo.require("jazz.ensemble.internal.client.RestClient");
dojo.require("jazz.ensemble.internal.util.Xml");
dojo.require("jazz.ensemble.internal.util.XmlBindingFactory");

dojo.requireLocalization("jazz.ensemble.internal", "Messages");

var HttpConstants = jazz.ensemble.internal.client.HttpConstants;
var RestClient = jazz.ensemble.internal.client.RestClient;
var WidgetPanel = jazz.ensemble.WidgetPanel;
var Xml = jazz.ensemble.internal.util.Xml;
var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

var messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "Messages");

/*
 * A WidgetPanel that loads and saves itself to and from a URL via GET and PUT.
 * 
 * NOTE: This internal widget is being used in RTC in the plan dashboard. Do not
 * make any breaking changes without opening an adoption item for RTC.
 * 
 * @see 156713: Expose the WidgetEnsemble as API
 */
dojo.declare("jazz.ensemble.internal.WidgetEnsemble", [WidgetPanel], {
	
	/*
	 * {boolean} Whether or not to automatically save when the state changes.
	 * If false, you must explicitly call the save() method to save.
	 */
	autoSave: true,
	
	/*
	 * {string} The URL to load from and save to for persistence. This is a service
	 * that can handle both GETs and PUTs to load and save, respectively. The service
	 * serves and accepts XML/RDF where the root element of the document represents the
	 * state of the ensemble.
	 */
	url: null,
	
	/*
	 * {function} A callback to be called when an error occurs while loading the
	 * WidgetEnsemble.
	 */
	handleError: function() {},
	
	/*
	 * {function} A callback to be called when an error occurs while saving the
	 * WidgetEnsemble.
	 */
	handleSaveError: function() {},
	
	/*
	 * {function} A callback to be called when the WidgetEnsemble has loaded.
	 */
	onLoad: function() {},
	
	_resource: null,
	_putInTransit: null,
	_autoSavePending: null,
	
	postCreate: function() {
		this.inherited(arguments);
		if (this.autoSave) {
			this.connect(this, "onStateChange", this._autoSave);
		}
		this.refresh();
	},
	
	/*
	 * Refreshes the widget ensemble, replacing the current state with the latest
	 * state from the server.
	 */
	refresh: function() {
		var headers = {};
		headers[HttpConstants.ACCEPT] = HttpConstants.CT_APPLICATION_RDF_XML;
		var dfd = RestClient.getResource(this.url, RestClient.HANDLE_AS_XML, headers);
		dfd.addCallback(dojo.hitch(this, this._handleGet));
		dfd.addErrback(dojo.hitch(this, this._handleError));
	},
	
	/*
	 * Saves the state of the WidgetEnsemble by making a PUT request containing
	 * the new RDF/XML state to the provided URL.
	 * 
	 * @return dojo.Deferred
	 */
	save: function() {
		// to ensure fresh state, only allow one PUT at a time
		if (this._putInTransit) {
			throw new Error(messages.saveInProgress);
		}
		else {
			var dfd = this._doPut();
			dfd.addErrback(dojo.hitch(this, this._handleSaveError));
			dfd.addBoth(dojo.hitch(this, function(obj) {
				this._putInTransit = false;
				return obj;
			}));
			return dfd;
		}
	},
	
	/*
	 * Called when an auto-save is needed.
	 */
	_autoSave: function() {
		// to ensure fresh state, only allow one PUT at a time
		if (this._putInTransit) {
			this._autoSavePending = true;
		}
		else {
			var dfd = this._doPut();
			dfd.addBoth(dojo.hitch(this, this._handleAutoSavePut));
			dfd.addErrback(dojo.hitch(this, this._handleSaveError));
		}
	},
	
	/*
	 * Handles load error responses.
	 */
	_handleError: function(/* Error */ e) {
		if (this.handleError) {
			this.handleError(e);
		}
		else {
			console.error(e);
		}
		return e;
	},
	
	/*
	 * Handles all auto-save PUT responses.
	 */
	_handleAutoSavePut: function(/* any */ obj) {
		this._putInTransit = false;
		if (this._autoSavePending) {
			this._autoSavePending = false;
			this._autoSave();
		}
		return obj;
	},
	
	/*
	 * Handles save error responses.
	 */
	_handleSaveError: function(/* Error */ e) {
		if (this.handleSaveError) {
			this.handleSaveError(e);
		}
		else {
			console.error(e);
		}
		return e;
	},
	
	/*
	 * Handles successful GET requests for the state resource.
	 */
	_handleGet: function(/* object */ resource) {
		this._resource = resource;
		var binding = XmlBindingFactory.createBinding(resource.content.documentElement, SCHEMA);
		var pagesCollection = binding.pages;
		this.setState(pagesCollection.pages[0]);
		this.onLoad && this.onLoad();
		return resource;
	},
	
	/*
	 * Performs a PUT to save the state.
	 */
	_doPut: function() {
		this._putInTransit = true;
		return RestClient.putResource(this._resource);
	}
});

var SCHEMA = [
	{
		// pages collection property
		property: "pages",
		type: XmlBindingFactory.OBJECT,
		nodeType: XmlBindingFactory.CHILD_ELEMENT,
		elemName: "pages",
		elemNS: Xml.namespace.jw,
		schema: [
			{
				// page resources
				property: "pages",
				type: XmlBindingFactory.NODE,
				nodeType: XmlBindingFactory.CHILD_ELEMENT,
				elemName: "Page",
				elemNS: Xml.namespace.jw,
				multiValued: true
			}
		]
	}
];

})();
