/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.settings.SettingsPanel");

dojo.require("jazz.ensemble.internal.settings.SlideoutPanel");
dojo.require("jazz.ui.HoverPopup");
dojo.require("dijit.form.TextBox");

(function() {

var SlideoutPanel = jazz.ensemble.internal.settings.SlideoutPanel;
var HoverPopup = jazz.ui.HoverPopup;

var TYPES = {
	LIST: "LIST",
	BOOLEAN: "BOOLEAN",
	STRING: "STRING",
	HIDDEN: "HIDDEN"
};

/*
 * A slideout panel that contains preferences/settings.
 */
dojo.declare("jazz.ensemble.internal.settings.SettingsPanel", SlideoutPanel, {
	
	/*
	 * A map of objects representing the settings to be displayed. Each object contains
	 * the following properties:
	 * 	- label: The human readable label for the settings. (required)
	 * 	- id: A unique id for the settings. (required)
	 *  - type: A constant from jazz.ensemble.internal.settings.SettingsPanel.TYPES indicating the type of settings. (required)
	 *  - value: The initial value of the setting. (defaults to empty string)
	 * 	- required: A boolean indicating if the settings should be marked as required. (defaults to false)
	 *  - description: A human readable description about the setting to be shown on hover of the label. (optional)
	 */
	settings: null,
	
	readOnly: null,
	
	onChange: function(id, value) {},
	
	onCancel: null,
	
	onOk: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this.oldSettings = dojo.clone(this.settings);
		this.settingsNode = dojo.create("div", {className: "jazz-ensemble-internal-settings-SettingsPanel"}, this.contentNode);
		this._hoverPopups = [];
		this._populateButtonPanel();
		this._populate();
	},
	
	getSettingValue: function(id) {
		return this.settings[this.settingsMap[id]].value;
	},
	
	hasSettingChanged: function(id) {
		var index = this.settingsMap[id];
		return this.settings[index].value !== this.oldSettings[index].value;
	},
	
	_populateButtonPanel: function() {
		dojo.create("div", {className: "required-field", innerHTML: "* " + this._messages.settingsReq}, this._buttonPanel);
		if (!this.readOnly) {
			var okButton = dojo.create("input", {type: "button", value:this._messages.settingsOk}, this._buttonPanel);
			this.connect(okButton, "onclick", this._handleOk);
			var cancelButton = dojo.create("input", {type: "button", value: this._messages.settingsCancel}, this._buttonPanel);
			this.connect(cancelButton, "onclick", this._handleCancel);
		} else {
			var closeButton = dojo.create("input", {type: "button", value: this._messages.settingsClose}, this._buttonPanel);
			this.connect(closeButton, "onclick", this._handleCancel);
		}
	},
	
	_populate: function() {
		this.settingsMap = {};
		var table = dojo.create("table", null, this.settingsNode);
		var tbody = dojo.create("tbody", null, table);
		for(var i = 0; i < this.settings.length; i++) {
			var setting = this.settings[i];
			this.settingsMap[setting.id] = i;
			if (setting.type === TYPES.HIDDEN)
				continue;
			var tr = dojo.create("tr", null, tbody);
			var labelCell = dojo.create("td", {className: "label"}, tr)
			var fieldCell = dojo.create("td", {className: "field"}, tr);
			var field = document.createElement("td");
			field.className="field";
			var content = setting.label.substr(0,1).toUpperCase() + setting.label.substr(1) + (setting.required ? "<span class=\"required-asterisk\">*</span>" : "");
			var label = dojo.create("label", {htmlFor: this.id + "-" + setting.id, innerHTML: content}, labelCell);
			if (setting.description) {
				var popup = new HoverPopup({around: label, html: setting.description, persist: false});
				label.className = "hover-style";
				this._hoverPopups.push(popup);
			}
			this.createFormField(setting, fieldCell);
		}
	},
	
	createFormField: function(setting, field) {
		switch(setting.type) {
		case TYPES.STRING:
			var inputDiv = dojo.create("div", null, field)
			var textBox = new dijit.form.TextBox({name: setting.id, value: setting.value, trim: true, id: this.id + "-" + setting.id});
			dijit.setWaiState(textBox.domNode, "label", setting.label);
			textBox.startup();
			if (this.readOnly || setting.startDisabled)
				textBox.domNode.disabled = true;
			this.connect(textBox.domNode, "onchange", this._handleTextboxChange);
			// workaround for dojo bug: onchange not triggering on blur
			this.connect(textBox.domNode, "onblur", this._handleTextboxChange);
			inputDiv.appendChild(textBox.domNode);
			return textBox.domNode;
		case TYPES.BOOLEAN:
			var inputDiv = dojo.create("div", null, field)
			var input = dojo.create("input", {name: setting.id, type: "checkbox", className: "checkbox", id: this.id + "-" + setting.id}, inputDiv);
			dijit.setWaiState(input, "label", setting.label);
			if (this.readOnly || setting.startDisabled)
				input.disabled = true;
			// use onclick since onchange with checkboxes is buggy in IE
			this.connect(input, "onclick", this._handleCheckboxChange);
			var checked = setting.value === "true" ? true : false;
			if (checked) {
				input.checked = true;
				input.defaultChecked = true;
			}
			return input;
		case TYPES.LIST:
			var inputDiv = dojo.create("div", null, field);
			var select = dojo.create("select",{name: setting.id, id: this.id + "-" + setting.id}, inputDiv);
			if (setting.options)
				this._populateComboOptions(setting.options, select, setting.computedDefault ? setting.computedDefault : setting.value);
			else {
				for (var i = 0; i < setting.groups.length; i++) {
					if (setting.groups[i].name) {
						dojo.create("group", {label: setting.groups[i].name}, select)
						var group = document.createElement("optgroup");
						this._populateComboOptions(setting.groups[i].options, group, setting.value)
					} else
						this._populateComboOptions(setting.groups[i].options, select, setting.value)
				}
			}
			if (this.readOnly || setting.startDisabled)
				select.disabled = true;
			this.connect(select, "onchange", this._handleComboboxChange);
			return select;
		}
	},
	_populateComboOptions: function(options, parent, value) {
		for (var i = 0; i < options.length; i++) {
			var option = dojo.create("option",{value: options[i].value},parent)
			if (options[i].padding)
				option.style.paddingLeft = options[i].padding + "px";
			option.appendChild(document.createTextNode(options[i].label));
			if (value == options[i].value)
				option.selected = "selected";
		}
	},
	_handleTextboxChange: function(event) {
		var box = event.target;
		this.settings[this.settingsMap[box.name]].value = box.value;
		this.onChange(box.name, box.value);
	},
	_handleComboboxChange: function(event) {
		var box = event.target;
		this.settings[this.settingsMap[box.name]].value = box.options[box.selectedIndex].value;
		this.onChange(box.name, box.options[box.selectedIndex].value);
	},
	_handleCheckboxChange: function(event) {
		var box = event.target;
		var value = box.checked ? "true" : "false";
		this.settings[this.settingsMap[box.name]].value = value;
		this.onChange(box.name, value);
	},
	
	_handleOk: function() {
		if (this.onOk) {
			this.onOk();
		}
	},
	
	_handleCancel: function() {
		if (this.onCancel) {
			this.onCancel();
		}
	}, 
	
	destroy: function() {
		while (this._hoverPopups.length > 0)
			this._hoverPopups.pop().destroy();
		this.inherited(arguments);
	}
});
jazz.ensemble.internal.settings.SettingsPanel.TYPES = TYPES;

})();
