/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.util.XmlBindingFactory");

dojo.require("jazz.ensemble.internal.util.Xml");

(function() {

var Xml = jazz.ensemble.internal.util.Xml;

var ATTRIBUTE = 0;
var CHILD_ELEMENT = 1;
var CHILD_ELEMENT_ATTRIBUTE = 2;
var CHILD_ELEMENT_TEXT = 3;

var GET_FUNC = [_getAttributeNode, _getChildElement, _getChildElementAttribute, _getChildElementText];
var GET_FUNC_MULTIVALUED = [/* N/A */ null, _getChildElements, _getChildElementAttributes, _getChildElementTexts];

var SET_FUNC = [_setAttribute, _setChildElement, _setChildElementAttribute, _setChildElementText];
var SET_FUNC_MULTIVALUED = [/* N/A */ null, _setChildElements, _setChildElementAttributes, _setChildElementTexts];

var BOOLEAN = 0;
var NODE = 1;
var NUMBER = 2;
var OBJECT = 3;
var STRING = 4;

var FROM_NODE_FUNC = [_toBoolean, _toNode, _toNumber, _toObject, _toString];
var TO_NODE_FUNC = [null, null, null, _toElement, null];

/*
 * A factory that creates Javascript objects (bindings) with getters, setters, etc.
 * that work with XML DOM elements and attributes, making it easier to work with XML DOM.
 * 
 * Each binding is associated with a single XML DOM element, and has methods to work with
 * attributes of the element, as well as child elements. When creating a binding, you must
 * provide a schema object, which declares which attributes and elements the getters and
 * setters should work with. Getters and setters read and write directly to and from the
 * DOM; there is no intermediate caching, and the changes to the DOM are immediate.
 * 
 * For the read-only case, you can skip the getters altogether and specify properties
 * directly, where the properties will be populated ahead of time.
 * 
 * A schema is an array of entries containing method bindings for the binding object. That
 * is, for each attribute or child element or list of child elements, the method binding
 * declares the name of the attribute/element(s), the namespace, what type of binding it is
 * (i.e. attribute or child element or list of child elements), as well as the Javascript type
 * to use.
 * 
 * Each schema entry is an object with the following properties:
 * {
 *    get: {String} (optional) The name of the getter function to generate, e.g. "getFoo".
 *    set:  {String} (optional) The name of setter function to generate, e.g. "setFoo".
 *    create: {String} (optional) The name of the create function to generate, e.g. "createFoo".
 *    		This applies only to entries with nodeType OBJECT, for creating bindings of child
 *    		elements. This makes it easier to add new children, so that you don't need to create
 *    		a new element then bind it separately in two steps.
 *    property: {String} (optional) The name of the property on the object. This is for the
 *    		read-only case where you don't want a getter, but want it to be pre-populated
 *    		into an attribute. This is equivalent to a getter except that it is read ahead
 *    		of time and cannot be modified later.
 *    type: The Javascript type to use for the getters and setters. Must be one of BOOLEAN,
 *    		NODE, STRING, NUMBER, or OBJECT. For example, if an attribute is a number, you would
 *    		specify NUMBER, and the return value of the getter would be a Javascript number.
 *    		The OBJECT value is used for child bindings (where a child element is yet another
 *    		binding with its own schema). For example, for OBJECT, the getter would return
 *    		a child binding, and the setter would accept a child binding. NODE is used to
 *    		return the XML DOM node itself.
 *    nodeType: The type of node to bind to. Must be one of the following constants:
 *    		ATTRIBUTE: Bind to an attribute of the current element, where the name of the attribute
 *    			is specified in the nodeName (and nodeNS) attribute.
 *    		CHILD_ELEMENT: Bind to one or more child element(s).
 *    		CHILD_ELEMENT_ATTRIBUTE: Bind to one or more child element(s)'s attribute(s).
 *    		CHILD_ELEMENT_TEXT: Bind to one or more child element(s)'s text node(s).
 *    attrName: {String} The local name of the attribute to bind to (without namespace), e.g.
 *    		"myNode". Applicable only when binding to an attribute.
 *    attrNS: {String} (optional) The namespace URI of the attribute to bind to,
 *    		e.g. "http://www.w3.org/XML/1998/namespace", if the attribute has a namespace.
 *    elemName: {String} The local name of the element to bind to (without namespace), e.g.
 *    		"myNode". Applicable only for bindings involving a child element.
 *    elemNS: {String} (optional) The namespace URI of the element to bind to,
 *    		e.g. "http://www.w3.org/XML/1998/namespace", if the node has a namespace.
 *    schema: {object} For OBJECT type and CHILD_ELEMENT nodeType only. Specifies the schema
 *    		of the child element(s) to bind to.
 *    multiValued: {boolean} Whether or not this method binding is multi-valued. Not applicable
 *    		for attributes (since they are single-valued). The getters and setters return and
 *    		expect arrays, where each element of the array is of the type specified in the type
 *    		attribute.
 * }
 * 
 * Bindings will not remove or modify existing attributes or elements not specified in the schema,
 * and will attempt to maintain proper indentation by adding/modifying/removing surrounding text
 * nodes where appropriate.
 * 
 * For example, for the given XML:
 * 
 * 	<book isbn="12345">
 * 		<dc:title>The Book</dc:title>
 * 		<author>John Doe</author>
 * 		<author>Jane Doe</author>
 * 		<dc:publisher rdf:resource="http://publisher.com/abc"/>
 * 	</book>
 * 
 * You can use the following schema:
 * 
 * 	var schema = [
 * 		{
 * 			get: "getISBN",
 * 			set: "setISBN",
 * 			type: XmlBindingFactory.NUMBER,
 * 			nodeType: XmlBindingFactory.ATTRIBUTE,
 * 			attrName: "isbn"
 * 		},
 * 		{
 * 			get: "getTitle",
 * 			set: "setTitle",
 * 			type: XmlBindingFactory.STRING,
 * 			nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
 * 			elemName: "title",
 * 			elemNS: "http://purl.org/dc/terms/"
 * 		},
 * 		{
 * 			get: "getAuthors",
 * 			set: "setAuthors",
 * 			type: XmlBindingFactory.STRING,
 * 			nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
 * 			elemName: "author",
 * 			multiValued: true
 * 		},
 * 		{
 * 			get: "getPublisher",
 * 			set: "setPublisher",
 * 			type: XmlBindingFactory.STRING,
 * 			nodeType: XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
 * 			elemName: "publisher",
 * 			elemNS: "http://purl.org/dc/terms/"
 * 			attrName: "resource",
 * 			attrNS: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 * 		}
 * 	];
 * 
 * This would produce an object with 3 getters and 3 setters, to read and write the isbn
 * attribute, title child element, and author child elements. The following snippet shows
 * how you can create and work with the object.
 * 
 * var bookElem = ... // a reference to the book DOM node
 * var book = XmlBindingFactory.createBinding(bookElem, schema);
 * 
 * // read the isbn
 * var isbn = book.getISBN();
 * 
 * // change the title child element text
 * book.setTitle("Another Title");
 * 
 * // read authors
 * var authors = book.getAuthors();
 * for (var i=0;i<authors.length;i++) {
 *    alert(authors[i]);
 * }
 * 
 * // add a third author
 * authors.push("Third Author");
 * book.setAuthors(authors);
 * 
 * // remove all authors
 * book.setAuthors([]);
 * 
 * // get publisher URL
 * var publisher = book.getPublisher();
 * 
 * // set a new publisher URL
 * book.setPublisher("http://publisher.com/abcdefg");
 * 
 * For the simple read-only case, you can skip the getters and use properties directly,
 * which results in a javascript object with all the desired properties pre-populated.
 * The following schema could be used for the read-only case:
 * 
 * 	var schema_simple = [
 * 		{
 * 			property: "isbn",
 * 			type: XmlBindingFactory.NUMBER,
 * 			nodeType: XmlBindingFactory.ATTRIBUTE,
 * 			attrName: "isbn"
 * 		},
 * 		{
 * 			property: "title",
 * 			type: XmlBindingFactory.STRING,
 * 			nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
 * 			elemName: "title",
 * 			elemNS: "http://purl.org/dc/terms/"
 * 		},
 * 		{
 * 			property: "authors",
 * 			type: XmlBindingFactory.STRING,
 * 			nodeType: XmlBindingFactory.CHILD_ELEMENT_TEXT,
 * 			elemName: "author",
 * 			multiValued: true
 * 		},
 * 		{
 * 			property: "publisher",
 * 			type: XmlBindingFactory.STRING,
 * 			nodeType: XmlBindingFactory.CHILD_ELEMENT_ATTRIBUTE,
 * 			elemName: "publisher",
 * 			elemNS: "http://purl.org/dc/terms/"
 * 			attrName: "resource",
 * 			attrNS: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
 * 		}
 * 	];
 * 
 * Here's an example of how you would use it to read the information:
 * 
 * var bookElem = ... // a reference to the book DOM node
 * var book = XmlBindingFactory.createBinding(bookElem, schema_simple);
 * 
 * // print the isbn and title
 * alert(book.isbn);
 * alert(book.title);
 * 
 * // print the authors
 * for (var i=0;i<book.authors.length;i++) {
 *    alert(book.authors[i]);
 * }
 */
jazz.ensemble.internal.util.XmlBindingFactory = {
	
	/* nodeType constants */
	ATTRIBUTE: ATTRIBUTE,
	CHILD_ELEMENT: CHILD_ELEMENT,
	CHILD_ELEMENT_ATTRIBUTE: CHILD_ELEMENT_ATTRIBUTE,
	CHILD_ELEMENT_TEXT: CHILD_ELEMENT_TEXT,
	
	/* type constants */
	BOOLEAN: BOOLEAN,
	NODE: NODE,
	NUMBER: NUMBER,
	OBJECT: OBJECT,
	STRING: STRING,
	
	/*
	 * Creates a new binding for the given element, using the given schema. The indent param
	 * specifies how much the element should be indented (it is calculated if not specified).
	 */
	createBinding: function(/* Element */ elem, /* object[] */ schema, /* string */ indent) {
		indent = indent || _getIndent(elem);
		var binding = { elem: elem, indent: indent };
		for (var i=0;i<schema.length;i++) {
			var entry = schema[i];
			if (entry.get) {
				binding[entry.get] = _bindGet(binding, entry);
			}
			if (entry.set) {
				binding[entry.set] = _bindSet(binding, entry);
			}
			if (entry.create) {
				binding[entry.create] = _bindCreate(binding, entry);
			}
			if (entry.property) {
				binding[entry.property] = _bindProperty(binding, entry);
			}
		}
		return binding;
	}
};

var XmlBindingFactory = jazz.ensemble.internal.util.XmlBindingFactory;

/*
 * Returns a getter function for the given binding and schema entry.
 */
function _bindGet(/* object */ binding, /* object */ entry) {
	var castFunc = FROM_NODE_FUNC[entry.type];
	if (entry.multiValued) {
		// multi-valued getter
		var getFunc = GET_FUNC_MULTIVALUED[entry.nodeType];
		return function() {
			var array = getFunc.apply(binding, [entry]);
			for (var i=0;i<array.length;i++) {
				array[i] = castFunc(array[i], entry);
			}
			return array;
		}
	}
	// single-valued getter
	var getFunc = GET_FUNC[entry.nodeType];
	return function() {
		var node = getFunc.apply(binding, [entry]);
		if (node) {
			return castFunc(node, entry);
		}
	}
}

/*
 * Returns a setter function for the given binding and schema entry.
 */
function _bindSet(/* object */ binding, /* object */ entry) {
	var toNodeFunc = TO_NODE_FUNC[entry.type];
	if (entry.multiValued) {
		// multi-valued setter
		var setFunc = SET_FUNC_MULTIVALUED[entry.nodeType];
		return function(value) {
			setFunc.apply(binding, [entry, value, toNodeFunc]);
		}
	}
	// single-valued setter
	var setFunc = SET_FUNC[entry.nodeType];
	return function(value) {
		setFunc.apply(binding, [entry, value, toNodeFunc]);
	}
}

/*
 * Returns a creater function for the given binding and schema entry.
 */
function _bindCreate(/* object */ binding, /* object */ entry) {
	return function() {
		var childElem = Xml.createElementNS(this.elem.ownerDocument, entry.elemNS, entry.elemName);
		return XmlBindingFactory.createBinding(childElem, entry.schema, this.indent + "\t");
	}
}

/*
 * Returns the value of the requested property. This is for the simple read-only case
 * where you don't want getters, but just want simple properties on an object that are
 * pre-populated. Instead of returning a getter like _bindGet, it just returns the
 * value that the getter would return if called.
 */
function _bindProperty(/* object */ binding, /* object */ entry) {
	return _bindGet(binding, entry)();
}

function _getAttributeNode(/* object */ entry) {
	return Xml.getAttributeNodeNS(this.elem, entry.attrNS, entry.attrName);
}

function _getChildElement(/* object */ entry) {
	return Xml.getChildByNameNS(this.elem, entry.elemNS, entry.elemName);
}

function _getChildElements(/* object */ entry) {
	return Xml.getChildrenByNameNS(this.elem, entry.elemNS, entry.elemName);
}

function _getChildElementAttribute(/* object */ entry) {
	var childElem = Xml.getChildByNameNS(this.elem, entry.elemNS, entry.elemName);
	if (childElem) {
		return Xml.getAttributeNodeNS(childElem, entry.attrNS, entry.attrName);
	}
}

function _getChildElementText(/* object */ entry) {
	var childElem = Xml.getChildByNameNS(this.elem, entry.elemNS, entry.elemName);
	if (childElem) {
		if (childElem.childNodes.length > 1) {
			childElem.normalize();
		}
		return childElem.firstChild;
	}
}

function _getChildElementTexts(/* object */ entry) {
	var nodes = Xml.getChildrenByNameNS(this.elem, entry.elemNS, entry.elemName);
	for (var i=0;i<nodes.length;i++) {
		var node = nodes[i];
		if (node.childNodes.length > 1) {
			node.normalize();
		}
		nodes[i] = node.firstChild;
	}
	return nodes;
}

function _getChildElementAttributes(/* object */ entry) {
	var nodes = Xml.getChildrenByNameNS(this.elem, entry.elemNS, entry.elemName);
	for (var i=0;i<nodes.length;i++) {
		nodes[i] = Xml.getAttributeNodeNS(nodes[i], entry.attrNS, entry.attrName);
	}
	return nodes;
}

/*
 * Computes and returns the given element's indentation string, by looking at
 * its preceding text node.
 */
function _getIndent(/* Element */ elem) {
	var indent = "";
	var parentNode;
	while ((parentNode = elem.parentNode) && parentNode.nodeType == Xml.NodeType.ELEMENT) {
		indent += "\t";
		elem = parentNode;
	}
	return indent;
}

function _setAttribute(/* object */ entry, /* any */ value) {
	var name = entry.attrName;
	var ns = entry.attrNS;
	if (value != null) {
		Xml.setAttributeNS(this.elem, ns, name, value + "");
	}
	else {
		return Xml.removeAttributeNS(this.elem, ns, name);
	}
}

function _setChildElement(/* object */ entry, /* any */ value, /* function */ toNodeFunc) {
	var elem = this.elem;
	var name = entry.elemName;
	var ns = entry.elemNS;
	var childElem = toNodeFunc ? toNodeFunc(value) : value;
	if (childElem) {
		var oldElem = Xml.getChildByNameNS(elem, ns, name);
		if (oldElem) {
			if (oldElem !== childElem) {
				elem.replaceChild(childElem, oldElem);
			}
		}
		else {
			_appendChild(elem, childElem, this.indent);
		}
	}
	else {
		var oldElems = Xml.getChildrenByNameNS(elem, ns, name);
		for (var i=0;i<oldElems.length;i++) {
			_removeChild(elem, oldElems[i], this.indent);
		}
	}
}

function _setChildElementText(/* object */ entry, /* any */ value) {
	var elem = this.elem;
	var name = entry.elemName;
	var ns = entry.elemNS;
	var childElem = null;
	if (value != null) {
		childElem = Xml.getChildByNameNS(elem, ns, name);
		if (!childElem) {
			childElem = Xml.createElementNS(elem.ownerDocument, ns, name);
		}
		Xml.setTextValue(childElem, value + "");
	}
	_setChildElement.apply(this, [entry, childElem]);
}

function _setChildElementAttribute(/* object */ entry, /* any */ value) {
	var elem = this.elem;
	var childElem = null;
	if (value != null) {
		childElem = Xml.getChildByNameNS(elem, entry.elemNS, entry.elemName);
		if (!childElem) {
			childElem = Xml.createElementNS(elem.ownerDocument, entry.elemNS, entry.elemName);
		}
		Xml.setAttributeNS(childElem, entry.attrNS, entry.attrName, value);
	}
	_setChildElement.apply(this, [entry, childElem]);
}

function _setChildElements(/* object */ entry, /* any[] */ values, /* function */ toNodeFunc) {
	var elem = this.elem;
	var name = entry.elemName;
	var ns = entry.elemNS;
	var doc = elem.ownerDocument;
	var oldElems = Xml.getChildrenByNameNS(elem, ns, name);
	var refNode = (oldElems.length > 0) && oldElems[oldElems.length-1].nextSibling;
	// remove old elems
	for (var i=0;i<oldElems.length;i++) {
		_removeChild(elem, oldElems[i], this.indent);
	}
	// add new elems
	for (var i=0;i<values.length;i++) {
		var childElem = toNodeFunc ? toNodeFunc(values[i]) : values[i];
		refNode ? _insertBefore(elem, childElem, refNode, this.indent) : _appendChild(elem, childElem, this.indent);
	}
}

function _setChildElementTexts(/* object */ entry, /* any[] */ values) {
	var childElems = [];
	for (var i=0;i<values.length;i++) {
		var childElem = Xml.createElementNS(this.elem.ownerDocument, entry.elemNS, entry.elemName);
		Xml.setTextValue(childElem, values[i] + "");
		childElems.push(childElem);
	}
	_setChildElements.apply(this, [entry, childElems]);
}

function _setChildElementAttributes(/* object */ entry, /* any[] */ values) {
	var childElems = [];
	for (var i=0;i<values.length;i++) {
		var childElem = Xml.createElementNS(this.elem.ownerDocument, entry.elemNS, entry.elemName);
		Xml.setAttributeNS(childElem, entry.attrNS, entry.attrName, values[i] + "");
		childElems.push(childElem);
	}
	_setChildElements.apply(this, [entry, childElems]);
}

/*
 * A pretty-print version of appendChild that also adds/modifies surrounding text nodes
 * to give the element proper indentation and formatting.
 * 
 * Note: IE doesn't create text nodes for whitespace, and automatically tries to
 * format parsed nodes with proper indentation, but not newly added nodes.
 */
function _appendChild(/* Element */ elem, /* Element */ childElem, /* string */ indent) {
	var doc = elem.ownerDocument;
	var lastChild = elem.lastChild;
	if (lastChild && lastChild.nodeType == Xml.NodeType.TEXT) {
		var lastText = lastChild.nodeValue;
		lastChild.nodeValue = "\n" + indent + "\t";
		elem.appendChild(childElem);
		elem.appendChild(doc.createTextNode(lastText));
	}
	else {
		elem.appendChild(doc.createTextNode("\n" + indent + "\t"));
		elem.appendChild(childElem);
		elem.appendChild(doc.createTextNode("\n" + indent));
	}
}

/*
 * A pretty-print version of insertBefore that also adds/modifies surrounding text nodes
 * to give the element proper indentation and formatting.
 * 
 * Note: IE doesn't create text nodes for whitespace, and automatically tries to
 * format parsed nodes with proper indentation, but not newly added nodes.
 */
function _insertBefore(/* Element */ elem, /* Element */ childElem, /* Node */ refNode, /* string */ indent) {
	var doc = elem.ownerDocument;
	var prevNode = refNode.previousSibling;
	if (prevNode && prevNode.nodeType == Xml.NodeType.TEXT) {
		prevNode.nodeValue = "\n" + indent + "\t";
	}
	else {
		elem.insertBefore(doc.createTextNode("\n" + indent + "\t"), refNode);
	}
	elem.insertBefore(childElem, refNode);
	if (refNode.nodeType != Xml.NodeType.TEXT) {
		elem.insertBefore(doc.createTextNode("\n" + indent + "\t"), refNode);
	}
}

/*
 * A pretty-print version of removeChild that also modifies/removes surrounding text nodes
 * to give the resulting DOM proper indentation and formatting.
 * 
 * Note: IE doesn't create text nodes for whitespace, and automatically tries to
 * format parsed nodes with proper indentation, but not newly added nodes.
 */
function _removeChild(/* Element */ elem, /* Element */ childElem, /* string */ indent) {
	var text = childElem.previousSibling;
	if (text && text.nodeType == Xml.NodeType.TEXT) {
		elem.removeChild(text);
	}
	elem.removeChild(childElem);
}

function _toBoolean(/* Node */ node) {
	return node.nodeValue.toLowerCase() == "true";
}

function _toElement(/* object */ bindingObj) {
	return bindingObj && bindingObj.elem;
}

function _toNode(/* Node */ node) {
	return node;
}

function _toNumber(/* Node */ node) {
	var str = node.nodeValue;
	if (str.indexOf(".") != -1) {
		return parseFloat(str);
	}
	return parseInt(str);

}

function _toObject(/* Node */ node, /* object */ entry) {
	return XmlBindingFactory.createBinding(node, entry.schema);
}

function _toString(/* Node */ node) {
	return node.nodeValue;
}

})();
