/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("jazz.core.loader");

dojo.require("dojox.collections.ArrayList");
dojo.require("dojox.collections.Dictionary");

jazz.core.loader.CurrentlyLoading = new dojox.collections.Dictionary();

jazz.core.loader.load_async = function(/*String*/module, /*Function*/cb) {
	// summary:
	//		Asynchronously loads the specified module and all of the modules
	//      it depends on that are not already loaded.
	// description: 
	//		In order to provide a responsive user interface, the Jazz web UI
	//      attempts to load the minimal amount of code necessary to satisfy a
	//      particular use case, and attempts to minimize the number of server
	//      round trips.  The 'load_async' method requests a set of modules using
	//      the following algorithm:
	//
	//      1) Include the specified module and all of the modules it
	//         transitively depends on.
	//      2) From the set of modules determined by step 1, subtract any
	//         modules that are alrady loaded.
	//      3) Makes an XHR request to load the modules
	//      4) Calls the callback function
	//
	// module: 
	//		The id of a module.
	// cb: 
	//		The callback function to be called once the module has been loaded.
	// examples:
	//		load_async("com.ibm.team.workitem.web.WorkItemPage", function() {...});
	// since:
	//		0.7
	
	if (!dojo.isString(module)) {
		throw new Error("module argument must be a String");	
	}
	
	if (!dojo.isFunction(cb)) {
		throw new Error("cb argument must be a Function");	
	}
	
	var jsModule = dojo.dependency.moduleGraph.item(module);
	if (jsModule && jsModule.l === true) {
		jazz.core.loader._findAndLoadStyleSheet(module);
		cb();
		return;
	}
	
	var callbackList;
	var ids = new dojox.collections.ArrayList();
	jazz.core.loader._getFullDependencySet(jsModule.i, ids, new dojox.collections.ArrayList());
	
	if (jazz.core.loader.CurrentlyLoading.containsKey(module)) {
		//console.debug("load already in progress for ["+module+"] callback added to list");
		callbackList = jazz.core.loader.CurrentlyLoading.item(module).callbackList;
		callbackList.add(cb);
		return;
	} else {
		var keyList = jazz.core.loader.CurrentlyLoading.getKeyList();
		//console.debug("trying to load ["+module+"]");
		for (var i = 0; i < keyList.length; i++) {
			var currentlyLoadingId = keyList[i];
			//console.debug("load in progress for ["+currentlyLoadingId+"]");
			var callbackDetails = jazz.core.loader.CurrentlyLoading.item(currentlyLoadingId);
			if (jazz.core.loader._findCommonDependencies(currentlyLoadingId, ids)) {
				//console.debug("common dependency found load of module ["+module+"] will be deferred");

				var dependencyCallback = {id : module, callback: cb};
				callbackDetails.callbackDependencyList.add(dependencyCallback);
				return;
			}
		}
	}
	
	callbackList = new dojox.collections.ArrayList();
	callbackList.add(cb);
	var callbackDependencyList = new dojox.collections.ArrayList();
	callbackDetails = {callbackList : callbackList, callbackDependencyList : callbackDependencyList};
	jazz.core.loader.CurrentlyLoading.add(module, callbackDetails);
	
	if (window.location.search.indexOf("debug=true") != -1) {
		jazz.core.loader._findAndLoadStyleSheet(module);
		itr = ids.getIterator();
		
		var f = function(itr){
			if (!itr.atEnd()) {
				var nextId = itr.get();
				jsModule = dojo.dependency.indexGraph.item(nextId);
				jsModule.l = true;
				var syms = dojo._getModuleSymbols(jsModule.m);
				var relpath = syms.join("/") + '.js';
				var url = dojo.baseUrl + relpath + "?debug=true";
				var element = dojo.doc.createElement("script");
				element.type = "text/javascript";
				element.src = url;
				element.charset = "utf-8";
				element.onloadDone = false;
				element.onload = function() {
					if (!element.onloadDone) {
						element.onloadDone = true;
						f(itr);
					}
				};
				element.onreadystatechange = function(){
					if (("loaded" === element.readyState || "complete" === element.readyState) && !element.onloadDone) {
						element.onload();
					}
				};
				dojo.doc.getElementsByTagName("head")[0].appendChild(element);
			} else {
				var callbackItr = callbackList.getIterator();
				while (!callbackItr.atEnd()) {
					var callback = callbackItr.get();
					callback();
				}
				var dependencyCallbackItr = callbackDependencyList.getIterator();
				while (!dependencyCallbackItr.atEnd()) {
					dependencyCallback = dependencyCallbackItr.get();
					//console.debug("retrying load for ["+dependencyCallback.id+"]");
					jazz.core.loader.load_async(dependencyCallback.id, dependencyCallback.callback);
				}
				jazz.core.loader.CurrentlyLoading.remove(module);
			}
		};
		
		f(itr);
		return;
	}
	
	var syms = dojo._getModuleSymbols(module);
	var relpath = syms.join("/") + '.js';
	
	var unprefixedUri = relpath;
	if (unprefixedUri.search(/..\//) === 0) {
		unprefixedUri = unprefixedUri.substring(2);
	}	
	
	var suffix = dojo.dependency.getDependencyUri(unprefixedUri);
	
	if (window.location.search.indexOf("debugDojo=true") != -1) {
		suffix == "" ? suffix = "" : suffix += "&debugDojo=true";
	}
	
	if (dojo.baseUrl.indexOf("proxy") !== -1) {
		suffix = encodeURIComponent(suffix);
	}
	
	var bindArgs = {
		url: dojo.baseUrl + relpath + suffix,
		handleAs: "text",
		handle: function(response, ioArgs) {
			callbackDetails = jazz.core.loader.CurrentlyLoading.item(module);
			jazz.core.loader.CurrentlyLoading.remove(module);
			if (response instanceof Error) {
				//console.debug("xhr failed for ["+module +"] ["+ioArgs.xhr.status+"] ["+ioArgs.xhr.statusText+"]");
				throw new Error("Failed to asynchronously load module ["+module+"] ["+ioArgs.xhr.status+"] ["+ioArgs.xhr.statusText+"]");
			}
			else {
				//console.debug("xhr for ["+module +"] complete");

				try {
					jazz.core.loader._findAndLoadStyleSheet(module);
					jazz.core.loader._waitForCSS(jazz.core.loader._getLastStylesheet(), -1);
					dojo["eval"](response);
					var callbackItr = callbackDetails.callbackList.getIterator();
					while (!callbackItr.atEnd()) {
						var callback = callbackItr.get();
						//console.debug("calling callback for ["+module+"]");
						callback();
					}
					var dependencyCallbackItr = callbackDetails.callbackDependencyList.getIterator();
					while (!dependencyCallbackItr.atEnd()) {
						dependencyCallback = dependencyCallbackItr.get();
						//console.debug("retrying load for ["+dependencyCallback.id+"]");
						jazz.core.loader.load_async(dependencyCallback.id, dependencyCallback.callback);
					}
				}
				catch (e) {
					//console.debug("error :"+e);
					throw new Error("Failed to asynchronously load module ["+module+"] eval failed ["+e+"]");
				}
			}
		}
	};
	//console.debug("xhr for ["+module +"] started");
	var xhr = dojo.xhrGet(bindArgs);
};

jazz.core.loader.batch_load_async = function(/*Array*/modules, /*Function*/ cb){
	// summary:
	//		Asynchronously loads the specified array of modules and all of the modules
	//      they depend on that are not already loaded.
	// description: 
	//		In order to provide a responsive user interface, the Jazz web UI
	//      attempts to load the minimal amount of code necessary to satisfy a
	//      particular use case, and attempts to minimize the number of server
	//      round trips.  The 'batch_load_async' method requests a set of modules using
	//      the following algorithm:
	//
	//      1) Include the specified array of modules and all of their modules that they
	//         transitively depends on.
	//      2) From the set of modules determined by step 1, subtract any
	//         modules that are alrady loaded.
	//      3) Makes an XHR request to load the modules
	//      4) Calls the callback function
	//
	// module: 
	//		The id of a module.
	// cb: 
	//		The callback function to be called once the modules have been loaded.
	// examples:
	//		batch_load_async(["com.ibm.team.dashboard.web.Viewlet1","com.ibm.team.dashboard.web.Viewlet2"], function() {...});
	
	if (!dojo.isArray(modules)) {
		throw new Error("modules argument must be an Array");	
	}
	if (!dojo.isFunction(cb)) {
		throw new Error("cb argument must be a Function");	
	}
	
	if (window.location.search.indexOf("debug=true") != -1) {
		for (var j = 0; j < modules.length; j++) {
			eval("do" + "jo.require(\"" + modules[j] + "\")");
		}
		jazz.core.loader._findAndLoadStyleSheet(modules);
		cb();
		return;
	}
	
	var seen = new dojox.collections.ArrayList();	
	var dependencyUri = "";
	var id = "";
	
	for (var i = 0; i < modules.length; i++) {
		var jsModule = dojo.dependency.moduleGraph.item(modules[i]);
		if (jsModule && jsModule.l === false) {
			id += jsModule.i;
			dependencyUri += dojo.dependency.findDependency(jsModule, seen);
			dependencyUri += jsModule.i;
		}
	}
	if (id === "") {
		cb();
		return;
	}

	if (window.location.search.indexOf("debugDojo=true") != -1) {
		dependencyUri += "&debugDojo=true";
	}
	
	seen = new dojox.collections.ArrayList();	
	var versionSeen = new dojox.collections.ArrayList();	
	var version = "";
	for (i = 0; i < modules.length; i++) {
		jsModule = dojo.dependency.moduleGraph.item(modules[i]);
		if (jsModule && jsModule.l === false) {
			version += dojo.dependency.resolveVersion(jsModule, seen, versionSeen);
		}
	}
	
	var syms = dojo._getModuleSymbols("jazz.core.internal._internalBatchLoader");
	var relpath = syms.join("/") + '.js';
	
	var queryString = "?ver=" + version + "&id="+id+"&loc=" + dojo.locale + "&dep=" + dependencyUri;
	if (dojo.baseUrl.indexOf("proxy") !== -1) {
		queryString = encodeURIComponent(queryString);
	}
	
	var bindArgs = {
		url: dojo.baseUrl + relpath + queryString,
		handleAs: "text",
		handle: function(response, ioArgs) {
			if (response instanceof Error) {
				throw new Error("Failed to asynchronously load modules ["+modules+"] ["+ioArgs.xhr.status+"] ["+ioArgs.xhr.statusText+"]");
			}
			else {
				try {
					jazz.core.loader._findAndLoadStyleSheet(modules);
					jazz.core.loader._waitForCSS(jazz.core.loader._getLastStylesheet(), -1);
					dojo["eval"](response);
					cb();
				}
				catch (e) {
					throw new Error("Failed to asynchronously load module ["+modules+"] eval failed ["+e+"]");
				}
			}
		}
	};
	var xhr = dojo.xhrGet(bindArgs);
};

jazz.core.loader.load_sync = function(/*String*/module) {
	// summary:
	//		Synchronously loads the specified module and all of the modules
	//      it depends on that are not already loaded.
	// description: 
	//		In order to provide a responsive user interface, the Jazz web UI
	//      attempts to load the minimal amount of code necessary to satisfy a
	//      particular use case, and attempts to minimize the number of server
	//      round trips.  The 'load_sync' method requests a set of modules using
	//      the following algorithm:
	//
	//      1) Include the specified module and all of the modules it
	//         transitively depends on.
	//      2) From the set of modules determined by step 1, subtract any
	//         modules that are alrady loaded.
	//
	//      Note: 'load_sync' results in a synchonous XHR request; this can
	//      result in the browser appearing to lock up over high-latency
	//		connections.
	// module: 
	//		The id of a module.
	// examples:
	//		load_sync("com.ibm.team.workitem.web.WorkItemPage");
	
	jazz.core.loader._findAndLoadStyleSheet(module);
	eval("do" + "jo.require(\"" + module + "\")");
};


jazz.core.loader._findUnloadedCSS = function(jsModule, cssURIList, seen) {
	if (jsModule.c) {
		for (var i = 0; i < jsModule.c.length; i++)	{
			var cssModule = net.jazz.ajax.core.internal.style.CSSModuleMap.findCSSModuleByIndex(jsModule.c[i]);
			if (cssURIList.contains(jsModule.c[i]) === false && cssModule.l === false) {
				cssURIList.add(jsModule.c[i]);
				cssModule.l = true;
				for (var j = 0; j < cssModule.d.length; j++) {
					var importedCSSModule = net.jazz.ajax.core.internal.style.CSSModuleMap.findCSSModuleByIndex(cssModule.d[j]);
					if (cssURIList.contains(cssModule.d[j]) === false && importedCSSModule.l === false) {
						jazz.core.loader._findUnloadedImportedCSS(importedCSSModule, 
						                                          cssURIList, 
																  new dojox.collections.ArrayList());	
					}
				}
			}
		}	
	}
	for (var j = 0; j < jsModule.d.length; j++) {
		var dependency = dojo.dependency.indexGraph.item(jsModule.d[j]);
		if (dependency && seen.contains(jsModule.d[j]) === false) {
			seen.add(jsModule.d[j]);
			jazz.core.loader._findUnloadedCSS(dependency, cssURIList, seen);
		}
	}
};

jazz.core.loader._findUnloadedImportedCSS = function(cssModule, cssURIList, seen) {
	for (var i = 0; i < cssModule.d.length; i++) {
		var importedCSSModule = net.jazz.ajax.core.internal.style.CSSModuleMap.findCSSModuleByIndex(cssModule.d[i]);
		if (importedCSSModule && 
		    cssURIList.contains(importedCSSModule.i) === false && 
			seen.contains(importedCSSModule.i) === false &&
			importedCSSModule.l === false) {
			seen.add(importedCSSModule.i);
			jazz.core.loader._findUnloadedImportedCSS(importedCSSModule, cssURIList, seen);
		}	
	}
	cssURIList.add(cssModule.i);
	cssModule.l = true;
};

jazz.core.loader._loadStyleSheet = function(styleURLSuffix) {
	var styleURL = net.jazz.ajax._contextRoot + 
	               net.jazz.ajax._webuiPrefix + 
				   "_style/"+
				   dojo.dependency.getFullVersion()+
				   "/" +
				   styleURLSuffix;
				   
	var queryString = "?importDojoCSS=false";
				   
	if (window.location.search.indexOf("debug=true") != -1) {
		queryString += "&debug=true";
	}
	
	if (dojo.baseUrl.indexOf("proxy") !== -1) {
		queryString += "&_proxyURL="+encodeURIComponent(net.jazz.ajax._contextRoot);
	}
	styleURL += queryString;
	if (!document.createStyleSheet) {
		var styleElement = document.createElement("link");
		styleElement.rel = "stylesheet";
		styleElement.type = "text/css";
		styleElement.href = styleURL;							
		document.getElementsByTagName("head")[0].appendChild(styleElement);
	} else {
		var styleSheet = null;
		if (document.styleSheets.length > 31) {
			for (var i = 0; i < document.styleSheets.length; i++) {
				if (djConfig.isDebug) {	console.debug("stylesheet ["+i+"] has "+document.styleSheets[i].imports.length+" imports"); }
				if (document.styleSheets[i].imports.length < 32) {
					styleSheet = document.styleSheets[i];
					break;
				}
			}
			if (styleSheet !== null) {
				styleSheet.addImport(styleURL);	
			} else {
				throw new Error("No stylesheet is available to add an import to");	
			}	
		} else {
			styleSheet = document.createStyleSheet(styleURL);
			if (djConfig.isDebug) { console.debug("Created new stylesheet. Number of current stylesheets is "+document.styleSheets.length); }
		}
	}
};

jazz.core.loader._findAndLoadStyleSheet = function(modules){
	var cssURIList = new dojox.collections.ArrayList();
	var styleURLSuffix = "";		

	var seen = new dojox.collections.ArrayList();

	var _modules = modules;
	if (!dojo.isArray(modules)) {
		_modules = [modules];
	}
	for (var i = 0; i < _modules.length; i++) {
		var jsModule = dojo.dependency.moduleGraph.item(_modules[i]);
		if (jsModule) {
			jazz.core.loader._findUnloadedCSS(jsModule, cssURIList, seen);
		}
	}
	var cssURIs = cssURIList.toArray();
	for (i = 0; i < cssURIs.length; i++) {
		styleURLSuffix += cssURIs[i];	
	}	
	if (styleURLSuffix !== "") {
		jazz.core.loader._loadStyleSheet(styleURLSuffix);
	}
};

jazz.core.loader._waitForCSS = function(styleSheet, lastRuleCount){
	try {
		var cssRules = document.createStyleSheet ? styleSheet.rules : styleSheet.cssRules;
		if (cssRules.length > lastRuleCount) {
			setTimeout(function() {
				jazz.core.loader._waitForCSS(styleSheet, cssRules.length);
			}, 200);
		}
	}
	catch (exc) {
		setTimeout(function() {
			jazz.core.loader._waitForCSS(styleSheet, -1);
		}, 200);
	}
};

jazz.core.loader._getLastStylesheet = function() {
	var styleSheet = null;
	if (!document.createStyleSheet) {
		styleSheet = document.styleSheets[document.styleSheets.length - 1];
	} else {
		if (document.styleSheets.length > 31) {
			for (var i = 0; i < document.styleSheets.length; i++) {
				if (document.styleSheets[i].imports.length < 32) {
					styleSheet = document.styleSheets[i];
					break;
				}
			}
		}
		else {
			styleSheet = document.styleSheets[document.styleSheets.length - 1];
		}
	}
	return styleSheet;	
};

jazz.core.loader._getFullDependencySet = function(moduleIndex, ids, seen) {
	var jsModule = dojo.dependency.indexGraph.item(moduleIndex);
	if (jsModule && jsModule.l === false && seen.contains(moduleIndex) === false) {
		seen.add(moduleIndex);
		for (var i = 0; i < jsModule.d.length; i++) {
			jazz.core.loader._getFullDependencySet(jsModule.d[i], ids, seen);
		}
		ids.add(moduleIndex);
	}
};

jazz.core.loader._findCommonDependencies = function(currentlyLoadingId, ids) {
	var currentlyLoadingIds = new dojox.collections.ArrayList();
	var jsModule = dojo.dependency.moduleGraph.item(currentlyLoadingId);
	jazz.core.loader._getFullDependencySet(jsModule.i, currentlyLoadingIds, new dojox.collections.ArrayList());
	
	var itr = ids.getIterator();
	while (!itr.atEnd()) {
		var id = itr.get();
		if (currentlyLoadingIds.contains(id)) {
			var dependencyJSModule = dojo.dependency.indexGraph.item(id);
			//console.debug("common dependency found ["+dependencyJSModule.m+"]");
			return true;
		}
	}
	return false;
};
})();