/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.declare("com.ibm.team.repository.web.transport.ServiceResponseHandler", null, {

	// -------------------------------------------------------------------------
	// Constructor
	// -------------------------------------------------------------------------
	/**
	 * An container for the object and functions required to handle an
	 * asyncronous service response.
	 * <p>
	 * Example usage:
	 * </p>
	 * <pre>
	 *   var obj= {
	 *       mySuccessFunction: function(returnedStuff) {
	 *           // do something
	 *       },
	 *       myErrorFunction: function(errorType, errorMessage) {
	 *           // do something with the error type and message
	 *       }
	 *   };
	 *   var myServiceResponseHandler=
	 *        new com.ibm.team.repository.web.transport.ServiceResponseHandler(
	 *                obj,
	 *                "mySuccessFunction",
	 *                "myErrorFunction");
	 * </pre>
	 *
	 * @param contextObject a valid object containing the functions named by
	 *  <code>callbackName</code> and <code>errorCallbackName</code>.
	 * @param callbackName the name of a function existing in the context of
	 *  <code>contextObject</code>, represented as a string.  This function will
	 *  be invoked upon an successful service request.
	 * @param errorCallbackName the name of a function existing in the context
	 *  of <code>contextObject</code>, represented as a string.  This function
	 *  will be invoked upon an unsuccessful service request.
	 *
	 * @throws Exception if parameters count and types do not conform to the
	 *  specification above.
	 *  
	 *  @since 0.5
	 */
	constructor: function(contextObject, callbackName, errorCallbackName) {
		if (arguments.length != 3) {
			throw new Error("com.ibm.team.repository.web.transport.ServiceResponseHandler(): invalid number of parameters: " + arguments.length);
		}
		
	    if (!dojo.isObject(contextObject) ||
				!dojo.isFunction(contextObject[callbackName]) ||
	        	!dojo.isFunction(contextObject[errorCallbackName])) {
	    	throw new Error("com.ibm.team.repository.web.transport.ServiceResponseHandler(): invalid parameter types.  Expected (object contextObject, string successFunctionName, string errorFunctionName).");
		}
		
		this._contextObject= contextObject;
		this._callbackName= callbackName;
		this._errorCallbackName= errorCallbackName;
	},
	
	/**
	 * Performs a callback to this <code>ServiceReponseHandler</code>'s success
	 * method.  This method is generally intended to be called by the web client
	 * framework, but it is sometimes useful to intercept, inspect, and perhaps 
	 * filter responses and then call back to the original caller.
	 * 
	 * @since 0.5
	 */
	doCallback: function(returnVal) {
		var obj= this._contextObject;
		obj[this._callbackName].call(obj, returnVal);
	},
	
	/**
	 * Performs a callback to this <code>ServiceReponseHandler</code>'s error
	 * method.  This method is generally intended to be called by the web client
	 * framework, but it is sometimes useful to intercept, inspect, and perhaps 
	 * filter responses and then call back to the original caller.
	 * 
	 * @since 0.5
	 */
	doErrorCallback: function(errorObj) {
		var obj= this._contextObject;
		obj[this._errorCallbackName].call(obj, errorObj);
	}
});
