/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("jazz.app.ua.SectionHoverHelp");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");

dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.ua.internal.HelpHoverPopup");

/*
 * jazz.app.ua.SectionHoverHelp
 * 
 * This widget provides a hover-over help icon that is meant to provide section-level
 * help.  To prevent cluttering the interface, this widget should only be visible when
 * the user's focus is on the section this help widget is associated with.  The widget
 * is typically placed at the far right of the section header.
 * 
 * To enable help in a Jazz Team Server, add the <jd:infoCenter> property to your
 * root services document.  For example:
 * 
 * <jd:infocenterRoot rdf:resource="http://publib.example.com/infocenter/rtc/v1r0m1" />
 * 
 * @since 0.7
 */

dojo.declare("jazz.app.ua.SectionHoverHelp", [dijit._Widget, dijit._Templated], {
	templateString: "<span><img src='${_imageSrc}' dojoAttachPoint='_helpImg' style='margin-right:5px'></img></span>",
	_imageSrc: dojo.moduleUrl("jazz.app", "ua/images/sctn-help.gif"),
	_imageHoverSrc: dojo.moduleUrl("jazz.app", "ua/images/sctn-help-hov.gif"),
	
	/*
	 * The constructor expects one of uaKey, contentNode, or contentString to be defined.
	 * 
	 * uaKey is a string that is a key into the information center.  If uaKey is defined, the
	 * widget will retrieve help content from the information center based on that key.
	 * 
	 * contentNode is a DOM node.  If contentNode is specified, the help content that is displayed
	 * on hover will be the contents of this DOM node.
	 * 
	 * contentString is a string.  If contentString is defined, then the literal contents of that
	 * string will be displayed on hover.  No custom styling can be provided when using contentString.
	 * If any HTML markup is required, use contentNode instead.
	 * 
	 * showOnHoverOver is an array of DOM nodes.  The help icon will be displayed when the mouse is
	 * hovering over any one of the DOM nodes in this array
	 * 
	 * showOnFocus is an array of input elements.  The help icon will be displayed when any of these
	 * input elements are in focus.
	 */
	
	constructor: function (args) {
		this.uaKey = args.uaKey;
		this.showOnHoverOver = args.showOnHoverOver;
		this.showOnFocus = args.showOnFocus;
		this.contentNode = args.contentNode;
		this.contentString = args.contentString;
	},
	postCreate: function () {
		this.connect(this._helpImg, "onmouseover", function (event) {
			event.currentTarget.src = this._imageHoverSrc;
		});
		this.connect(this._helpImg, "onmouseout", function (event) {
			event.currentTarget.src = this._imageSrc;
		});
		if (this.uaKey) {
			this.hoverPopup = new jazz.app.ua.internal.HelpHoverPopup({
				uaKey: this.uaKey,
				around: this._helpImg
			});
		} else {
			if (!this.contentNode) {
				this.contentNode = document.createElement("div");
				this.contentNode.appendChild(document.createTextNode(this.contentString));
			}
			this.hoverPopup = new jazz.ui.HoverPopup({
				around: this._helpImg,
				createContents: dojo.hitch(this, function (popup) {
					return this.contentNode;
				})
			});
		}
		if (this.showOnHoverOver || this.showOnFocus) {
			dojo.toggleClass(this.domNode, "hidden", true);
			this.connect(this.hoverPopup, "open", this._handlePopupOpen);
			this.connect(this.hoverPopup, "close", this._handlePopupClose);
		}
		if (this.showOnHoverOver) {
			for (var i=0; i < this.showOnHoverOver.length; i++) {
				this.connect(this.showOnHoverOver[i], "onmouseover", this._handleMouseOver);
				this.connect(this.showOnHoverOver[i], "onmouseout", this._handleMouseOut);
			}
		}
		if (this.showOnFocus) {
			for (var i=0; i < this.showOnFocus.length; i++) {
				this.connect(this.showOnFocus[i], "onfocus", this._handleFocus);
				this.connect(this.showOnFocus[i], "onblur", this._handleBlur);
			}
		}
	},
	/*
	 * internal methods
	 */
	_handleMouseOver: function (event) {
		event.currentTarget._hovering = true;
		this._updateShowHide();
	},
	_handleMouseOut: function (event) {
		event.currentTarget._hovering = false;
		this._updateShowHide();
	},
	_handleFocus: function (event) {
		event.currentTarget._focused = true;
		this._updateShowHide();
	},
	_handleBlur: function (event) {
		event.currentTarget._focused = false;
		this._updateShowHide();
	},
	_handlePopupOpen: function () {
		this._popupOpen = true;
		this._updateShowHide();
	},
	_handlePopupClose: function () {
		this._popupOpen = false;
		this._updateShowHide();
	},
	_updateShowHide: function () {
		var show = this._popupOpen;
		if (!show && this.showOnHoverOver) {
			for (var i=0; i < this.showOnHoverOver.length; i++) {
				if (this.showOnHoverOver[i]._hovering) {
					show = true;
					break;
				}
			}
		}
		if (!show && this.showOnFocus) {
			for (var i=0; i < this.showOnFocus.length; i++) {
				if (this.showOnFocus[i]._focused) {
					show = true;
					break;
				}
			}
		}
		dojo.toggleClass(this.domNode, "hidden", !show);
	},
	
	destroy: function() {
		
		if (this.hoverPopup) {
			this.hoverPopup.destroy();
			this.hoverPopup = null;
		}
		
		this.inherited(arguments);
	}
	
});
})();