/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.PagedTableViewlet");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.util.html");
dojo.require("jazz.ui.PagedTable");

dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {
	
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

dojo.declare("com.ibm.team.dashboard.web.ui.PagedTableViewlet",
	Viewlet, {
	
	// summary:
	//		An abstract subclass of Viewlet that provides paging support.
	//
	//      NOTE: This is provisional API. Clients using this API do so with the
	//		awareness that this class may be removed or substantially changed at
	//		future times without warning.
	// description:
	//		Provides several functions that can be overriden to create a paged viewlet.
	//		The pages of the viewlet show tabular content and the consumer is reponsible
	//		for providing functions to fetch and render the data. In order for the viewlet
	//		to function, the methods that must be implemented are: fetch and renderEntry.
	//		If the viewlet is zoomable, then the renderDetails function must also be
	//		implemented.
	//
	//		The viewlet supports zoomable subclasses by allowing more detailed fetches to
	//		be performed when the user zooms in. Calls to the fetch function will provide
	//		flags to indicate if basic or detailed data should be returned. When zoomed,
	//		two rows will be rendered for each entry. The first will be generated by the
	//		renderEntry function and the second by the renderDetails function.
	//
	//		Since all table cells are generated by calls to functions implemented by the
	//		subclass, consumers are free to style and structure the table however they
	//		desire. This can be done by specifying attributes and css classes on the DOM
	//		nodes returned from the render functions.
	//
	//		Consumers overriding any Viewlet methods must call this.inherited(arguments)
	//		to ensure they do not override any PagedTableViewlet functionality.
	// since:
	//		0.7
	
	/* TO BE OVERRIDEN BY SUBCLASSES IF NEEDED */
	
	//		Used to customize the message that will be shown to the user when the viewlet is
	//		loading. The content will be applied as text, so html tags should not be used.
	loadingText: null,
	
	//		A string which will be applied as a css class to the content area of the PagedTable
	//		and the global heading of the viewlet.
	//
	//		NOTE: The DOM structure and CSS classes of the PagedTableViewlet are provisional
	//		and subject to change at future times without warning.
	contentClass: null,
	
	onDestroy: function() {
		// summary:
		//		Called in the viewlet's destroy method. Can be used to perform required cleanup.
	},
	
	/* int */ getItemsPerPage: function() {
		// summary:
		//		Will be called by the viewlet to get the number of items to show per page. The
		//		default implementation returns 10. This method can be overriden to perform
		//		some logic to determine the number of items per page, or simply to return a
		//		different value. A common practice is to declare a viewlet preference to allow
		//		the user to configure the number of items per page and to implement this
		//		method to simply return the value of the preference.
		return 10;
	},
	
	/* int */ getResultCount: function() {
		// summary:
		//		Will be called by the viewlet to get the number of results for the current query.
		//		If the viewlet has no valid query or target, 0 should be returned as if there
		//		were simply no results. This will trigger a call to the getNoResultsMessage
		//		method which allows the viewlet to be populated with a helpful message to guide
		//		the user about its configuration. If the viewlet is configured correctly, but the
		//		number of results is not known yet, -1 should be returned. This will trigger a
		//		a call to the fetch method.
		return 0;
	},
	
	/* Node */ getNoResultsMessage: function() {
		// summary:
		//		Called when getResultCount returns 0 to get a message for the user.
		// description:
		//		This method can be overriden to customize what is shown to the user when there
		//		are no results. It can also be used to guide the user if the viewlet is not
		//		configured correctly. The return type should be a dom node.
		return document.createTextNode(messages.pagedTableNoResults);
	},
	
	fetch: function (/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details) {
		// summary:
		//		This function will be called when more entries are required to render a given page.
		// description:
		//		Flags are passed to the function to indicate the type of content that is required.
		//		The viewlet should use these flags to determine what sort of requests to make to the
		//		server and to determine what to include in the result.
		//
		//		To allow for asynchronous requests, nothing is returned from this method. Instead
		//		the data should be provided by calling the provided ServiceResponseHandler's
		//		callback function and passing in the result object. This should be an object with
		//		one or two attributes. If the entries flag is specified it must have an a entries
		//		attribute and if the details flag is specified, it must have a details attribute.
		//		Each attribute on the result object should be an array of objects. The objects can
		//		take whatever form the viewlet likes and they will be passed to the renderEntry
		//		and renderDetails functions for rendering. Thus, they will only be consumed by the
		//		viewlet implementor, not by the base class. Each array in the result must be equal
		//		to or shorter than the count parameter in length. They should only be shorter in
		//		the case that the last page is being requested and there are not enough items left
		//		to fill them.
		// srh:
		//		The ServiceResponseHandler object to return the results to once they are ready.
		// startIndex:
		//		The index of the first item to fetch in the current query.
		// count:
		//		The number of items to fetch.
		// entries:
		//		A flag indicating that the entry data should be included. This is the data used to
		//		render the main row for each item.
		// details:
		//		A flag indicating that the detail data should be included. This is the data used to
		//		render the details for an item when the viewlet is zoomed. Will only ever be true
		//		for zoomable viewlets.
		console.error(messages.pagedTableFetchNotOverriden);
	},
	
	preRender: function(/* object[] */ entries) {
		// summary:
		//		Called immediately before rendering a page of data.
		// description:
		//		This method can be overriden by subclasses that need to perform some calculations
		//		that depend all the entries for a given page before it can render any of them
		//		individually.
		// entries:
		//		An array containing all the entries that are about to be rendered.
	},
	
	/* Node[] */ renderEntry: function(/* object */ entryData, /* object */ detailsData, /* boolean */ zoomed) {
		// summary:
		//		Called when an entry needs to be rendered into DOM nodes.
		// description:
		//		When the viewlet is rendering a page of data, it will call this function once per
		//		item that needs to be rendered. The function is responsible for reading the entry
		//		data that was originally generated by the fetch function and creating dom nodes
		//		out of it.
		//
		//		The return value must be an array of <td> DOM nodes. They will be used to populate the
		//		table row for the provided entry. Consumers may apply custom css styles and column
		//		spans to the returned cells.
		// entryData:
		//		The entry data object that was returned from a previous call to the fetch function.
		// detailsData:
		//		Will only be provided when the viewlet is zoomed. When zoomed, the value of this
		//		attribute is equal to the detail data returned from a previous call to the fetch
		//		funciton. The details correspond to the provided entryData. This is provided to allow
		//		consumers to change the rendering of the entry node when the viewlet is zoomed and
		//		to do so based on the information in the detailed data.
		console.error("renderEntry has not been overriden.");
	},
	
	/* Node[] */ renderDetails: function(/* object */ detailsData) {
		// summary:
		//		Called when the details for an entry need to be rendered (when the viewlet is zoomed).
		// description:
		//		When the viewlet is rendering a page of data while zoomed, it will call this function
		//		once per item that needs to be rendered. The function is responsible for reading the
		//		details data that was originally generated by the fetch function and creating dom nodes
		//		out of it.
		//
		//		The return value must be an array of <td> DOM nodes. They will be used to populate the
		//		details table row for the provided entry. Consumers may apply custom css styles and
		//		column spans to the returned cells.
		//
		//		The detail row for each item appear directly below their corresponding entry row in the
		//		table.
		// entries:
		//		The details data object that was returned from a previous call to the fetch function.
		return null;
	},
	
	onPageChange: function(/* number */ newPage) {
		// summary:
		//		Called when the user changes pages.
	},
	
	/* TO BE CALLED BY SUBCLASSES */
	
	updateTotalPageFooter: function() {
		this._updateVariables();
	},
	
	reset: function(/* boolean */ refresh) {
		// summary:
		//		Resets the viewlet to page 1.
		// description:
		//		Should be called when the number of items per page changes or when the viewlet's query has
		//		changed. Contains a call to updateTotalPageFooter, so consumers do not need to call both
		//		functions.
		// refresh:
		//		A boolean parameter that indicates if a full viewlet refresh is required. If this is set to
		//		true, all cached items will be invalidated and a fetch call will be made to retrieve page
		//		1. The loading message will also be shown until the fetch completes. If refresh is set to
		//		false, a call to update will be made.
		this._updateVariables();
		if (this.pagedTable)
			this.pagedTable.reset(false);
		// need to refresh viewlet, not just PagedTable, so that consumer's refresh method runs
		if (refresh) {
			this.getSite().refresh();
		}
	},
	
	setGlobalHeading: function(/* domNode */ node) {
		// summary:
		//		Places the provided DOM node at the top of the viewlet content area.
		// description:
		//		This can be used to provide a heading to the viewlet that will be visible regardless of
		//		which page is showing. It will still be visible if there are no results and will display
		//		above the no results message.
		// node:
		//		The DOM node to be used as the global heading.
		this.clearGlobalHeading();
		this._globalHeading.appendChild(node);
	},
	
	setCurrentPage: function(/* number */ page) {
		// summary:
		//		Sets the page of the viewlet.
		// page:
		//		The desired page number.
		if (this.pagedTable) {
			this.pagedTable.setCurrentPage(page);
			return true;
		}
		return false;
	},
	
	clearGlobalHeading: function() {
		// summary:
		//		Removes any previously created global heading.
		while (this._globalHeading.firstChild)
			this._globalHeading.removeChild(this._globalHeading.firstChild);
	},
	
	update: function() {
		// summary:
		//		Re-renders the current page of the viewlet.
		// description:
		//		Can trigger calls to fetch if the data for the current page is unavailable. Will trigger
		//		calls to renderEntry to perform the rendering. If the viewlet is zoomed, will also trigger
		//		calls to renderDetails.
		this._repaint(false);
	},
	
	/* INTERNAL FUNCTIONS */

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/PagedTableViewlet.html"),

	init: function() {
		this._previousIsZoomed = false;
		if (this.contentClass) {
			dojo.addClass(this._contentDiv, this.contentClass);
			dojo.addClass(this._messageDiv, this.contentClass);
		}
	},
	
	refresh: function() {
		if (!this.pagedTable) {
			this._oldResultCount = this.getResultCount();
			if (this._oldResultCount === 0) {
				this._renderNoResults();
			} else {
				this._oldZoomed = this.isZoomed();
				this._oldItemsPerPage = this.getItemsPerPage();
				var pagedTableArgs = {
					clearOnFlush: false,
					showDetails: this._oldZoomed,
					itemsPerPage: this._oldItemsPerPage,
					resultCount: this._oldResultCount,
					fetch: dojo.hitch(this,this._fetch),
					renderEntry: dojo.hitch(this,this.renderEntry),
					renderDetails: dojo.hitch(this,this.renderDetails),
					preRender: dojo.hitch(this,this.preRender),
					setLoading: dojo.hitch(this,this._setLoading),
					onPageChange: dojo.hitch(this,this.onPageChange)
				};
				this.pagedTable = new jazz.ui.PagedTable(pagedTableArgs, this.pagedTableNode);
			}
		} else {
			this._repaint(true);
		}
	},
	
	_repaint: function(flush) {
		var isZoomed = this.isZoomed();
		if (isZoomed != this._previousIsZoomed) {
			if (isZoomed) {
				this._onZoom();
			}
			else {
				this._onUnzoom();
			}
			this._previousIsZoomed = isZoomed;
		}
		this._updateVariables();
		this.pagedTable.refresh(flush);
	},
	
	_fetch: function(startIndex, count, entries, details) {
		var deferred = new dojo.Deferred();
		var srh = new ServiceResponseHandler(deferred, "callback", "errback");
		this.fetch(srh, startIndex, count, entries, details);
		deferred.addErrback(dojo.hitch(this,this._onFetchError));
		return deferred;
	},
	
	_updateVariables: function() {
		var newResultCount = this.getResultCount();
		if (newResultCount === 0) {
			this._renderNoResults();
		}
		if (this.pagedTable) {
			if (this._updateVariablesTimeout) {
				clearTimeout(this._updateVariablesTimeout);
				delete this._updateVariablesTimeout;
			}
			var newZoomed = this.isZoomed();
			if (newZoomed !== this._oldZoomed) {
				this.pagedTable.setShowDetails(newZoomed);
				this._oldZoomed = newZoomed;
			}
			var newItemsPerPage = this.getItemsPerPage();
			if (newItemsPerPage !== this._oldItemsPerPage) {
				this.pagedTable.setItemsPerPage(newItemsPerPage);
				this._oldItemsPerPage = newItemsPerPage;
			}
			if (newResultCount !== this._oldResultCount) {
				this.pagedTable.setResultCount(newResultCount);
				this._oldResultCount = newResultCount;
			}
		} else if (!this._updateVariablesTimeout) {
			// if the pagedTable is not finished instantiating, yield the call to ensure it fires.
			this._updateVariablesTimeout = setTimeout(dojo.hitch(this,this._updateVariables),0);
		}
	},
	
	_onZoom: function() {
		// don't auto-refresh while zoomed
		this.getSite().setAutoRefresh(false);
	},
	
	_onUnzoom: function() {
		// resume auto-refresh
		this.getSite().setAutoRefresh(true);
	},
	
	_onFetchError: function(error) {
		if (error && error.status == 403) {
			var span = document.createElement("span");
			span.className = "errorText";
			span.appendChild(document.createTextNode(messages.pagedTablePermissionDenied));
			this._showMessage(span);
		}
		else {
			this.getSite().error(error);
		}
		this.getSite().doneRefresh();
	},
	
	_renderNoResults: function() {
		this._showMessage(this.getNoResultsMessage());
	},

	_showMessage: function(/* Node */ node) {
		while (this._messageDiv.firstChild)
			this._messageDiv.removeChild(this._messageDiv.firstChild);
		this._messageDiv.appendChild(node);
		this._setMessageVisible(true);
	},
	
	_setLoading: function(/* boolean */ loading) {
		this.getSite().setLoading(loading);
		if (!loading && this.getResultCount() !== 0)
			this._setMessageVisible(false);
	},
	
	_setMessageVisible: function(visible) {
		if (visible) {
			dojo.removeClass(this._messageDiv, "hidden");
			dojo.addClass(this._contentDiv, "hidden");
		} else {
			dojo.addClass(this._messageDiv, "hidden");
			dojo.removeClass(this._contentDiv, "hidden");
		}
	},
	
	destroy: function() {
		if (this.pagedTable){
			this.pagedTable.destroy();
		}
		this.onDestroy();
		this.inherited(arguments);
	}
});

})();
