/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("net.jazz.ajax.ui.ResourceRegistry");

dojo.require("net.jazz.ajax.core.internal.Action");
dojo.require("jazz.core.RegistryFactory");

dojo.declare("net.jazz.ajax.ui.ResourceRegistry", null, {
	
	// summary:
	//		provides a registry of installed resource handlers for the current
	//		application
	// usage note:
	//		do not instantiate, obtain via:
	//		net.jazz.ajax.ui.PlatformUI.getWorkbench().getResourceRegistry()
	// since:
	//		0.5
	
	findModuleNameForPattern: function(/*String*/pattern) {
		var entry = this._pMap[pattern];
		return (entry && entry._rp) ? entry._rp : null;
	},
	
	findHandlersForPattern: function(/*String*/pattern) {
		// summary
		//		returns an array containing the 0-n resource handlers that match
		//		the specified pattern
		// since:
		//		0.6
		
		validateSyntax(pattern);
		
		var handlers = [];
		
		// if we have a matching pattern then we have an entry in _pMap; just
		// need to check if we have a registered action
		var entry = this._pMap[pattern]
		if (entry && entry.action) {
			handlers.push(entry.action); 
		}
		
		return handlers;
	},
	
	findPatternForUrl: function(/*String*/url) {
		// summary
		//		returns the closest matching url pattern given the specified URL
		//		or null if no URL matches the pattern. 
		// since:
		//		0.6
		try {
			validateSyntax(url);	
		} catch(e) {
			return false;
		}
		return this._pTree.findClosestMatchingPattern(url);
	},
	
	registerHandlerForPattern: function(/*String*/ pattern, /*Object|Function*/ arg1, /*String?*/ arg2) {
		// summary:
		//		register a resource handler for the pattern with this registry
		// description:
		//		this method has a two forms:
		//		- a two parameter version where the second argument is a
		//		  function
		//		- a three parameter version where the second argument is a
		//		  context object and the third argument is a string
		//		  corresponding to a method on the context object
		// id:
		//		the resource pattern declared in the extension registry;
		//		if not found in the extension registry, calling this method
		//		results in an error
		// arg1:
		//		either a context object or a function
		// arg2:
		//		if specified, a string corresponding to a method name attached to arg1
		// since:
		//		0.6
		
		// validate parameters
		if (arguments.length !== 2 && arguments.length !== 3) {
			var msg = "ResourceRegistry#registerResourcePatternHandler: Error - invalid number of parameters. Expected 2 or 3, saw: " + arguments.length;
			alert(msg);
			throw new Error(msg);
		}
		
		// make sure we have a URI
		validateSyntax(pattern);
		
		// if the pattern is not already defined, register it
		if (!this._pMap[pattern]){
			this._pMap[pattern] = {};
			this._pTree.createNodeForPattern(pattern);
		}
		
		this._pMap[pattern].action = createAction(arguments);
	},
	
	// 
	// Internal methods; do not call
	// 
	
	constructor: function() {
		// description:
		//		Initializes the resource handling registry for the workbench by loading
		//		both built-in action entries (e.g. jazz.viewPage) and action entries
		//		contributed by extensions.
		
		this._pTree = new patternTree();
		
		this._pMap = getAllRegisteredPatterns();
		
		for (var pattern in this._pMap) {
			this._pTree.createNodeForPattern(pattern);
		}
	}
	
});

// class aliases
var Action = net.jazz.ajax.core.internal.Action;
var RegistryFactory	= jazz.core.RegistryFactory;

// constants
var bpExtPointId = "net.jazz.ajax.pages";
var MSG_PREFIX = "ResourceRegistry#validateSyntax: ";



// utility function definitions
function createAction(args) {
	if(args.length === 2) {
		var action = new Action(args[0], args[1]);
	}
	if(args.length === 3) {
		action = new Action(args[0], args[1], args[2]);
	}
	return action;
}

function validateSyntax(uri) {
	if (!dojo.isString(uri)) {
		var errMsg = "Not a string (saw: " + uri + ")";
	} else if (uri.charAt(0) !== "/") {
		errMsg = "First character not a slash";
	}
	if (errMsg) {
		throw new Error(MSG_PREFIX + errMsg);
	}
}

function patternTree() {
	
	this.root = {};
	
	this.createNodeForPattern = function(pattern) {
	
		var segments = getSegments(pattern);
		var iterator = this.root;
		
		for (var i = 0; i < segments.length; i++) {
			var dir = segments[i];
			if (!iterator.subdirs) {
				iterator.subdirs = {};
			}
			if (!iterator.subdirs[dir]) {
				iterator.subdirs[dir] = {};
			}
			iterator = iterator.subdirs[dir];
		}
		
		iterator.pEnd = true;
		
	}
	
	this.findClosestMatchingPattern = function(url) {
		var segments = getSegments(url);
		var iterator = this.root;
		var matchPath = null;
		var totalPath = "";
		
		for (var i = 0; i < segments.length; i++) {
			var dir = segments[i];
			if (iterator.subdirs && iterator.subdirs[dir]) {
				iterator = iterator.subdirs[dir];
				totalPath += "/" + dir;
				if (iterator.pEnd) {
					matchPath = totalPath;
				}
			} else if (i === segments.length - 1 && iterator.subdirs && iterator.subdirs["/"]) {
				totalPath += "/";
				matchPath = totalPath;
			} else {
				break;
			}
		}
		
		return matchPath;
	}
}

function getSegments(uri) {
	var segments = uri.split("/");
	// remove the first element, which is always empty
	segments.shift();
	var lastElement = segments.length - 1;
	if (segments[lastElement] === "") { segments[lastElement] = "/"; }
	return segments;
}

function getAllRegisteredPatterns() {
	var bpExtPt = RegistryFactory.getRegistry().getExtensionPoint(bpExtPointId);
	var allPatternMap = {};
	dojo.forEach(bpExtPt.getExtensions(), function(extension) {
		dojo.forEach(extension.getConfigurationElements(), function(bpCfgElement) {
			if (bpCfgElement.getName() !== "page") { return; }
			dojo.forEach(bpCfgElement.getChildren(), function(actionCfgElement){
				if (actionCfgElement.getName() !== "resourcePattern") { return; }
				var pattern = actionCfgElement.getAttribute("pattern");
				try {
					validateSyntax(pattern);
				} catch (e) {
					console.error("Invalid pattern definition in extension registry; skipping (pattern: " + pattern + ")");
					return;
				}
				if (allPatternMap[pattern]) {
					console.warn("Multiple handlers for a resource not yet supported.")
					return;
				}
				allPatternMap[pattern] = {
					// _rp means 'requires page'
					_rp: bpCfgElement.getAttribute("id")
				}
			});
		});
	});
	return allPatternMap;
}

})();
