/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.Viewlet");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.web.ui.internal.LegacyViewletSite");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");

(function() {

var LegacyViewletSite = com.ibm.team.dashboard.web.ui.internal.LegacyViewletSite;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;

/*
 * An abstract viewlet class which all viewlets must subclass.
 * 
 * CAUTION: Must remain backward compatible for cross-version interoperability.
 * Dashboard framework must check for existence of new methods added after 0.7.
 * 
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.6
 */
dojo.declare("com.ibm.team.dashboard.web.ui.Viewlet",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Base class constructor; do not call or override in viewlets.
	 */
	constructor: function(args) {
		this._viewletContainer = args.container;
		this._dashboardBundle = args.dashboardBundle;
		this._definition = args.definition;
		this._helper = args.helper;
		this._site = args.site;
		if (this._site) {
			this._site.viewlet = this;
		}
		else {
			this._site = new LegacyViewletSite({
				container: args.container,
				dashboardItem: args.dashboardItem,
				definition: args.definition,
				viewlet: this,
				viewletHelper: this._helper
			});
		}
		this._webURL = args.webURL;
		this._zoomed = false;
		this._editing = false;
	},
	
	destroy: function() {
		this._isDestroyed = true;
		// abort any remaining requests
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		this.inherited(arguments);
	},

	/*
	 * Called by the framework after the viewlet is instantiated, but before the
	 * first refresh. Viewlets that need to initialize themselves shoudl override.
	 */
	init: function() {
	},
	
	/*
	 * Returns whether or not this viewlet has been destroyed.
	 */
	isDestroyed: function() {
		return this._isDestroyed ? true : false;
	},
	
	/*
	 * Returns the ViewletSite for this viewlet. The ViewletSite is used as a facade
	 * to the ViewletContainer allowing viewlets to perform various actions at the
	 * framework level.
	 */
	/* ViewletSite */ getSite: function() {
		return this._site;
	},
	
	/*
	 * Returns a memento that is personalized to the current user and will not
	 * be shared with other users. This can be used to store data that is specific
	 * to the user but should not be visible to other users.
	 * 
	 * This is available only when the user is the owner of the dashboard. To check
	 * whether or not it is available, call hasContributorState(). If unavailable,
	 * any data stored will the memento object will not be persisted.
	 */
	/* IMemento */ getContributorState: function() {
		// delegate to site
		return this._site.getContributorState();
	},
	
	/*
	 * Returns the viewlet's ServiceTracker, which subclasses should
	 * register every service call with. Any remaining requests will
	 * automatically be aborted on viewlet destroy.
	 */
	/* ServiceTracker */ getServiceTracker: function() {
		if (!this._serviceTracker) {
			this._serviceTracker = new ServiceTracker();
		}
		return this._serviceTracker;
	},
	
	/*
	 * Returns a memento for persisting viewlet state info that is the same for
	 * all users, e.g. viewlet content.
	 */
	/* IMemento */ getViewletState: function() {
		// delegate to site
		return this._site.getViewletState();
	},
	
	/*
	 * Returns an absolute URL to the Web context root for the Web UI of
	 * the viewlet's origin server, e.g. "https://server.domain.com:9443/jazz".
	 * This should be used to create links to other parts of the Web UI viewlets,
	 * as it works for both local and remotely hosted viewlets.
	 * 
	 * e.g.
	 * var adminUILinkHref = this.getWebURL() + "/admin";
	 * 
	 * @returns a URL to the viewlet's Web UI context root
	 * @since 0.8
	 */
	/* string */ getWebURL: function() {
		return this._webURL;
	},
	
	/*
	 * Returns whether or not this viewlet allows contributor state to be
	 * stored for the current user. Viewlets that enable extra UI if contributor
	 * state is available should check this first.
	 * 
	 * @returns whether or not contributor state is allowed
	 * @since 3.0
	 */
	/* boolean */ hasContributorState: function() {
		// delegate to site
		if (this._site.hasContributorState) {
			return this._site.hasContributorState();
		}
		else return true;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * Recieves notification when the setting areas is opened or closed.
 	 * 
 	 * @param boolean opened
 	 * @return <none>
 	 */		
	settingsAreaToggled: function(opened) {
		// NO-OP
		// Subclasses to override
	},	
	
	/*
	 * This function is always called before the Memento for the viewlet is saved.
	 * Viewlets that persist their state and need to know when the Memento is about
	 * to be updated (e.g. when save is pressed) should override this method.
	 */
	preSaveState: function() {
	},
	
	/*
 	 * Public Method
 	 * 
 	 * If the "Save" button is clicked in edit mode, this method is called to 
 	 * validate the edited content before leaving edit mode and saving the 
 	 * edited content.  
 	 * 
 	 * @param <none>
 	 * @return boolean
 	 * 		Returns "true" if the content is valid.  The viewlet leaves edit
 	 *      mode and saves the edited content.
 	 *      Otherwise, returns "false" if the content is not valid.  The 
 	 *      viewlet stays in edit mode and does not save the edited content.
 	 */		
	validateEditedContent: function() {
		return true;
	},		
	
	/*
	 * This function is used to generate the default state for a viewlet when
	 * one has not been previously saved. Viewlets should override to provide
	 * their default state object.
	 */
	/* object */ getDefaultState: function() {
		return new Object();
	},

	/*
	 * Returns the viewlet preference by the given id.
	 */
	/* string */ getPreference: function(/* string */ id) {
		// delegate to site
		return this._site.getPreference(id);
	},

	/*
	 * Sets the viewlet preference with the given id. Note that this will dirty the
	 * containing dashboard or will trigger an auto save, so it should only be used
	 * as a result of a user action.
	 */
	setPreference: function(/* string */ id, /* string */ value) {
		// delegate to site
		return this._site.setPreference(id, value);
	},
	
	/*
	 * Returns the viewlet's scope. If the viewlet is set to automatic scope,
	 * then the dashboard's scope is returned.
	 */
	/* string */ getScope: function() { 
		// delegate to site
		return this._site.getScope();
	},
	
	/*
	 * Returns the viewlet's scope item. If the viewlet's scope item is set
	 * to automatic, then a default will be assigned as follows:
	 * if the scope is contributor, the authenticated user will be used
	 * if the scope is project or team, the dashboard's scope item
	 * will be used.
	 */
	/* object */ getScopeItem: function() {
		// delegate to site
		return this._site.getScopeItem();
	},
	
	/*
	 * Returns the scope setting that indicates if subteams should be considered
	 * part of the viewlets scope. Only applicible when the scope is set to a team
	 * area and only to viewlets who define include-subteams as "yes" or "yes-and-check"
	 * in their definition.
	 */
	/* boolean */ getScopeSubteams: function() {
		// delegate to site
		return this._site.getScopeSubteams();
	},
	
	/*
	 * Returns the URL to navigate to when the viewlet title link is
	 * clicked.
	 * 
	 * Viewlet that have title-as-hyperlink set to true should override
	 * this method to return their specific URLs.
	 */
	/* string */ getTitleHyperlinkURL: function() {
		return null;
	},

	/*
 	 * Public Method
 	 * 
 	 * Refreshes the viewlet content.
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	refresh: function() {
		// NO-OP
		// Viewlet subclass to override
	},
	
	/*
	 * Repopulates the viewlet. New content should not be retrieved from the server.
	 * Called when presentation of the viewlet is changed in some way, but new information
	 * is not required.
	 * 
	 * Viewlets should override.
	 */
	update: function() {
	},
	
	/*
	 * Called by the platform when the zoomed state of the viewlet
	 * changes. This only applies to viewlets declared as zoomable.
	 * Viewlets should implement update() and call isZoomed().
	 */
	setZoomed: function(/* boolean */ isZoomed) {
		this._zoomed = isZoomed;
		this.update();
	},
	
	/*
	 * Indicates whether or not this viewlet is zoomed. Should be called
	 * by zoomable viewlets when updating.
	 */
	/* boolean */ isZoomed: function() {
		return this._zoomed;
	},

	/*
 	 * Public Method
 	 * 
 	 * Called just before a viewlet zoom operation is invoked
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	preZoom: function() {
		// NO-OP
		// Viewlet subclass to override
	},

	/*
 	 * Public Method
 	 * 
 	 * Called right after a viewlet zoom operation is completed
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	postZoom: function() {
		// NO-OP
		// Viewlet subclass to override
	},
	
	/*
	 * To be overridden by viewlets that need to update their contents
	 * when dashboard enters edit mode.
	 */
	setDashboardEditing: function(editing) {

	},
	
	/*
	 * Called by the platform to inform the viewlet whether or not it is active.
	 * A viewlet is active only when it is visible in the user's browser meaning
	 * the dashboard is the current workbench page and the viewlet is on the currently
	 * displayed dashboard tab.
	 */
	setActive: function(/* boolean */ active) {
	},
	
	/*
	 * Called by the platform when the viewlet enters or leaves edit mode.
	 * This only applies to viewlets declared as editable. Viewlets should
	 * implement update() and call isEditing
	 */
	setEditing: function(/* boolean */ isEditing) {
		this._editing = isEditing;
		this.update();
	},
	
	/*
	 * Indicates whether or not this viewlet is in edit mode. Should be called
	 * by editable viewlets when updating.
	 */
	/* boolean */ isEditing: function() {
		return this._editing;
	},
	
	/*
	 * Called by the platform to determine if the zoom button should be created.
	 * Applicable to zoomable viewlets only. This method is used to determine if
	 * the current content of the viewlet requires zooming. Default implementation
	 * always returns true.
	 * 
	 * May be overriden by zoomable viewlets.
	 */
	/* boolean */ hasZoomableContent: function() {
		return true;
	},
	
	/*
	 * Called by the platform when the list for a dynamic preference is opened.
	 * This method should be overriden by any viewlet that defines dynamic preferences.
	 * The definition id of the preference will be passed in. The result should
	 * be:
	 * 
	 * 1) an object containing a property named 'options'. This property will
	 * contain an array of objects each of which contains a 'label' and a 'value'.
	 * (e.g. "{options: [{label: 'First Option', value: 'option1'}, {label: 'Second
	 * Option', value: 'option2'}]}" ).
	 * 
	 * 2) an object containing a property named 'groups'. This property will
	 * contain an array of objects each of which contains a 'name' and an 'options' array
	 * as described in (1).
	 * 
	 * 3) null to indicate that the preferences are not ready yet. This will display
	 * a message indicating that the preferences are being loaded and will trigger a
	 * polling loop to keep calling this function until preference options are returned.
	 * 
	 * In cases (1) and (2) above, a 'computedDefault' property can be added to the object.
	 * This will cause the settings area to prime the combobox accordingly.
	 * (e.g. "{options: [{label: 'First Option', value: 'option1'}, {label: 'Second
	 * Option', value: 'option2'}], computedDefault: 'option2'}" ).
	 * 
	 * Since the preference options may need to be computed based on dirty (unsaved)
	 * settings, the second parameter will provide a working copy of the viewlet's
	 * settings. It will contain 2 to 5 properties:
	 * 
	 * title: the display title of the viewlet.
	 * scope (where applicable): an integer representing the scope of the viewlet; if the viewlet is auto
	 *     scoped, this value will be resolved to the automatic scope, so it can be used
	 *     unconditionally. Only applicable to scopable viewlets.
	 * scopeItemId (where applicable): a string representing the itemId of the current scopeItem; if the viewlet
	 *     is set to auto scope item, this value will be resolved to the automatic scope,
	 *     so it can be used unconditionally. Only applicable to scopable viewlets.
	 * scopeSubteams (where applicable): a boolean indicating whether or not subteam scoping
	 *     is enabled. Only applicable when team scope is selected and the viewlet declares
	 *     itself as sensative to this option.
	 * preferences: an associative array (by preference id) of the viewlet's preferences
	 * 
	 * Note, any preferences which are in the process of loading will not be included in
	 * currentSettings.preferences. If a preference depends on another preference that is
	 * still loading it should return null to indicate that it is loading as well.
	 */
	getPreferenceOptions: function(/* string */ preferenceId, /* object */ currentSettings) {
	},
	
	/*
	 * Called by the framework to get an implementation of CustomPreferenceProvider to be used
	 * for the specified custom preference.
	 */
	getPreferenceProvider: function(/* string */ preferenceId) {
		return null;
	},
	
	/*
	 * Called by the platform immediately after the viewlet's settings are saved. This method
	 * should be overriden by any viewlet that does not wish to refresh on all setting changes
	 * or by viewlets that would like to do something more on setting changes.
	 * 
	 * The newSettings parameter contains the new settings values that are about to be applied
	 * and the oldSettings parameter contains the settings values that existed before the save.
	 * It is up to the viewlet to compare these values and react accordingly. The parameters
	 * will each be an object with 2 to 5 properties as follows:
	 * 
	 * title: the display title of the viewlet.
	 * scope (where applicable): an integer representing the scope of the viewlet; if the viewlet is auto
	 *     scoped, this value will be resolved to the automatic scope, so it can be used
	 *     unconditionally. Only applicable to scopable viewlets.
	 * scopeItemId (where applicable): a string representing the itemId of the current scopeItem; if the viewlet
	 *     is set to auto scope item, this value will be resolved to the automatic scope,
	 *     so it can be used unconditionally.  Only applicable to scopable viewlets.
	 * scopeSubteams (where applicable): a boolean indicating whether or not subteam scoping
	 *     is enabled. Only applicable when team scope is selected and the viewlet declares
	 *     itself as sensative to this option.
	 * preferences: an associative array (by preference id) of the viewlet's preferences
	 */
	settingsChanged: function(/* object */ oldSettings, /* object */ newSettings) {
		this._site.refresh();
	},
	
	/*
	 * Called by the platform immediately after the viewlet's settings changes are cancelled. This
	 * method should be overriden by any viewlet that needs to back out live updates made while
	 * the user changed settings in the settings area.
	 * 
	 * The canceledSettings parameter contains the state of the settings at the time the changes were
	 * canceled and the restoredSettings contains the state of the viewlet settings as they were
	 * before the settings area was opened. This is the state the viewlet will be reverted to.
	 * Each object will be structured identically to the parameters of the settingsChanged method.
	 */
	settingsCanceled: function(/* object */ canceledSettings, /* object */ restoredSettings) {
	},
	
	/*
	 * Called by the platform when any setting is changed inside the open settings area.
	 * This function should return an array of strings representing the preference ids of
	 * computed list or custom preferences that depends on indicated setting. For computed
	 * preferences this will cause getPreferenceOptions to be called again to allow the list
	 * to be recomputed. For custom preferences, it will revert the preference value to the 
	 * default specified in the viewlet definition.
	 * 
	 * The settingId parameter will give the id of the modified setting.For a viewlet
	 * title change the settingId will be "title", for a scope change it will be "scope"
	 * and for a scopeItem change it will be "scopeItem". Note that a scope change will
	 * cause the scopeItem to change as well, but only one call will be made to
	 * settingChanging and the parameter will be "scope".
	 * 
	 * The workingCopy parameter will give the current state of the dirty settings. This
	 * allows viewlets to use this callback to react to the dirty setting changes immediately.
	 * If this is done, the settingsCanceled method should be overriden to ensure any invalid
	 * changes are properly reverted. The format of the workingCopy parameter is the same as
	 * both parameters to the settingsChanged method.
	 */
	/* string[] */ settingChanging: function(/* string */ settingId, /* object */ workingCopy) {
		return new Array();
	}
});

})();
